/**@file

@brief This library contains unicos CPC constants

@author Enrique Blanco (EN/ICE)
@author Alexey Merezhin (EN/ICE)

@copyright
        &copy;Copyright CERN 2013 - all rights reserved

@par Creation Date
  14/11/2011

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI, CTRL

@par Modification History

    13/09/2012: Alexey
        [UCPC-931] add constants for the alarms menu item

    16/05/2012: Alexey
        [UCPC-787] introduce AlSt bit and letter constants

    14/05/2012: Alexey
        [UCPC-803] unify ConfigW bit

    27/03/2012: Alexey
        [UCPC-733] added FuStopISt bit position for PCO

    13/03/2012: Alexey
        [UCPC-688] added set parameter constants

    20/01/2012: Alexey
        [UCPC-599] added MFC constants

    12/12/2011: Alexey
        [UCPC-621] modified alarm constants

    23/11/2011: Alexey
        removed custom AA/DA mask event constants
        cloned right menu constants

    25/10/2011: Alexey
        [UCPC-587] all letter constants has been ported

    20/10/2011: Alexey
        [UCPC-585] added CPC_STSREG01_CONTROLLER_ARMRCPST

    21/09/2011: Alexey
        [UCPC-555] added CPC_CONFIG_ALARM_LEVEL

    20/09/2011: Alexey
        [CPC_PARAMS_HLDRIVE] added CPC_PARAMS_HLDRIVE

    13/09/2011: Alexey
        [UCPC-531] added CPC_STSREG02_ANALOGONST/CPC_STSREG02_ANALOGOFFST
        [UCPC-511] fixed AA inhibited thresholds constants
        [UCPC-532] added CPC_STSREG02_OUTOFFOVST and CPC_PARAMS_OUTOFF

    07/09/2011: Alexey
        [UCPC-430] added CPC_STSREG02_OUTONOVST

    25/08/2011: Alexey
        [UCPC-511] removed UN_CPC_MANREG01_MNEWTHRESR/UN_CPC_MANREG01_MEDHH/UN_CPC_MANREG01_MEDH/UN_CPC_MANREG01_MEDL/UN_CPC_MANREG01_MEDLL

    24/08/2011: Alexey
        [UCPC-466] added CPC_PARAMS_ALDT constant for alarm's delay time parameter

    11/08/2011: Alexey
          [UCPC-476] added CPC_WIDGET_TEXT_CONFIGW/CPC_WIDGET_TEXT_CONFIGW/CPC_WIDGET_TEXT_POSHHW/CPC_WIDGET_TEXT_POSLLW/CPC_WIDGET_TEXT_POSW

    10/08/2011: Alexey
      [UCPC-460] added CPC_POPUPMENU_MASK_IST_TEXT

    04/08/2011: Alexey
      [UCPC-456] added CPC_PARAMS_OUTMAIN and CPC_PARAMS_PWMMODE

    28/07/2011: Alexey
      [UCPC-436] added sms message storage's key CPC_CONFIG_SMS_MESSAGE

    15/07/2011: Alexey
      [UCPC-411] removed UN_STSREG02_PLDCMDST

    13/07/2011: Alexey
      [UCPC-392] changed CPC_MANREG02_MNEWPOSR and added CPC_MANREG01_MPSAVE/CPC_MANREG01_MPREST

    08/07/2011: Alexey
      [UCPC-384] removed "parameters" field from the meta-info block

    01/07/2011: Alexey
      added param constants
      added AA enable/disable manreg constants
      [UCPC-374] added PRstartFS constant

    2011/06/30 Alexey
      [UCPC-362] added manreg constants to set AA limits

    2011/06/28 Alexey
      [UCPC-311] added CPC_STSREG02_SOFTLDST
      added UN_WIDGET_TEXT_CONTROL to follow cpcGenericObject_WidgetControlStateAnimation convention
      [UCPC-345] added CPC_STSREG01_SOFTLDST

    2011/06/23 Alexey
      [UCPC-323] added CPC_STSREG02_NEEDRESTART and CPC_WIDGET_TEXT_MANUAL_RESTART

    2011/06/22 Alexey
      [UCPC-323] added CPC_MANREG01_MRESTART

    2011/06/20 Alexey
      [UCPC-316] added CPC_MANREG01_MRSETTSASFS and CPC_MANREG01_MDALACT
      [UCPC-320] removed UN_MANREG02_ARMRCP

    2011/05/31 Alexey
      [UCPC-298] added CPC_CONFIG_CONTROLLER_SCALING_METHOD_KEY constant

    2011/05/30 Alexey
      [UCPC-292] removed UN_CPC_CONFIG length constants

    2011/05/25 Alexey
      added CPC_WIDGET_TEXT_WARNING_ALARM_BLOCKED

    2011/05/19 amerezhi
      removed UN_STSREG01_AUINHSAVRES as it exists in unicoObjects_declarations

    2011/05/17 amerezhi
      added unCPC6 StsReg and ManReg bit constants
      added interlock status constants for widget
      added IOErrorBlocking popup constant and SetAALimit and OutputPositioning faceplate constants

    2011/05/10 amerezhi
      [UCPC-268] added second ALIAS to metainfo
      added key constant for metainfo

    2011/05/09 amerezhi
      [UCPC-235] added CPC length constants and meta info position constants

    2011/05/09 amerezhi
      [UCPC-195][UCPC-196] added StsReg constants for Alarm objects

    Revision 1.22  2011/01/13 14:00:10  amerezhi
      Merged with gcsObjects_declarations.ctl
    
    Revision 1.22  2009/03/20 14:00:10  gthomas
      -Updated variable names <_GCSDELAYEDALARM_CLASS> with <_ALERT_CLASS_>
    
    Revision 1.21  2008/09/04 14:54:31  gthomas
      -moved back to version 1.15  and added "MFCSetParameters" constant

    Revision 1.20  2008/06/23 08:57:31  gthomas
      added variable const string CPC_CONFIG_EMPTY_DESCRIPTION_STRING = "-";

    Revision 1.19  2008/06/23 08:29:10  gthomas
      Kept only variables for xStatus and xParameters

    Revision 1.18  2008/06/19 12:24:25  gthomas
      uncommented 2 variables used in xParameters lib and pnl

    Revision 1.17  2008/06/13 15:16:38  aabadz
      CPC_MENU_ALARM_MASK_REPLY commented because is already declared in another file.

    Revision 1.16  2008/06/09 14:54:27  aabadz
      Result after move some constants, only use use inside this file, from gcsObjects_declarations.

    Revision 1.15  2007/10/30 16:22:09  gthomas
      Introduced the CVS keyword $Log to track changes made in the library

*/

const string CPC_UNDEFINED_CONSTANT_VALUE = "undefined"; //!< Undefined value
const unsigned CPC_METAINFO_LENGTH      = 5; //!< Lenght of meta-info in configuration db
const unsigned CPC_CONFIG_MASTER_NAME	  = 1; //!< Position of master in the meta-info block in config db
const unsigned CPC_CONFIG_PARENTS 	   	= 2; //!< Position of parents in the meta-info block in config db
const unsigned CPC_CONFIG_CHILDREN 		  = 3; //!< Position of children in the meta-info block in config db
const unsigned CPC_CONFIG_TYPE			    = 4; //!< Position of type in the meta-info block in config db
const unsigned CPC_CONFIG_SECOND_ALIAS	= 5; //!< Position of second alias in the meta-info block in config db
const string CPC_CONFIG_MASTER_NAME_KEY	 = "MASTER_NAME";  //!< Key for master in parameters
const string CPC_CONFIG_PARENTS_KEY 		 = "PARENTS";      //!< Key for parents in parameters
const string CPC_CONFIG_CHILDREN_KEY 	   = "CHILDREN";     //!< Key for children in parameters
const string CPC_CONFIG_TYPE_KEY			   = "TYPE";         //!< Key for type in parameters
const string CPC_CONFIG_SECOND_ALIAS_KEY = "SECOND_ALIAS"; //!< Key for second alias in parameters
const string CPC_CONFIG_SMS_MESSAGE		   = "SMS_MESSAGE";  //!< Key for sms message in parameters
const string CPC_CONFIG_ALARM_LEVEL      = "ALARM_LEVEL";  //!< Key for alarm level in parameters

const string UN_CONFIG_CPC_CONTROLLER_SCALING_METHOD_KEY = "SCALING_METHOD"; //!< Parameter constant
const string CPC_CONFIG_MV_DEVICE        = "MV";  //!< Key for controller mv parameter
const string CPC_CONFIG_OUT_DEVICES      = "OUT";  //!< Key for controller out parameter


//---------
// WIDGETS
//---------
const string CPC_WIDGET_TEXT_CONTROL_AUIHFOMOST = "h";          //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_CONTROL_AUINHFMO = "h";            //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_CONTROL_AUINHMMO = "h";            //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_MANUAL_RESTART = "R";              //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_ALARM_STARTIST = "I";              //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_ALARM_TSTOPIST = "S";              //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_ALARM_FUSTOPIST = "F";             //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_ALARM_ALST = "A";                  //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_BLOCK_TFUSTOP = "T";               //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_WARNING_ALARM_BLOCKED = "B";       //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_CONTROL_MMOST = "M";               //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_CONTROL_LDST = "HL";               //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_CONTROL_FORCED = "F";              //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_CONTROL_MANUAL = "M";              //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_CONTROL_SOFTLDST = "L";            //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_CONTROL_FOMOST = "F";              //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_CONTROL_REMOST = "R";              //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_CONTROL_TST = "T";                 //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_CONFIGW = "C";                     //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_POSHHW = "W";                      //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_POSHW = "W";                      //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_POSLW = "W";                      //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_POSLLW = "W";                      //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_POSW = "W";                        //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_WARNING_DEFAULT = "W";             //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_WARNING_SIMU = "S";                //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_WARNING_ERROR = "E";               //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_OLD_DATA = "O";                    //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_INVALID = "N";                     //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_ALARM_MASKED = "M";                //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_WARNING_BLOCKED = "B";             //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_ALARM_BLOCKED = "B";               //!< Letter constant for widget animation
const string CPC_WIDGET_TEXT_ALARM_DOES_NOT_EXIST = "";         //!< Letter constant for widget animation

// StsReg01 constants
const unsigned CPC_STSREG01_AST =					     0;		       //!< Alarm Status AA (for PosAl DPE; un_POSAL = 9)
const unsigned CPC_STSREG01_SOFTLDST =		     0;		       //!< Controller software local mode
const unsigned CPC_STSREG01_WST =					     1;		       //!< Warning Status AnalogAlarm
const unsigned CPC_STSREG01_AuMoSt =			     2;		       //!< Auto Mode
const unsigned CPC_STSREG01_MaMoSt =			     3;		       //!< Manual Mode
const unsigned CPC_STSREG01_FoMoSt =			     4;		       //!< Forced mode
const unsigned CPC_STSREG01_MONRST = 			     4;		       //!< StsReg bit constant
const unsigned CPC_STSREG01_MOFFRST = 		     5;		       //!< StsReg bit constant
const unsigned CPC_STSREG01_AUEAIST = 		     8;          //!< StsReg bit constant
const unsigned CPC_STSREG01_PREMASKALARM =	   8;	         //!< Premask alarm for DA
const unsigned CPC_STSREG01_REGST = 			     9;		       //!< StsReg bit constant
const unsigned CPC_STSREG01_POSW = 			 	     9;		       //!< StsReg bit constant
const unsigned CPC_STSREG01_MIOERBRST = 	     9;  	       //!< I/O Error blocked
const unsigned CPC_STSREG01_FUSTOPIST =        9;		       //!< Full stop interlock
const unsigned CPC_STSREG01_TRST = 			      10;		       //!< StsReg bit constant
const unsigned CPC_STSREG01_POSALST = 		    10;		       //!< StsReg bit constant
const unsigned CPC_STSREG01_TSTOPIST = 		    11;		       //!< Temporary Stop Interlock status
const unsigned CPC_STSREG01_OUTPST = 		      11;			     //!< StsReg bit constant
const unsigned CPC_STSREG01_AUIHFOMOST = 	    13;		       //!< Forced mode inhibited
const unsigned CPC_STSREG01_AUIHMBST = 		    15;          //!< StsReg bit constant
const unsigned CPC_STSREG01_LLST =			       8;			     //!< ANALOG ALARM -LL
const unsigned CPC_STSREG01_LST=			 	       9;			     //!< ANALOG ALARM -L
const unsigned CPC_STSREG01_HST =			        10;				   //!< ANALOG ALARM -H
const unsigned CPC_STSREG01_HHST =			      11;			     //!< ANALOG ALARM -HH

const unsigned CPC_STSREG01_MAMSKST =	      	15;			     //!< STSREG info alarm setMask
const unsigned CPC_STSREG01_PFSPOSONST_MFC =   5;	         //!< Auto inhibit manual mode (MFC)
const unsigned CPC_STSREG01_POSOV =			      11;			     //!< StsReg bit constant

const unsigned CPC_MANREG01_AMSKST =			    11;		       //!< AlarmMaskOn (Off is when is not bit 11) AA, DA. Output
const unsigned CPC_MANREG01_AMSKRST =		      12;			     //!< AalarmMaskOff (Reset alarm AA, DA) Output.

// AnalogAlarm constants
const unsigned CPC_STSREG01_IST		=               0;       //!< StsReg bit constant
const unsigned CPC_STSREG01_CONFIGW =             5;       //!< StsReg bit constant
const unsigned CPC_STSREG01_ARMRCPST =            3;       //!< StsReg bit constant
const unsigned CPC_STSREG01_PIDARMRCPST         = 5;       //!< StsReg bit constant
const unsigned CPC_STSREG01_ACTHHST	=             4;       //!< StsReg bit constant
const unsigned CPC_STSREG01_ACTLLST	=             5;       //!< StsReg bit constant
const unsigned CPC_STSREG01_POSHHW	=             8;       //!< StsReg bit constant
const unsigned CPC_STSREG01_POSHW	=               9;       //!< StsReg bit constant
const unsigned CPC_STSREG01_POSLW	=              10;       //!< StsReg bit constant
const unsigned CPC_STSREG01_POSLLW	=            11;       //!< StsReg bit constant
const unsigned CPC_STSREG01_MALBRST	=            14;       //!< StsReg bit constant
const unsigned CPC_STSREG01_ALST	=              14;       //!< StsReg bit constant
const unsigned CPC_STSREG01_AUIHMB =             15;       //!< StsReg bit constant

const unsigned CPC_STSREG02_EHHST	  =             0;				//!< StsReg bit constant
const unsigned CPC_STSREG02_EHST		=             1;				//!< StsReg bit constant
const unsigned CPC_STSREG02_ELST		=             2;				//!< StsReg bit constant
const unsigned CPC_STSREG02_ELLST	  =             3;				//!< StsReg bit constant
const unsigned CPC_STSREG02_HHALST	=             4;				//!< StsReg bit constant
const unsigned CPC_STSREG02_HWST		=             5;				//!< StsReg bit constant
const unsigned CPC_STSREG02_LWST		=             6;				//!< StsReg bit constant
const unsigned CPC_STSREG02_LLALST	=             7;				//!< StsReg bit constant
const unsigned CPC_STSREG02_FUSTOPIST =          10;				//!< Full  Stop Interlock status
const unsigned CPC_STSREG02_NEEDRESTART =        11;        //!< StsReg bit constant
const unsigned CPC_STSREG02_SOFTLDST =           12;        //!< StsReg bit constant
const unsigned CPC_STSREG02_IHMHHST	=             8;				//!< StsReg bit constant
const unsigned CPC_STSREG02_IHMHST	=             9;				//!< StsReg bit constant
const unsigned CPC_STSREG02_IHMLST	=            10;				//!< StsReg bit constant
const unsigned CPC_STSREG02_IHMLLST	=            11;				//!< StsReg bit constant
const unsigned CPC_STSREG02_POUTMAIN =           15;        //!< StsReg bit constant
const unsigned CPC_STSREG02_OUTONOVST    =        0;        //!< StsReg bit constant
const unsigned CPC_STSREG02_ANALOGONST   =        8;        //!< StsReg bit constant
const unsigned CPC_STSREG02_ANALOGOFFST  =        9;        //!< StsReg bit constant
const unsigned CPC_STSREG02_OUTOFFOVST   =       14;        //!< StsReg bit constant



// ManReg01 constants --- seems we not use any more
const unsigned CPC_MANREG02_MNEWPOSR		  =       3;				//!< Manual Position request trigger bit in ManReg01
const unsigned CPC_MANREG01_MRESTART			=       9;				//!< ManReg bit constant
const unsigned CPC_MANREG01_MIOERBSETRST 	=      10;				//!< ManReg bit constant
const unsigned CPC_MANREG01_MALBSETRST 		=      10;				//!< ManReg bit constant
const unsigned CPC_MANREG01_MRSETTSASFS	  =       8;        //!< Manual reset temporary as full stop bit position in Manreg01^M
const unsigned CPC_MANREG01_MDALACT		    =      10;        //!< Manual deblock alarm action bit position in ManReg01^M
const unsigned CPC_MANREG01_MNEWHHR		    =       8;				//!< ManReg bit constant
const unsigned CPC_MANREG01_MNEWHR		    =       9;				//!< ManReg bit constant
const unsigned CPC_MANREG01_MNEWLR		    =      12;				//!< ManReg bit constant
const unsigned CPC_MANREG01_MNEWLLR		    =      13;				//!< ManReg bit constant

const unsigned CPC_MANREG02_MREGR 			  =       0;				//!< ManReg bit constant
const unsigned CPC_MANREG02_MOUTPR		  	=       1;				//!< ManReg bit constant
const unsigned CPC_MANREG01_MPSAVE        =       5;			  //!< Request restoring PID parameters
const unsigned CPC_MANREG01_MPREST        =       6;			  //!< Request SAVE PID parameters (version PID without addressing)

// Access rights constants
int CPC_ACCESS_RIGHTS_OPERATOR  = 1;  //!< Access constant constant
int CPC_ACCESS_RIGHTS_EXPERT    = 2;  //!< Access constant constant
int CPC_ACCESS_RIGHTS_ADMIN     = 3;  //!< Access constant constant
int CPC_ACCESS_RIGHTS_NOONE     = 4;  //!< Access constant constant - noone has an access

const string CPC_FACEPLATE_MISSED_VALUE = "missed"; //!< Used as missed value in dynamic callbacks


// Button constants
const string CPC_FACEPLATE_BUTTON_OUTPUTPOSITIONING_MODE = "OutputPositioning";     //!< Output positioning button

const string CPC_FACEPLATE_BUTTON_PARAMETER_SET_VALUE	=		"Parameter_SetValue"; 			//!< Set Value for AnalogParameter and WordParameter
const string CPC_FACEPLATE_BUTTON_DIGITALPARAMETER_SET_VALUE	=	"DigitalParameter_SetValue"; 	//!< Set Value DigitalParameter
const string CPC_FACEPLATE_BUTTON_PUMP_SET_FREQUENCY   = 		"SetFrecuency";					//!< Set Frecuency Pump

const string CPC_FACEPLATE_BUTTON_AA_SUBPANEL_SET_ALARM   = "SetAlarm";			//!< Test subpanel Analog Alarm
const string CPC_FACEPLATE_BUTTON_AA_SUBPANEL_RESET_ALARM   = "ResetAlarm";		//!< Test subpanel Analog Alarm
const string CPC_FACEPLATE_BUTTON_AA_SUBPANEL_SET_WARNING   = "SetWarning";		//!< Test subpanel Analog Alarm
const string CPC_FACEPLATE_BUTTON_AA_SUBPANEL_RESET_WARNING   = "ResetWarning";	//!< Test subpanel Analog Alarm
const string CPC_FACEPLATE_BUTTON_SET_AA_LIMITS = "SetAALimits";		//!< Setting limits for AA
const string CPC_FACEPLATE_BUTTON_SET_DA_MAIL_SMS = "SetDAMailSMS";		//!< Setting limits for DA

const string CPC_FACEPLATE_BUTTON_MFC_PARAMS ="MFCSetParameters"; //!< MFC Set Parameters action
const string CPC_FACEPLATE_BUTTON_ALLOW_RESTART = "AllowRestart"; //!< Allow restart action
const string CPC_FACEPLATE_BUTTON_SAVE_VALUE = "SaveValue";       //!< Save value action


//	AA and DA RBMenu
const unsigned CPC_MENU_ALARM_MASK_REPLY		 = 55;			//!< Menu Choice result for Digital Alarm and Analog Alarm
const unsigned CPC_MENU_RECIPES_REPLY		 = 56;			//!< Menu Choice result for Recipes from PCO
const unsigned CPC_MENU_CURRNTPARAMS_REPLY	 = 57;			//!< Menu Choice result for Current Parameters from PCO
const string CPC_POPUPMENU_CURRENTPARAMS 	 = "Current Parameters";	//!< Menu Item for calling Current Parameters from PCO
const string CPC_POPUPMENU_MANUAL_IO_ERROR_BLOCK = "IOErrorBlocking"; //!< IO Error blocking menu item's label

const string CPC_POPUPMENU_RECIPES_TEXT 	  = "Recipes";	//!< Menu Item for calling Recipes from PCO
const string CPC_POPUPMENU_DIAG_INFO_TEXT   = "Info_Diagnostic";	//!<	Unicos "info-diagnostic" menu-text
const string CPC_POPUPMENU_MASK_IST_TEXT    = "MaskISt";   //!< Mask ISt menu item's label
const string CPC_POPUPMENU_MASK_ALST_TEXT   = "MaskAlSt"; //!< Mask ISt menu item's label
const string CPC_POPUPMENU_ALARMS_TEXT      = "Alarms";      //!< Alarms menu item's label

const string CPC_POPUPMENU_HIERARCHY = "HierarchyPopup"; //!< Hierarchu menu item's label


const int CPC_POPUPMENU_ACK_NUMBER               = 2;    //!< Popup item's number
const int CPC_POPUPMENU_MASK_POSST_NUMBER        = 3;    //!< Popup item's number
const int CPC_POPUPMENU_BLOCK_PLC_NUMBER         = 5;    //!< Popup item's number
const int CPC_POPUPMENU_FACEPLATE_NUMBER         = 6;    //!< Popup item's number
const int CPC_POPUPMENU_DIAGNOSTIC_NUMBER        = 7;    //!< Popup item's number
const int CPC_POPUPMENU_INFO_NUMBER              = 8;    //!< Popup item's number
const int CPC_POPUPMENU_DEVICE_CONFIG_NUMBER     = 10;   //!< Popup item's number
const int CPC_POPUPMENU_UNMASKEVENT_NUMBER       = 11;   //!< Popup item's number
const int CPC_POPUPMENU_MASKEVENT_NUMBER         = 12;   //!< Popup item's number
const int CPC_POPUPMENU_OPENSYNOPTIC_NUMBER      = 15;   //!< Popup item's number
const int CPC_POPUPMENU_MASK_ALST_NUMBER         = 16;   //!< Popup item's number
const int CPC_POPUPMENU_MANUAL_IO_ERROR_NUMBER   = 17;   //!< Popup item's number
const int CPC_POPUPMENU_MASK_IST_NUMBER          = 18;   //!< Popup item's number
const int CPC_POPUPMENU_HIERARCHY_NUMBER         = 19;   //!< Popup item's number
const int CPC_POPUPMENU_DEVICELINK_NUMBER        = 20;   //!< Popup item's number
const int CPC_POPUPMENU_ALARMS_NUMBER            = 301;  //!< Popup item's number - UN_POPUPMENU_MIN + 1
const int CPC_POPUPMENU_RECIPES                  = 302; // cpc


// MFC constants
const unsigned CPC_MANREG01_MSPR        = 6;					//!< Manual Set Point Request trigger
const unsigned CPC_MANREG01_MDRMODE     = 12;					//!< Manual Drive Mode Request Trigger 
const unsigned CPC_MANREG01_MTOTMRT     = 13;					//!< Manual Totalizer Mode Request Trigger 
const unsigned CPC_MANREG01_MCCRT       = 14;					//!< Manual Calibration Curve Mode Request Trigger 

// Config constants
const string CPC_CONFIG_EMPTY_DESCRIPTION_STRING = "-";							//!< A dummy string to be put in description when no actual string is valid
const string CPC_CONFIG_DEFAULT_CALIBRATION_CURVE = "1=default";	  //!< Default string when there is not string for the calibration curve

// Alarm
const string CPC_WIDGET_ALARM_HIGH	  	= "H";						//!< High alarm AnalogAlarm
const string CPC_WIDGET_ALARM_HIGHHIGH	= "HH";						//!< High High alarm AnalogAlarm
const string CPC_WIDGET_ALARM_LOW		    = "L";						//!< Low alarm AnalogAlarm
const string CPC_WIDGET_ALARM_LOWLOW	  = "LL";						//!< Low Low alarm - for AnalogAlarm
const string CPC_WIDGET_ALARM			      = "A";						//!< Alarm for Digital Alarm

// Mail-SMS alert class constants
const string CPC_CONFIG_ALARM_BASE_CLASS   = "_cpcAnalog"; //!< Base alarm class @see cpcConfigGenericFunctions_getAlertClass
const string CPC_CONFIG_ALARM_ACK_POSTFIX  = "_Ack";       //!< Acknowledge postfix for alarm class @see cpcConfigGenericFunctions_getAlertClass
const string CPC_CONFIG_ALARM_MAIL_POSTFIX = "_Mail";      //!< Mail postfix for alarm class @see cpcConfigGenericFunctions_getAlertClass
const string CPC_CONFIG_ALARM_CUSTOM_OK_POSTFIX = " down"; //!< Used as postfix on OK message for custom alarm

const int CPC_INT16 	= 1;  //!< Virtual datatype @see cpcConfigGenericFunctions_translateDataType
const int CPC_INT32 	= 2;  //!< Virtual datatype @see cpcConfigGenericFunctions_translateDataType
const int CPC_UINT16	= 3;  //!< Virtual datatype @see cpcConfigGenericFunctions_translateDataType
const int CPC_BOOL 	= 4;    //!< Virtual datatype @see cpcConfigGenericFunctions_translateDataType
const int CPC_FLOAT 	= 5;  //!< Virtual datatype @see cpcConfigGenericFunctions_translateDataType

const string CPC_PARAMS_FSPOSON = "PFSPosOn";                     //!< Parameter constant
const string CPC_PARAMS_HFONST  = "PHFOnSt";                      //!< Parameter constant
const string CPC_PARAMS_HFOFFST = "PHFOffSt";                     //!< Parameter constant
const string CPC_PARAMS_PULSEST = "PPulseSt";                     //!< Parameter constant
const string CPC_PARAMS_HFON    = "PHFOn";                        //!< Parameter constant
const string CPC_PARAMS_HFOFF   = "PHFOff";                       //!< Parameter constant
const string CPC_PARAMS_MRESTART= "PEnRstart";                    //!< Parameter constant
const string CPC_PARAMS_RSTARTFS= "PRstartFS";                    //!< Parameter constant
const string CPC_PARAMS_OUTOFF  = "POutOff";                      //!< Parameter constant
const string CPC_PARAMS_OUTMAIN = "POutMain";                     //!< Parameter constant
const string CPC_PARAMS_PWMMODE = "PPWMMode";                     //!< Parameter constant
const string CPC_PARAMS_ALDT    = "PAlDt";                        //!< Parameter constant
const string CPC_PARAMS_HLDRIVE = "PHLDrive";                     //!< Parameter constant
const string CPC_PARAMS_NOISE_FILTER = "PNoiseFilter";            //!< Parameter constant
const string CPC_PARAMS_FEEDBACK_SIM = "PFeedbackSim";            //!< Parameter constant
const string CPC_PARAMS_INHIBITMANUALTOTALIZER = "PIhMTot";       //!< Parameter constant
const string CPC_MFC_CALIBRATION_UNIT = "CalibrationFlowUnit";    //!< Parameter constant
const string CPC_MFC_CALIBRATION_GAS_NAME = "CalibrationGasName"; //!< Parameter constant

const string CPC_CMW_ADDRESS_CONFIG_SEPARATOR = "|"; //!< CMW address separator constant
//TODO: use another approach - IS-1154
const string CPC_CMW_DPTYPE = "CMW"; //!< CMW dp type; cloned from CPC_CMW_DPTYPE

