/**@file

// cpcAnalogStatus.ctl
This library contains the widget, faceplate, etc. functions of AnalogStatus.

@par Creation Date
  dd/mm/yyyy

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  the author (DEP-GROUP)

@copyright
        &copy;Copyright CERN 2013 - all rights reserved
*/
//@{


/** Function called from snapshot utility of the treeDeviceOverview to get the time and value
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name
@param deviceType input, device type
@param dsReturnData output, return data, array of 5 strings
*/
CPC_AnalogStatus_ObjectListGetValueTime(string deviceName, string deviceType, dyn_string &dsReturnData)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
  float value;
  string sTime;
	bool bBad;

  dpGet(deviceName+".ProcessInput.PosSt", value,
        deviceName+".ProcessInput.PosSt:_online.._stime", sTime,
        deviceName+".ProcessInput.PosSt:_online.._bad", bBad);

  dsReturnData[1] = sTime;
  dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(deviceName + ".ProcessInput.PosSt"), value) + " " + dpGetUnit(deviceName + ".ProcessInput.PosSt"); // formated value with unit
	dsReturnData[3] = bBad;
	dsReturnData[4] = "OK";
	dsReturnData[5] = "unEventList_ForeColor";
//end_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
}

/** pop-up menu
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param dsAccessOk input, the access control 
@param menuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
*/
CPC_AnalogStatus_MenuConfiguration(string deviceName, string dpType, dyn_string dsAccessOk, dyn_string &menuList)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
	dyn_string dsMenuConfig = makeDynString(CPC_POPUPMENU_DIAG_INFO_TEXT, UN_POPUPMENU_TREND_TEXT);

	cpcGenericObject_addDefaultUnicosActionToMenu(deviceName, dpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addTrendActionToMenu(deviceName, dpType, dsMenuConfig, dsAccessOk, menuList);
//end_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
}

/** handle the answer of the popup menu
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param menuList input, the access control 
@param menuAnswer input, selected menu value 
*/
CPC_AnalogStatus_HandleMenu(string deviceName, string dpType, dyn_string menuList, int menuAnswer)
{
  dyn_string exceptionInfo;

//begin_TAG_SCRIPT_DEVICE_TYPE_HandleMenu
	cpcGenericObject_HandleUnicosMenu(deviceName, dpType, menuList, menuAnswer, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_HandleMenu

  if (dynlen(exceptionInfo) > 0) {
      unSendMessage_toExpertException("Right Click",exceptionInfo);
  }
}

/**  widget register callback

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
CPC_AnalogStatus_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
  string deviceName, systemName, frontEnd;
  int iAction, iRes;
  dyn_string exceptionInfo;
  bool bRemote;

  deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
  if (deviceName == "") deviceName = g_sDpName; // In case of disconnection
  systemName = unGenericDpFunctions_getSystemName(deviceName);
  frontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);
  if (frontEnd != "") frontEnd = systemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+frontEnd;

  unDistributedControl_isRemote(bRemote, systemName);
  g_bSystemConnected = bRemote ? bSystemConnected : true;
  g_bUnSystemAlarmPlc = frontEnd != "" && dpExists(frontEnd);
  if (deviceName != "") {
    if (dpTypeName(deviceName) != UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME) g_bUnSystemAlarmPlc = false;
  }

  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  dyn_string lockDPEs;
  switch (iAction) {
    case UN_ACTION_DISCONNECT:
      CPC_AnalogStatus_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_DPCONNECT:
      g_sDpName = deviceName;
      if (g_bUnSystemAlarmPlc) {
        lockDPEs = cpcGenericAnimation_WidgetLockDPEs(deviceName);
        if (dynlen(lockDPEs) > 0) {
          iRes = dpConnect("cpcGenericAnimation_WidgetLockCallBack", lockDPEs);
          g_bCallbackConnected = (iRes >= 0);
        } else {
          cpcGenericAnimation_WidgetConnect(deviceName, frontEnd);
        }
      } else {
        CPC_AnalogStatus_WidgetDisconnection(g_sWidgetType);
      }
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      lockDPEs = cpcGenericAnimation_WidgetLockDPEs(deviceName);
      if (dynlen(lockDPEs) > 0) {
        if (g_bUnSystemAlarmPlc) {
          iRes = dpDisconnect("cpcGenericAnimation_WidgetLockCallBack", lockDPEs);
        }
        if (g_bDisconnectCB) {
          cpcGenericAnimation_WidgetDisconnect(deviceName, frontEnd);
        }
        g_bCallbackConnected = !(iRes >= 0);
        g_bDisconnectCB = false;
      } else {
        cpcGenericAnimation_WidgetDisconnect(deviceName, frontEnd);
      }
      CPC_AnalogStatus_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_NOTHING:
      break;
    default:
      break;
  }
}

/** Init static values which are used in widget animation
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_AnalogStatus_WidgetInitStatics(string deviceName)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
  g_sPosStFormat = dpGetFormat(deviceName + ".ProcessInput.PosSt");
  g_sPosStUnit = dpGetUnit(deviceName + ".ProcessInput.PosSt");
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
}

/** Returns the list of AnalogStatus DPEs which should be connected on widget animation
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_AnalogStatus_WidgetDPEs(string deviceName, dyn_string &dpes)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
  dynRemove(dpes, dynlen(dpes)-1);
  dynAppend(dpes, deviceName + ".ProcessInput.PosSt");
  dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_online.._invalid");
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
}

/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names 
@param values input, the dpe values

*/
CPC_AnalogStatus_WidgetAnimation(dyn_string dpes, dyn_anytype values, string widgetType)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation
  int iSystemIntegrityAlarmValue = values[1];
  bool bSystemIntegrityAlarmEnabled = values[2];
  bool bLocked = values[3];
  string sSelectedManager = values[4];
  bool isCommentsActive = values[5];

  float fPosSt = values[6];
  bool bPosStInvalid = values[7];

	string sBody1Text, sBody1Color;
	string sWarningLetter, sWarningColor;

	sBody1Text = cpcGenericObject_FormatValueWithUnit(fPosSt, g_sPosStFormat, g_sPosStUnit);
  sBody1Color = bPosStInvalid ? "unDataNotValid" : "cpcColor_Widget_Status";

  cpcGenericObject_WidgetValidnessAnimation(bSystemIntegrityAlarmEnabled, iSystemIntegrityAlarmValue,
                                            bPosStInvalid,
                                            sWarningLetter, sWarningColor);

  if(g_bSystemConnected)
    setMultiValue("Body1", "text", sBody1Text,"Body1", "foreCol", sBody1Color,
                  "WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor,
                  "WidgetArea", "visible", true,
                  "commentsButton", "visible", isCommentsActive);
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation
}

/** Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in OnOff faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.

*/
CPC_AnalogStatus_WidgetDisconnection(string sWidgetType)
{
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
  setMultiValue("Body1", "foreCol", "unDataNoAccess","Body1", "backCol", "unWidget_Background",
                "WarningText", "text", "", "WidgetArea", "visible", false);
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
}

//@}