/**@file

// cpcAnalogStatusConfig.ctl
This library contains the import and export function of the CPC_AnalogStatus.

@par Creation Date
  dd/mm/yyyy

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  the author (DEP-GROUP)

@copyright
        &copy;Copyright CERN 2013 - all rights reserved
*/
//@{



// Constants
const string UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME = "CPC_AnalogStatus";
//begin_TAG_SCRIPT_DEVICE_TYPE_constants
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_LENGTH                        = 9;

const unsigned UN_CONFIG_CPC_ANALOGSTATUS_UNIT                          = 1;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_FORMAT                        = 2;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_DEADBAND                      = 3;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_DEADBAND_TYPE                 = 4;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_RANGEMAX                      = 5;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_RANGEMIN                      = 6;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ARCHIVE_ACTIVE                = 7;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ARCHIVE_TIME_FILTER           = 8;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ADDRESS_POSST                 = 9;

const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ADDITIONAL_LENGTH          = 6;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ADDITIONAL_PARAMETERS      = 1;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ADDITIONAL_MASTER_NAME     = 2;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ADDITIONAL_PARENTS         = 3;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ADDITIONAL_CHILDREN        = 4;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ADDITIONAL_TYPE            = 5;
const unsigned UN_CONFIG_CPC_ANALOGSTATUS_ADDITIONAL_SECOND_ALIAS    = 6;

// SOFT_FE constants
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGSTATUS_UNIT                 = 1;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGSTATUS_FORMAT               = 2;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGSTATUS_RANGEMAX             = 3;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGSTATUS_RANGEMIN             = 4;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGSTATUS_ARCHIVE_ACTIVE       = 5;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGSTATUS_ARCHIVE_TIME_FILTER  = 6;
const unsigned UN_CONFIG_SOFT_FE_CPC_ANALOGSTATUS_LENGTH               = 6;

// CMW constants
const unsigned UN_CONFIG_CMW_CPC_ANALOGSTATUS_UNIT                          = 1;
const unsigned UN_CONFIG_CMW_CPC_ANALOGSTATUS_FORMAT                        = 2;
const unsigned UN_CONFIG_CMW_CPC_ANALOGSTATUS_DEADBAND                      = 3;
const unsigned UN_CONFIG_CMW_CPC_ANALOGSTATUS_DEADBAND_TYPE                 = 4;
const unsigned UN_CONFIG_CMW_CPC_ANALOGSTATUS_RANGEMAX                      = 5;
const unsigned UN_CONFIG_CMW_CPC_ANALOGSTATUS_RANGEMIN                      = 6;
const unsigned UN_CONFIG_CMW_CPC_ANALOGSTATUS_ARCHIVE_ACTIVE                = 7;
const unsigned UN_CONFIG_CMW_CPC_ANALOGSTATUS_ARCHIVE_TIME_FILTER           = 8;
const unsigned UN_CONFIG_CMW_CPC_ANALOGSTATUS_ADDRESS_POSST                 = 9;
const unsigned UN_CONFIG_CMW_CPC_ANALOGSTATUS_LENGTH                        = 9;
//end_TAG_SCRIPT_DEVICE_TYPE_constants


/**
DPE configuration
TODO: use fw info instead
*/
mapping CPC_AnalogStatusConfig_getConfig() {
  mapping config;
  mapping props;

//begin_TAG_SCRIPT_DEVICE_TYPE_deviceConfig
	mappingClear(props);
	props["hasUnit"] 		= true;
	props["hasFormat"]		= true;
	props["hasSmooth"] 		= true;
	props["hasArchive"] 	= true;
	props["hasPvRange"] 	= true;
	props["address"] 		= ".ProcessInput.PosSt";
	props["dataType"] 		= CPC_FLOAT;
	config["PosSt"] 		= props;
//end_TAG_SCRIPT_DEVICE_TYPE_deviceConfig

  return config;
}

/**
Check data before configuration
  @param[in]  configLine   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_AnalogStatusConfig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH) {
    // 1. Common
    unConfigGenericFunctions_checkParameters(configLine, UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, exceptionInfo);
    // 2. call the device check function
    unConfigGenericFunctions_checkDeviceConfig(configLine, UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, exceptionInfo);
  } else {
    fwException_raise(exceptionInfo,"ERROR","CPC_AnalogStatusConfig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
  }
}

/**
Check the CPC_AnalogStatus device configuration for the SOFT_FE front-end
  @param dsConfigs input, config line as dyn_string
  @param exceptionInfo output, for errors
*/
SOFT_FE_CPC_AnalogStatus_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, configLine, exceptionInfo);
}

/**
Check the CPC_AnalogStatus device configuration for the CMW front-end
  @param dsConfigs input, config line as dyn_string
  @param exceptionInfo output, for errors
*/
CMW_CPC_AnalogStatus_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, configLine, exceptionInfo);
}

/**
Check the CPC_AnalogStatus device configuration for the S7_PLC front-end
  @param dsConfigs input, config line as dyn_string
  @param exceptionInfo output, for errors
*/
S7_PLC_CPC_AnalogStatus_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, configLine, exceptionInfo);
}

/**
Check the CPC_AnalogStatus device configuration for the _UnPlc front-end
  @param dsConfigs input, config line as dyn_string
  @param exceptionInfo output, for errors
*/
_UnPlc_CPC_AnalogStatus_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, configLine, exceptionInfo);
}

/**
Creates dp (or checks if it is already presented)and sets configuration for CPC_AnalogStatus object.
  
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_AnalogStatusConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName;

  // 1. Initialization : checking + dpCreate
  unConfigGenericFunctions_checkAll(dsConfigs, UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, exceptionInfo);
  if (dynlen(exceptionInfo) <= 0) { // Good inputs 
    sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
    unConfigGenericFunctions_createDp(sDpName, UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, exceptionInfo);
    if (dynlen(exceptionInfo) <= 0) { // Creation ok or dp already exists
      // 2. Generate device
      // 2.1. Parameters
      unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
      // 2.2 call the device check function
      unConfigGenericFunctions_setDeviceConfig(dsConfigs, UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME, exceptionInfo);
    }
  }
}

/**
Set the CPC_AnalogStatus device configuration for the SOFT_FE front-end
  @param dsConfigs input, config line as dyn_string
  @param exceptionInfo output, for errors
*/
SOFT_FE_CPC_AnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

/**
Set the CPC_AnalogStatus device configuration for the CMW front-end
  @param dsConfigs input, config line as dyn_string
  @param exceptionInfo output, for errors
*/
CMW_CPC_AnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

/**
Set the CPC_AnalogStatus device configuration for the S7_PLC front-end
  @param dsConfigs input, config line as dyn_string
  @param exceptionInfo output, for errors
*/
S7_PLC_CPC_AnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

/**
Set the CPC_AnalogStatus device configuration for the _UnPlc front-end
  @param dsConfigs input, config line as dyn_string
  @param exceptionInfo output, for errors
*/
_UnPlc_CPC_AnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

/**
Purpose: Export CPC_AnalogStatus Devices

Usage: External function

PVSS manager usage: NG, NV
*/
CPC_AnalogStatusConfig_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    string sObject = UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME;
    int amount = dynlen(dsDpList);
    for(int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if(!dpExists(currentDP)) continue;

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
//begin_TAG_SCRIPT_DEVICE_TYPE_export
        cpcExportGenericFunctions_getUnit(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getFormat(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getDeadband(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getRange(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "PosSt", true, dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getParameters(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_export

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}

/**
Purpose: Export CPC_AnalogStatus Devices for S7_PLC front-end

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_CPC_AnalogStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    CPC_AnalogStatusConfig_ExportConfig(dsDpList, exceptionInfo);
}

/**
Purpose: Export CPC_AnalogStatus Devices for _UnPlc front-end

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_CPC_AnalogStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    CPC_AnalogStatusConfig_ExportConfig(dsDpList, exceptionInfo);
}

/*
  Use this place to define any unexisting constants or functions than you need for the device implementation.
*/
//begin_TAG_SCRIPT_DEVICE_TYPE_CustomConfigFunctions
void SOFT_FE_CPC_AnalogStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    string sObject = UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME;
    int amount = dynlen(dsDpList);
    for(int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if(!dpExists(currentDP)) continue;

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
        cpcExportGenericFunctions_getUnit(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getFormat(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getRange(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString(), dsDpParameters);

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}
void CMW_CPC_AnalogStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    string sObject = UN_CONFIG_CPC_ANALOGSTATUS_DPT_NAME;
    int amount = dynlen(dsDpList);
    for(int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if(!dpExists(currentDP)) continue;

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
        cpcExportGenericFunctions_getUnit(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getFormat(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getDeadband(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getRange(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
        cpcExportGenericFunctions_processCMWAddress(currentDP, sObject, "PosSt", true, dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString(), dsDpParameters);

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}
//end_TAG_SCRIPT_DEVICE_TYPE_CustomConfigFunctions


//@}