/**@file

// cpcLocalConfig.ctl
This library contains the import and export function of the CPC_Local.

@par Creation Date
  dd/mm/yyyy

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  the author (DEP-GROUP)

@copyright
        &copy;Copyright CERN 2013 - all rights reserved
*/
//@{



// Constants
const string UN_CONFIG_CPC_LOCAL_DPT_NAME = "CPC_Local";
//begin_TAG_SCRIPT_DEVICE_TYPE_constants
const unsigned UN_CONFIG_CPC_LOCAL_LENGTH = 8;

const unsigned UN_CONFIG_CPC_LOCAL_ARCHIVE_ACTIVE		= 1;
const unsigned UN_CONFIG_CPC_LOCAL_SMS_CATEGORIES		= 2;
const unsigned UN_CONFIG_CPC_LOCAL_SMS_MESSAGE			= 3;
const unsigned UN_CONFIG_CPC_LOCAL_ACK_ALARM			= 4;
const unsigned UN_CONFIG_CPC_LOCAL_NORMAL_POSITION		= 5;
const unsigned UN_CONFIG_CPC_LOCAL_ADDRESS_STSREG01		= 6;
const unsigned UN_CONFIG_CPC_LOCAL_ADDRESS_EVSTSREG01	= 7;
const unsigned UN_CONFIG_CPC_LOCAL_ADDRESS_MANREG01		= 8;

const unsigned UN_CONFIG_CPC_LOCAL_ADDITIONAL_LENGTH          = 7;
const unsigned UN_CONFIG_CPC_LOCAL_ADDITIONAL_MASKEVENT       = 1;
const unsigned UN_CONFIG_CPC_LOCAL_ADDITIONAL_PARAMETERS      = 2;
const unsigned UN_CONFIG_CPC_LOCAL_ADDITIONAL_MASTER_NAME     = 3;
const unsigned UN_CONFIG_CPC_LOCAL_ADDITIONAL_PARENTS         = 4;
const unsigned UN_CONFIG_CPC_LOCAL_ADDITIONAL_CHILDREN        = 5;
const unsigned UN_CONFIG_CPC_LOCAL_ADDITIONAL_TYPE            = 6;
const unsigned UN_CONFIG_CPC_LOCAL_ADDITIONAL_SECOND_ALIAS    = 7;
//end_TAG_SCRIPT_DEVICE_TYPE_constants


/**
DPE configuration
TODO: use fw info instead
*/
mapping CPC_LocalConfig_getConfig() {
  mapping config;
  mapping props;

//begin_TAG_SCRIPT_DEVICE_TYPE_deviceConfig
	mappingClear(props);
	props["address"] 		= ".ProcessInput.StsReg01";
	props["dataType"] 		= CPC_UINT16;
	config["StsReg01"] 		= props;

	mappingClear(props);
	props["hasArchive"] 	= true;
	props["address"] 		= ".ProcessInput.evStsReg01";
	props["dataType"] 		= CPC_INT32;
	config["evStsReg01"] 	= props;

	mappingClear(props);
	props["hasAcknowledge"] = true;
	props["hasArchive"] 	= true;
	props["isAlarm"]		= true;
	props["address"] 		= ".ProcessInput.PosAl";
	props["dpe"]			= "StsReg01";
	props["bitPosition"]	= CPC_STSREG01_POSALST;
	config["PosAl"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessOutput.ManReg01";
	props["dataType"] 		= CPC_UINT16;
	config["ManReg01"] 		= props;
//end_TAG_SCRIPT_DEVICE_TYPE_deviceConfig

  return config;
}

/**
Returns the list of parameter names that passed via config line.
*/
dyn_string CPC_LocalConfig_getParamNames() {
//begin_TAG_SCRIPT_DEVICE_TYPE_getParamNames
	return makeDynString(CPC_PARAMS_HFON, CPC_PARAMS_HFOFF);
//end_TAG_SCRIPT_DEVICE_TYPE_getParamNames
}

/**
Check data before configuration
  @param[in]  configLine   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_LocalConfig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH) {
    // 1. Common
    unConfigGenericFunctions_checkParameters(configLine, UN_CONFIG_CPC_LOCAL_DPT_NAME, exceptionInfo);
    // 2. call the device check function
    unConfigGenericFunctions_checkDeviceConfig(configLine, UN_CONFIG_CPC_LOCAL_DPT_NAME, exceptionInfo);
  } else {
    fwException_raise(exceptionInfo,"ERROR","CPC_LocalConfig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
  }
}

/**
Check the CPC_Local device configuration for the S7_PLC front-end
  @param dsConfigs input, config line as dyn_string
  @param exceptionInfo output, for errors
*/
S7_PLC_CPC_Local_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_LOCAL_DPT_NAME, configLine, exceptionInfo);
}

/**
Check the CPC_Local device configuration for the _UnPlc front-end
  @param dsConfigs input, config line as dyn_string
  @param exceptionInfo output, for errors
*/
_UnPlc_CPC_Local_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_LOCAL_DPT_NAME, configLine, exceptionInfo);
}

/**
Creates dp (or checks if it is already presented)and sets configuration for CPC_Local object.
  
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_LocalConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName;

  // 1. Initialization : checking + dpCreate
  unConfigGenericFunctions_checkAll(dsConfigs, UN_CONFIG_CPC_LOCAL_DPT_NAME, exceptionInfo);
  if (dynlen(exceptionInfo) <= 0) { // Good inputs 
    sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
    unConfigGenericFunctions_createDp(sDpName, UN_CONFIG_CPC_LOCAL_DPT_NAME, exceptionInfo);
    if (dynlen(exceptionInfo) <= 0) { // Creation ok or dp already exists
      // 2. Generate device
      // 2.1. Parameters
      unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
      // 2.2 call the device check function
      unConfigGenericFunctions_setDeviceConfig(dsConfigs, UN_CONFIG_CPC_LOCAL_DPT_NAME, exceptionInfo);
    }
  }
}

/**
Set the CPC_Local device configuration for the S7_PLC front-end
  @param dsConfigs input, config line as dyn_string
  @param exceptionInfo output, for errors
*/
S7_PLC_CPC_Local_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

/**
Set the CPC_Local device configuration for the _UnPlc front-end
  @param dsConfigs input, config line as dyn_string
  @param exceptionInfo output, for errors
*/
_UnPlc_CPC_Local_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

/**
Purpose: Export CPC_Local Devices

Usage: External function

PVSS manager usage: NG, NV
*/
CPC_LocalConfig_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    string sObject = UN_CONFIG_CPC_LOCAL_DPT_NAME;
    int amount = dynlen(dsDpList);
    for(int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if(!dpExists(currentDP)) continue;

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
//begin_TAG_SCRIPT_DEVICE_TYPE_export
        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters, ".ProcessInput.PosAl");
        dynRemove(dsDpParameters, dynlen(dsDpParameters));
        cpcExportGenericFunctions_getCategories(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getKeyDeviceConfiguration(currentDP, dsDpParameters, CPC_CONFIG_SMS_MESSAGE, exceptionInfo);
        cpcExportGenericFunctions_getAcknowledgeAlarm(currentDP, ".ProcessInput.PosAl", dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getDigitalNormalPosition(currentDP, sObject, makeDynString(".ProcessInput.PosAl"), dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg01", 	false, dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString("ProcessInput.PosAl"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01"), dsDpParameters);
        cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getParameters(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_export

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}

/**
Purpose: Export CPC_Local Devices for S7_PLC front-end

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_CPC_Local_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    CPC_LocalConfig_ExportConfig(dsDpList, exceptionInfo);
}

/**
Purpose: Export CPC_Local Devices for _UnPlc front-end

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_CPC_Local_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
    CPC_LocalConfig_ExportConfig(dsDpList, exceptionInfo);
}

//@}