/*
@author: Alexey Merezhin

Creation Date: 04/02/2013
	
version 1.0

External Function : 
	. unHVACUserlib_UnicosInfosCV
	
Purpose: 
This library contains the user functions to create Configuration file for the LHCLogging API in CV projects

Usage: Public

PVSS manager usage: NV

Constraints:
	. PVSS version: 3.1
	. operating system: wds XP
	. distributed system: No

@copyright
        &copy;Copyright CERN 2013 - all rights reserved
*/

//-----------------------------------------------------------------------------------------------------------

unHVACUserlib_UnicosInfosCV(string sDpe, string &sName,  string &sDescription, string &sHierarchy, string &sAlias, string &sE)
{
dyn_string dsParameters, dsSplit;
string deviceName, sSystemName, sLocation, sApplication, sDomaine, sNature;
bool bState,bUseDpe, bOK;
int iNum, iLen;

    sSystemName=getSystemName();
    sHierarchy = "ERROR";
    sE = sDpe;

    //remove system name
    unCoreLHCServices_set_SystemName(sDpe, bState, false);			

    //Domain / nature
    deviceName = unGenericDpFunctions_getDpName(sDpe);			
    unGenericDpFunctions_getParameters(deviceName, dsParameters);		
    
    //Location
    dynClear(dsSplit);													
    dsSplit=strsplit(sSystemName,"_");
    if (dynlen(dsSplit)>1)
	    sLocation=dsSplit[1];
    else
	    sLocation=substr(sSystemName, 0, strlen(sSystemName)-1);

    deviceName = unGenericDpFunctions_getDpName(sDpe);			
    dsSplit = strsplit(sDpe,".");
    if (dynlen(dsSplit) == 3) { //UNICOS Device
	// if (dsSplit[2] == "ProcessInput" && dsSplit[3] == "PosSt") { // only ProcessInput.PosSt DPEs

	string sDevType = dpTypeName(sDpe);
	// if (sDevType == "CPC_AnalogInput" || sDevType == "CPC_AnalogOutput" || sDevType == "CPC_DigitalInput" || sDevType == "CPC_DigitalOutput" || sDevType == "CPC_WordStatus" || sDevType == "CPC_AnalogStatus" || sDevType == "CPC_AnalogParameter") {

	sAlias=dpGetAlias(dsSplit[1] + "."); // TODO: check me
	sName = sAlias + "." + dsSplit[3];
	sE = substr(sDpe,strpos(sDpe,".")+1, strlen(sDpe));
	sDescription=unGenericDpFunctions_getDescription(deviceName);
        strreplace(sDescription, "$", "_");
        strreplace(sDescription, "?", "_");

	dynClear(dsSplit);													
	dsSplit=strsplit(sSystemName,"_");
	if (dynlen(dsSplit)>1)
	    sLocation=dsSplit[1];
	else
	    sLocation=substr(sSystemName, 0, strlen(sSystemName)-1);

	//Application 
	dynClear(dsSplit);
	dsSplit=strsplit(sDpe,"-");
	if (dynlen(dsSplit)>3)
		sApplication=dsSplit[3];
	else
		sApplication="UNKNOWN";

	sHierarchy = unHVACUserlib_Hierarchy(sApplication, sDevType, sDpe);
    } else { // process front-end
      int s7pos = strpos(dsSplit[1], S7_PLC_DPTYPE);
      if (s7pos >= 0) {
	  unUserlib_SystemAlarmInfo(dsSplit, sLocation, sDpe, sDescription, sAlias, sName, sHierarchy);
	  string plcName = substr(dsSplit[1], s7pos);
	  dpGet(sSystemName + plcName + ".configuration.subApplications", sApplication);
	  sHierarchy = unHVACUserlib_Hierarchy(sApplication, "WCCOA_INTEGRITY", sDpe);
      } else {
	  DebugN("Can't process dpe: " + sDpe);
      }
    }
}

/** Function builds a hierarchy for CV project.
 * The hierarchy format: Location/SubLocation/Application/DeviceType
*/
string unHVACUserlib_Hierarchy(string application, string deviceType, string sDpe)
{
  string hierarchy = "CV/";
  if (patternMatch("*L4_*", application)) {
    hierarchy += "MEYRIN/LINAC4/";
  } else if (patternMatch("*RFQ4*", application)) {
    hierarchy += "MEYRIN/LINAC4/";
  } else if (patternMatch("*_AD_*", application)) {
    hierarchy += "MEYRIN/AD/";
  } else if (patternMatch("*513*", application)) {
    hierarchy += "MEYRIN/513/";
  } else if (patternMatch("*378*", application)) {
    hierarchy += "MEYRIN/378/";
  } else if (patternMatch("*ISOLDE*", application)) {
    hierarchy += "MEYRIN/ISOLDE/";
  } else if (patternMatch("*HVAC_SPS*", application)) {
    hierarchy += "SPS/TUNNEL/";
  } else if (patternMatch("*COOL_SPS*", application)) {
    hierarchy += "SPS/TUNNEL/";
  } else if (patternMatch("*PS*", application)) {
    hierarchy += "MEYRIN/PS/";
  } else if (patternMatch("*107*", application)) {
    hierarchy += "MEYRIN/HVAC/";
  } else if (patternMatch("*NA62*", application)) {
    hierarchy += "PREVESSIN/NA62/";
  } else if (patternMatch("*NA58*", application)) {
    hierarchy += "PREVESSIN/NA58/";
  } else if (patternMatch("*AMS*", application)) {
    hierarchy += "PREVESSIN/HVAC/";
  } else if (patternMatch("*867*", application)) {
    hierarchy += "PREVESSIN/COOLING/";
  } else if (patternMatch("*BE91*", application)) {
    hierarchy += "PREVESSIN/HVAC/";
  } else if (patternMatch("*CCR*", application)) {
    hierarchy += "PREVESSIN/HVAC/";
  } else if (patternMatch("*BA8*", application) || patternMatch("*ba8*", application)) {
    hierarchy += "SPS/North_Area/";
  } else if (patternMatch("*HIRADMAT*", application)) {
    hierarchy += "SPS/HIRADMAT/";
  } else if (patternMatch("*HRM*", application)) {
    hierarchy += "SPS/HIRADMAT/";
  } else if (patternMatch("*ALICE*", application)) {
    hierarchy += "LHC/P2/";
  } else if (patternMatch("*CMS*", application)) {
    hierarchy += "LHC/P5/";
  } else if (patternMatch("*LHCb*", application)) {
    hierarchy += "LHC/P8/";
  } else if (patternMatch("*ATLAS*", application)) {
    hierarchy += "LHC/P1/";
  } else if (patternMatch("*S[GHFWUR]18*", application) || patternMatch("*UW18*", application) || patternMatch("*LHC18*", application)) {
    hierarchy += "LHC/P18/";
  } else if (patternMatch("*S[GHFWUR]1*", application) || patternMatch("*UW1*", application) || patternMatch("*LHC1*", application)) {
    hierarchy += "LHC/P1/";
  } else if (patternMatch("*S[GHFWUR]2*", application) || patternMatch("*UW2*", application) || patternMatch("*LHC2*", application)) {
    hierarchy += "LHC/P2/";
  } else if (patternMatch("*S[GHFWUR]3*", application) || patternMatch("*UW3*", application) || patternMatch("*LHC3*", application)) {
    hierarchy += "LHC/P3/";
  } else if (patternMatch("*S[GHFWUR]4*", application) || patternMatch("*UW4*", application) || patternMatch("*LHC4*", application)) {
    hierarchy += "LHC/P4/";
  } else if (patternMatch("*S[GHFWUR]5*", application) || patternMatch("*UW5*", application) || patternMatch("*LHC5*", application)) {
    hierarchy += "LHC/P5/";
  } else if (patternMatch("*S[GHFWUR]6*", application) || patternMatch("*UW6*", application) || patternMatch("*LHC6*", application)) {
    hierarchy += "LHC/P6/";
  } else if (patternMatch("*S[GHFWUR]7*", application) || patternMatch("*UW7*", application) || patternMatch("*LHC7*", application)) {
    hierarchy += "LHC/P7/";
  } else if (patternMatch("*S[GHFWUR]8*", application) || patternMatch("*UW8*", application) || patternMatch("*LHC8*", application)) {
    hierarchy += "LHC/P8/";
  } else {
    DebugN("ERROR: can't detect application for " + sDpe + " app " + application);
    return "ERROR";
  }
  
  hierarchy += application + "/" + deviceType;

  return hierarchy;
}


