/**
 * UNICOS
 * Copyright (C) CERN 2013 All rights reserved
 */
/**

@name LIBRARY: unRecipeFunctions.ctl

@author: Ivan Prieto Barreiro (EN-ICE-SIC)

Creation Date: April 2011 

version 1.3.4

Modification History:
  01/11/2011: Ivan
    - v1.1.0: The device type names have been modified to use the "CPC_" prefix.
	
  01/08/2012: Ivan
	- v1.2.0 : Adding new functionalities to the component:
		- Dynamic creation of recipe classes.
		- Recipe activation in multiple PLCs.

  09/12/2013: Ivan
	- v1.3.4 : Bug fixing and improvements in existing functionalities.
		
External Functions: 
  .  
  
Internal Functions:
  .
*/

const int unRecipeFunctions_ERROR_NoRecipeClass=-1;
const int unRecipeFunctions_ERROR_InvalidElements=-2;
const int unRecipeFunctions_ERROR_NoRecipeInstance=-3;


const string unRecipeFunctions_RecipeClassDpType="UnRcpClass";
const string unRecipeFunctions_RecipeClassDpPrefix="";
const string unRecipeFunctions_RecipeInstanceDpPrefix="recipeInstance/";
const string unRecipeFunctions_RecipeInstanceDpType="UnRcpInstance";
const string unRecipeFunctions_RcpFEAlias="RCP_FE";


/********************************************************** 
 *  Indexes for the elements in the recipe instance data  *
 **********************************************************/
const int RCP_INSTANCE_ID=1;
const int RCP_INSTANCE_NAME=2;
const int RCP_INSTANCE_CLASS=3;
const int RCP_INSTANCE_AUTHOR=4;
const int RCP_INSTANCE_VERSION=5;
const int RCP_INSTANCE_CREATED=6;
const int RCP_INSTANCE_LAST_USAGE=7;
const int RCP_INSTANCE_LAST_MODIFIED=8;
const int RCP_INSTANCE_LAST_ACT_USER=9;
const int RCP_INSTANCE_LAST_MOD_USER=10;
const int RCP_INSTANCE_VALUES=11;
const int RCP_INSTANCE_PREDEFINED=12;


/******************************************************* 
 *  Indexes for the elements in the recipe class data  *
 *******************************************************/
const int RCP_CLASS_NAME=1;
const int RCP_CLASS_TYPE=2;
const int RCP_CLASS_DESC=3;
const int RCP_CLASS_ELEMENTS=4;
const int RCP_CLASS_ELEMENTS_DESC=5;
const int RCP_CLASS_ELEMENTS_RANGE=6;
const int RCP_CLASS_ELEMENTS_UNIT=7;
const int RCP_CLASS_INSTANCES=8;
const int RCP_CLASS_EDITABLE=9;
const int RCP_CLASS_LAST_MODIFIED=10;
const int RCP_CLASS_LAST_ACTIVATED=11;
const int RCP_CLASS_LAST_ACTIVATION_TIME=12;


/********************************************************* 
 *  Indexes for the elements in the recipe header buffer *
 *********************************************************/
const int RCP_HEADER_MAX_SIZE	= 15;
const int RCP_HEADER_ID_LOW		= 1;
const int RCP_HEADER_ID_HIGH	= 2;
const int RCP_HEADER_NUM_MANREG	= 3;
const int RCP_HEADER_NUM_DPAR	= 4;
const int RCP_HEADER_DB_DPAR	= 5;
const int RCP_HEADER_NUM_WPAR	= 6;
const int RCP_HEADER_DB_WPAR	= 7;
const int RCP_HEADER_NUM_APAR	= 8;
const int RCP_HEADER_DB_APAR	= 9;
const int RCP_HEADER_NUM_AALARM	= 10;
const int RCP_HEADER_DB_AALARM	= 11;
const int RCP_HEADER_NUM_CONTR	= 12;
const int RCP_HEADER_DB_CONTR	= 13;
const int RCP_HEADER_SENT_VALUE	= 14;
const int RCP_HEADER_CRC		= 15;


/********************************************************* 
 *  Indexes for the elements in the recipe status buffer *
 *********************************************************/
const int RCP_STATUS_MAX_SIZE			= 16;
const int RCP_STATUS_MIRROR_DATA_SIZE	= 13;	// Indicates the last position where the data in the status buffer is a copy
												// of the data in the header buffer
const int RCP_STATUS_ID_LOW				= 1;
const int RCP_STATUS_ID_HIGH			= 2;
const int RCP_STATUS_NUM_MANREG			= 3;
const int RCP_STATUS_NUM_DPAR			= 4;
const int RCP_STATUS_DB_DPAR			= 5;
const int RCP_STATUS_NUM_WPAR			= 6;
const int RCP_STATUS_DB_WPAR			= 7;
const int RCP_STATUS_NUM_APAR			= 8;
const int RCP_STATUS_DB_APAR			= 9;
const int RCP_STATUS_NUM_AALARM			= 10;
const int RCP_STATUS_DB_AALARM			= 11;
const int RCP_STATUS_NUM_CONTR			= 12;
const int RCP_STATUS_DB_CONTR			= 13;
const int RCP_STATUS_STATUS				= 14;
const int RCP_STATUS_CRC				= 15;
const int RCP_STATUS_ERR				= 16;


/********************************************************* 
 *                 Recipe Status                         *
 *********************************************************/
const int RCP_ACTIVATION_TRIGGERED = 0; // The recipe activation has been triggered
const int RCP_ACTIVATED=1;            	// The recipe is activated in the PLCs
const int RCP_NO_PLC_CONNECTION = 2;  	// There is no conection with one or more PLCs
const int RCP_INACTIVE=3;             	// The recipe is inactive
const int RCP_BAD_TYPES=4;            	// The value type is different than the DPE type
const int RCP_BAD_STRUCTURE=5;        	// The number of values is different than the number of DPEs
const int RCP_ACTIVATION_FAILED=6;    	// The recipe activation failed


/********************************************************* 
 *                 Recipe Activation Status              *
 *    (values received in the recipe status buffer)      *
 *********************************************************/
const int RCP_ACT_STATUS_ERROR      	   = -1;  // There was an error during the recipe activation
const int RCP_ACT_STATUS_STANDBY           = 0;   // Recipe activation in standby
const int RCP_ACT_STATUS_NEW_ID_RECEIVED   = 1;   // New recipe identified received
const int RCP_ACT_STATUS_CRC_OK			   = 2;
const int RCP_ACT_STATUS_DONE              = 3;   // Recipe activation done
                                  

/********************************************************* 
 *      Constants used in the recipe activation          *
 *********************************************************/
// Recipe header 'Sent Value'
const int RCP_ACT_HEADER_CANCEL			= -1;	// Cancel the recipe activation
const int RCP_ACT_HEADER_SENDING_DATA	= 1;	// Sending recipe data to the PLC
const int RCP_ACT_HEADER_DATA_SENT		= 2;	// All data sent to the PLC
const int RCP_ACT_HEADER_ACTIVATE		= 3;	// Recipe activation order 
const int RCP_ACT_HEADER_ACTIVATION_OK	= 4;  	// Recipe activation Ok
			

/**************************************************************************
 *      Constants used as return value for the recipe activation          *
 **************************************************************************/
const int RCP_ACT_RETVAL_ACTIVATION_OK		=  0;	// The recipe activation has been completed successfully
const int RCP_ACT_RETVAL_BAD_RECIPE_DATA	= -1;	// The recipe data is wrong (recipe doesn't exist, DPs and values don't match, ...)
const int RCP_ACT_RETVAL_BAD_PLC_STATUS		= -2;	// The PLC status is wrong
const int RCP_ACT_RETVAL_BAD_PLC_TYPE		= -3;	// The PLC Type is unknown
const int RCP_ACT_RETVAL_PLC_BUSY			= -4;	// The PLC is busy activating another recipe
const int RCP_ACT_RETVAL_ACTIVATION_TIMEOUT = -5;	// The recipe activation timeout has expired
const int RCP_ACT_RETVAL_PLC_CANCELLATION	= -6;	// The PLC has cancelled the recipe activation
const int RCP_ACT_RETVAL_ACTIVATION_ERROR	= -7;	// The PLC has cancelled the recipe activation


/**************************************************************************
 *      Constants used as return value while getting the initial recipes  *
 **************************************************************************/
const int RCP_INITIAL_OK					= 0;	// The initial recipe of the class has been found
const int RCP_INITIAL_NO_DEFINED			= -1;	// There is no initial recipe for the recipe class
const int RCP_INITIAL_TOO_MANY				= -2;	// There are several initial recipes for the class
const int RCP_INITIAL_UNDEFINED				= -3;	// Other error while getting the initial recipe			


/************************************
 *      Constants to define colors  *
 ************************************/
const string CELL_EDIT_COLOR 			   = "{204,255,153}";
const string CELL_VALUE_OUT_OF_RANGE_COLOR = "{255,64,64}";


/****************************** 
 *      Global   Variables    *
 *****************************/
global mapping unRecipe_rangeElements;  		// Map where key=device type, value=dpe containing the range definition for the device type
global mapping unRecipe_unRecipe_pidManRegBits; // Map where key=controller dpe name, value=bit position of the dpe inside the ManReg01 
global mapping unRecipe_aalarmManRegBits;		// Map where key=analog alarm dpe name, value=bit position of the dpe inside the ManReg01 
global int g_editingRowNumber = -1;
global string g_editingColName = "";
global mapping unRecipe_recipeActivationInfo; 	// Map where key=recipe instance Dp, value=Matrix containing the necessary information to activate recipes in multi-PLCs 								
const int PLCS_REQUIRED_FOR_ACTIVATION_INDEX	= 1;
const int PLCS_READY_TO_ACTIVATE_INDEX 			= 2;	// Index in the recipe activation info matrix for the PLCs that are ready to activate the recipe.
const int PLCS_ACTIVATED_RECIPE_INDEX			= 3;	// Index in the recipe activation info matrix for the PLCs that activated the recipe.

/****************************** 
 *       Recipe Buffers       *
 *****************************/
global mapping gHeaderMap;			// Header buffer map
global mapping gManRegAddrMap;  	// ManReg address buffer map
global mapping gManRegValMap;		// ManReg values buffer map: this are the values used in the ArmRcp
global mapping gReqAddrMap;			// Req address buffer map
global mapping gReqValMap;			// Req values buffer map


/****************************** 
 *    ManReg Bit positions    *
 *****************************/
// DPAR bit definitions
const int DPAR_ARM_BIT		= 2;
const int DPAR_MONR_BIT 	= 4;
const int DPAR_MOFFR_BIT	= 5;
// WPAR bit definitions
const int WPAR_ARM_BIT		= 2;
const int WPAR_NEWMR_BIT	= 6;
// APAR bit definitions
const int APAR_ARM_BIT		= 2;
const int APAR_NEWMR_BIT	= 6;
// AALARM bit definitions
const int AALARM_ARM_BIT	= 2;
// PID bit definitions
const int PID_ARM_BIT		= 4;

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Initialization of the ManReg bit positions used in the recipes by the different device types.
 */
void _unRecipeFunctions_initializeManRegBits()
{
  // Initializes the bit positions of the ManReg01 for the CPC_Controller Objects
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.ArmRcp"]= 4;
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MSP"]   = 7;
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MSPH"]  = 8;
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MSPL"]  = 9;  
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MOutH"] = 10;  
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MOutL"] = 11;  
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MKc"]   = 12;  
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MTi"]   = 14;
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MTd"]   = 13;
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MTds"]  = 15;  
  
  // Initializes the bit positions of the ManReg01 for the Analog Alarm Objects
  unRecipe_aalarmManRegBits[".ProcessOutput.HH"] = 8;
  unRecipe_aalarmManRegBits[".ProcessOutput.H"]  = 9;
  unRecipe_aalarmManRegBits[".ProcessOutput.L"]  = 12;
  unRecipe_aalarmManRegBits[".ProcessOutput.LL"] = 13;  
}

//--------------------------------------------------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//		Methods to get recipe buffers by PLC name
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/** 
 * Get the recipe header buffer for the specified PLC.
 * @param sPlcName - [IN] The PLC name which recipe header buffer is required.
 * @return dyn_int containing the values of the recipe header buffer for the specified PLC.
 */
dyn_int _unRecipeFunctions_getHeaderBuffer(string sPlcName) {
  if (mappingHasKey(gHeaderMap, sPlcName)) 
    return gHeaderMap[sPlcName];
  
  dyn_int buffer;
  dynAppend(buffer, 0);
  gHeaderMap[sPlcName] = buffer;
  return buffer;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the recipe ManRegAddress buffer for the specified PLC.
 * @param sPlcName - [IN] The PLC name which recipe ManRegAddress buffer is required.
 * @return dyn_int containing the values of the recipe ManRegAddress buffer for the specified PLC.
 */
dyn_int _unRecipeFunctions_getManRegAddrBuffer(string sPlcName) {
  if (mappingHasKey(gManRegAddrMap, sPlcName)) 
    return gManRegAddrMap[sPlcName];
  
  dyn_int buffer;
  dynAppend(buffer, 0);
  gManRegAddrMap[sPlcName] = buffer;
  return buffer;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the recipe ManRegValues buffer for the specified PLC.
 * @param sPlcName - [IN] The PLC name which recipe ManRegValues buffer is required.
 * @return dyn_int containing the values of the recipe ManRegValues buffer for the specified PLC.
 */
dyn_int _unRecipeFunctions_getManRegValBuffer(string sPlcName) {
  if (mappingHasKey(gManRegValMap, sPlcName)) 
    return gManRegValMap[sPlcName];
  
  dyn_int buffer;
  dynAppend(buffer, 0);
  gManRegValMap[sPlcName] = buffer;
  return buffer;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the recipe ReqAddress buffer for the specified PLC.
 * @param sPlcName - [IN] The PLC name which recipe ReqAddress buffer is required.
 * @return dyn_int containing the values of the recipe ReqAddress buffer for the specified PLC.
 */
dyn_int _unRecipeFunctions_getReqAddrBuffer(string sPlcName) {
  if (mappingHasKey(gReqAddrMap, sPlcName)) 
    return gReqAddrMap[sPlcName];
  
  dyn_int buffer;
  dynAppend(buffer, 0);
  gReqAddrMap[sPlcName] = buffer;
  return buffer;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the recipe ReqValues buffer for the specified PLC.
 * @param sPlcName - [IN] The PLC name which recipe ReqValues buffer is required.
 * @return dyn_float containing the values of the recipe ReqValues buffer for the specified PLC.
 */
dyn_float _unRecipeFunctions_getReqValBuffer(string sPlcName) {
  if (mappingHasKey(gReqValMap, sPlcName)) 
    return gReqValMap[sPlcName];
  
  dyn_float buffer;
  dynAppend(buffer, 0.0);
  gReqValMap[sPlcName] = buffer;
  return buffer;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Remove all the recipe buffers associated to the specified PLC.
 * @param sPlcName - [IN] The PLC name which recipe buffers must be removed.
 */
void _unRecipeFunctions_removeRecipeBuffers(string sPlcName) {
  mappingRemove(gHeaderMap, sPlcName);
  mappingRemove(gManRegAddrMap, sPlcName);
  mappingRemove(gManRegValMap, sPlcName);
  mappingRemove(gReqAddrMap, sPlcName);
  mappingRemove(gReqValMap, sPlcName);
}

//--------------------------------------------------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//		Methods to set DP values
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void _unRecipeFunctions_setDpValue(string dpName, string attribute, anytype value, dyn_string &exceptionInfo)
{
  int rc=dpSet(dpName+attribute, value);
  if (rc) {
    fwException_raise(exceptionInfo,"ERROR","Could not set the "+attribute+" value for the DP: "+dpName+attribute,"");
    return;
  }    
}

//------------------------------------------------------------------------------------------------------------------------
void _unRecipeFunctions_appendDpValue(string dpName, string attribute, dyn_anytype value, dyn_string &exceptionInfo)
{
  dyn_anytype currentValue;
  int rc=dpGet(dpName+attribute, currentValue);
  if (rc) {
    fwException_raise(exceptionInfo,"ERROR","Could not get the "+attribute+" value for the DP: "+dpName,"");
    return;
  }    
  
  dynAppend(currentValue, value);
  rc = dpSet(dpName+attribute, currentValue);
  if (rc) {
    fwException_raise(exceptionInfo,"ERROR","Could not set the "+attribute+" value for the DP: "+dpName,"");
    return;
  }      
}

//--------------------------------------------------------------------------------------------------------------------------
/** 
 * Gets the formatted values of a set of DPEs.
 * @param dpNames - [IN] List of dp names in the UNICOS format.
 * @param values - [IN] List of values for the specified dpNames.
 * @param formattedValues - [OUT] Formatted values.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_formatValues(dyn_string dpNames, dyn_string values, 
                                     dyn_anytype &formattedValues, dyn_string &exceptionInfo)
{
  string rangeElement, deviceType, rangeMin, rangeMax, pvssDpName;
  dyn_string dsOnlineDpes;
  float fFormattedValue, fValue;
  bool bOk;
  
  if (dynlen(dpNames)!=dynlen(values)) {
    fwException_raise(exceptionInfo, "ERROR", "The number of DPEs is different than the number of values!", "");
    return;
  }  
  
  for (int i=1; i<=dynlen(dpNames); i++) {  
    unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(dpNames[i], pvssDpName);  
    // Get the DPE name (e.g. .ProcessOutput.MPosR")
    string dp=unGenericDpFunctions_getDpName(pvssDpName);
    int pos = strpos(pvssDpName, dp);
    string element = substr(pvssDpName, pos+strlen(dp), strlen(pvssDpName)-strlen(dp));   
    deviceType = dpTypeName(pvssDpName);

	// Removes the white spaces to format properly the exponential values (1.5 e1 => 1.5e1)
	strreplace(values[i], " ", "");
	if (deviceType=="CPC_DigitalParameter") {	
		if (values[i]==0) {
			formattedValues[i] = "FALSE";
			continue;
		}
		else if (values[i]==1) {
			formattedValues[i] = "TRUE";
			continue;
		}
			
		string value = strtoupper(values[i]);
		if (value=="TRUE" || value=="FALSE") {
			formattedValues[i] = value;
		} 
		continue;
	}
		
    // Get the dpe where the range is defined for the current recipe element
    if (!mappingHasKey(unRecipe_rangeElements, deviceType+":"+element)){
      _unRecipeFunctions_getRangeElementFromPVSSDPE(pvssDpName, rangeElement, exceptionInfo);  
      unRecipe_rangeElements[deviceType+":"+element] = rangeElement;
    } else {
      rangeElement = unRecipe_rangeElements[deviceType+":"+element];    
    }   
    
    string format="";
    _unRecipeFunctions_getInstanceRange(pvssDpName, rangeElement, rangeMin, rangeMax, exceptionInfo);
    if (rangeMin!="" && rangeMax!=""){
      format = dpGetFormat(unGenericDpFunctions_getDpName(pvssDpName)+rangeElement);
    }
    
    if (format!="") {
      formattedValues[i]=unGenericObject_FormatValue(format, values[i]);
	  continue;
    } 
    
	// if the value is a float, format the value using the online DPE format
	if (dpElementType(dpNames[i]) == DPEL_FLOAT) {
		_unRecipeFunctions_getOnlineValueDpes(makeDynString(unGenericDpFunctions_getDpName(pvssDpName)+rangeElement), dsOnlineDpes);

		format = dpGetFormat(dsOnlineDpes[1]);
		//fFormattedValue = (float)values[i];
		//formattedValues[i] = unGenericObject_FormatValue(format, fFormattedValue);
		unGenericObject_StringToFloat(values[i], fValue, bOk);
		if (!bOk) {
			fwException_raise(exceptionInfo, "ERROR", "Float conversion error :" + values[i], "");
		}
		formattedValues[i] = unGenericObject_FormatValue(format, fValue);
	} else {
		// Applies the default PVSS format
		string s = values[i];  
		formattedValues[i] = s;
	}
  }  
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Show the online values in the recipe instance panel for the specified recipe instance.
 * @param sRecipeDpName - [IN] String containing the recipe instance DP name.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_displayOnlineValues(string sRecipeDpName, dyn_string &exceptionInfo)
{
  dyn_string dsDpNames, dsAlias, dsDescription, dsUnit, dsRange, dsDiffIndex, dsEqIndex;
  dyn_anytype daOnlineValues, daRecipeValues;
  dyn_int diIndex, diRowStyles, diRowStyleIndex, diDiffRowIndex;
  
  if (!shapeExists("RecipeElements")) {
	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	DebugTN(exceptionInfo);
	return;	
  }
  
  dpSet(sRecipeDpName + ".ProcessInput.State", UN_RCP_INSTANCE_STATE_ONLINE_VALUES);
  
  _unRecipeFunctions_writeInRecipeLog("Loading recipe online values.", true);
  getMultiValue("RecipeElementsCopy", "getColumnN", 0, diIndex,
				"RecipeElementsCopy", "getColumnN", 1, dsDpNames,
				"RecipeElementsCopy", "getColumnN", 2, dsAlias,
				"RecipeElementsCopy", "getColumnN", 3, dsDescription,
				"RecipeElementsCopy", "getColumnN", 4, daRecipeValues,
				"RecipeElementsCopy", "getColumnN", 5, dsUnit,
				"RecipeElementsCopy", "getColumnN", 6, dsRange);
				
  unRecipeFunctions_getOnlineValues(dsDpNames, daOnlineValues, exceptionInfo);
  if(dynlen(exceptionInfo))  if (dynlen(exceptionInfo))	{ DebugTN(exceptionInfo); return;}

  _unRecipeFunctions_getRowStylesForValueComparison(daRecipeValues, daOnlineValues, diRowStyleIndex, diRowStyles, diDiffRowIndex);
  setMultiValue("RecipeElements", "deleteAllLines",
				"RecipeElements", "appendLines", dynlen(daRecipeValues),
					"Index", diIndex,
					"DeviceDp", dsDpNames,
					"Alias", dsAlias,
					"Description", dsDescription,
					"Value", daOnlineValues,
					"Unit", dsUnit,
					"Range", dsRange);
  
  _unRecipeFunctions_setTableColumnBGColor("RecipeElements", "Value", CELL_EDIT_COLOR);
  setMultiValue("RecipeElements", "rowFontType", diRowStyleIndex, diRowStyles);
  
  int iLen = dynlen(diDiffRowIndex);
  if (iLen>0) {
	string sLogMessages = "Recipe values and online values are different:";
	for (int i=1; i<=iLen; i++) {
		string sDpe = dsDpNames[(diDiffRowIndex[i])];
		dyn_string sDpeSplit = strsplit(sDpe, ".");
		string sAlias = dsAlias[(diDiffRowIndex[i])] + "." + sDpeSplit[(dynlen(sDpeSplit))];
		sLogMessages += "\nDifferent values on row index " + diIndex[(diDiffRowIndex[i])] + 
			". Device: " + sAlias +
	  		" => Recipe Value: " + daRecipeValues[(diDiffRowIndex[i])] + 
			", Online Value: " + daOnlineValues[(diDiffRowIndex[i])];
	}
	_unRecipeFunctions_writeInRecipeLog(sLogMessages, false, true);
  }
  
  _unRecipeFunctions_writeInRecipeLog("Recipe online values loaded.", true);
  dpSet(sRecipeDpName + ".ProcessInput.State", UN_RCP_INSTANCE_STATE_EDIT);
}
//------------------------------------------------------------------------------------------------------------------------
bool _unRecipeFunctions_getRowStylesForValueComparison(dyn_anytype daValues1, dyn_anytype daValues2, dyn_int &diRowStyleIndex, dyn_int &diRowStyles, dyn_int &diDiffRowIndex) 
{
  if (dynlen(daValues1) != dynlen(daValues2) )
	return false;
  
  int iLen = dynlen(daValues1);
  for (int i=1; i<=iLen; i++) {
	dynAppend(diRowStyleIndex, (i-1));
	if (daValues1[i] != daValues2[i]) {
		dynAppend(diRowStyles, 1);
		dynAppend(diDiffRowIndex, i);
	} else {
		dynAppend(diRowStyles, 0);
	}
  }

  return true;
}
//------------------------------------------------------------------------------------------------------------------------
/**
 * Get the online values of a recipe instance.
 * @param dsDpNames 	- [IN] DP Names of the elements which online values are required.
 * @param values 		- [OUT] Online formatted values corresponding to the recipe class DPEs.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */ 
void unRecipeFunctions_getOnlineValues(dyn_string dsDpNames, dyn_anytype &values, dyn_string &exceptionInfo)
{
  //dyn_string dpNames;
  dyn_string outputDpNames;
  dyn_anytype formattedValues;
  dyn_dyn_mixed recipeObject;

  dynClear(values);
  
   // Get recipe dpes
  _unRecipeFunctions_getOnlineValueDpes (dsDpNames, outputDpNames);
  dpGet(outputDpNames, values);
  _unRecipeFunctions_formatValues(dsDpNames, values, formattedValues, exceptionInfo);  
  
  values = formattedValues;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the range of the specified DP.
 * @param sDpName - [IN] The DP which range is required.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 * @return String containing the valid range for the specified DP (format: [min, max]).
 */
string unRecipeFunctions_getDeviceRange(string sDpName, dyn_string &exceptionInfo)
{
  string range, deviceType, dp, element, rangeMin, rangeMax, rangeElement;
  
  if (!dpExists(sDpName)) return;
  
  deviceType = dpTypeName(sDpName);   
  dp=unGenericDpFunctions_getDpName(sDpName);
  int pos = strpos(sDpName, dp);
  element = substr(sDpName, pos+strlen(dp), strlen(sDpName)-strlen(dp));
  dyn_string splitElement = strsplit(element, ".");

  // If the device type is a DigitalParameter the range is [FALSE, TRUE]
  if (deviceType=="CPC_DigitalParameter") {
    range = "[FALSE, TRUE]";
    return range;
  }
       
  if (!mappingHasKey(unRecipe_rangeElements, deviceType+":"+element)){
    _unRecipeFunctions_getRangeElementFromPVSSDPE(sDpName, rangeElement, exceptionInfo);
    if (dynlen(exceptionInfo))  return;
    unRecipe_rangeElements[deviceType+":"+element] = rangeElement;
  } else {
    rangeElement = unRecipe_rangeElements[deviceType+":"+element];
  }

  // Get the range values
  _unRecipeFunctions_getInstanceRange(sDpName, rangeElement, rangeMin, rangeMax, exceptionInfo);       
  if (rangeMin!="" && rangeMax!="")       
    range = "[" + rangeMin +", " + rangeMax + "]";  
	
  return range;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the measurement unit of a DP.
 * @param sDpName - [IN] The datapoint name.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 * @return String containing the measurement unit of the specified DP.
 */
string unRecipeFunctions_getDpeUnit(string sDpName, dyn_string &exceptionInfo) {
  string deviceType, dp, element, rangeElement;
  
  if (!dpExists(sDpName)) return "";
  
  deviceType = dpTypeName(sDpName);   
  dp=unGenericDpFunctions_getDpName(sDpName);
  int pos = strpos(sDpName, dp);
  element = substr(sDpName, pos+strlen(dp), strlen(sDpName)-strlen(dp));
  dyn_string splitElement = strsplit(element, ".");
       
  if (!mappingHasKey(unRecipe_rangeElements, deviceType+":"+element)){
    _unRecipeFunctions_getRangeElementFromPVSSDPE(sDpName, rangeElement, exceptionInfo);
    if (dynlen(exceptionInfo))  return "";
    unRecipe_rangeElements[deviceType+":"+element] = rangeElement;
  } else {
    rangeElement = unRecipe_rangeElements[deviceType+":"+element];
  }
  
  return dpGetUnit(dp + rangeElement);
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Get the DPEs associated to each device type in a recipe type.
 * @param sRecipeType - [IN] Recipe type name.
 * @param map - [OUT] Mapping variable where Key=Device type name, Value=List containing the DPEs 
 *              included in the recipe type for the specified key (device type).
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_getTypeMapping(string sRecipeType, mapping &map, dyn_string &exceptionInfo)
{
   dyn_dyn_string rtData;
  	_fwConfigurationDB_getRecipeTypeData(sRecipeType,rtData,exceptionInfo);
    
   if (dynlen(exceptionInfo)) return;
   
   int len = dynlen(rtData);
   for (int i=1; i<=len; i++){
     if (strtoupper(rtData[i][5])=="TRUE") {
        string deviceType = rtData[i][2];
        dyn_string items;
        if (mappingHasKey(map,deviceType)){
           items = map[deviceType];
        }
        dyn_string splitElement = strsplit(rtData[i][3], ".");
        dynAppend(items, splitElement[dynlen(splitElement)]);
        map[deviceType] = items;
     }
   }
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Save the recipe values.
 * @param recipeObject - [IN] Recipe instance data.
 * @param values - [IN/OUT] Values for the recipe elements.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */ 
void _unRecipeFunctions_saveRecipeValues(dyn_dyn_mixed recipeObject, dyn_anytype &values, dyn_string &exceptionInfo)
{
  dynClear(exceptionInfo);
  
  dyn_anytype formattedValues;
  string instanceName = recipeObject[fwConfigurationDB_RO_META_ORIGNAME];
  dyn_string elements = recipeObject[fwConfigurationDB_RO_DPE_NAME];
  
  // The number of elements in the recipe must be the same as the number of values
  if (dynlen(elements)!=dynlen(values)){
    fwException_raise(exceptionInfo,"ERROR","The number of elements in the recipe " + 
                      "is different than the number of values.","");
    return;      
  }  
  
  _unRecipeFunctions_formatValues(elements, values, formattedValues, exceptionInfo);
  if (dynlen(exceptionInfo))  return;

  // Save the recipe values in the JCOP recipe
  recipeObject[fwConfigurationDB_RO_VALUE] = formattedValues;
  recipeObject[fwConfigurationDB_RO_META_LASTMODIFICATIONTIME] = getCurrentTime();
  recipeObject[fwConfigurationDB_RO_META_LASTMODIFICATIONUSER] = getUserName();
  fwConfigurationDB_saveRecipeToCache(recipeObject, "", instanceName, exceptionInfo);
  if (dynlen(exceptionInfo))   return;
  
  values = formattedValues;
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Get the element where the range is defined for a PVSS DPE
 * @param sDpName - DPE name in the PVSS format (e.g. dist_1:gcs-CFP_STP_CVSF18-SF18-CPC_AnalogParameter-00001.ProcessOutput.ValRq)
 * @param sElement - Variable to store the element name that contains the range (e.g. .ProcessInput.CurValSt)
 * @param exceptionInfo - Standard exception handling variable.
 */
void _unRecipeFunctions_getRangeElementFromPVSSDPE(string sDpName, string &sElement, dyn_string &exceptionInfo)
{
    string deviceType;
    dyn_string dpeInfo;
    
    sElement = "";
    dynClear(exceptionInfo);
    
    if (!dpExists(sDpName)) {
      fwException_raise(exceptionInfo,"ERROR","The data point element: "+sDpName+
                        " doesn't exist.",unRecipeFunctions_ERROR_InvalidElements);
      return;
    }
    
    deviceType = dpTypeName(sDpName);
    int pos = strpos(sDpName, ".");
    if (pos<0) {
 //     Debug("The elements defined in the recipe class: "+recipeClass+" are wrong.\n");
 /*     fwException_raise(exceptionInfo,"ERROR","The elements defined in the recipe class: "+recipeClass+
                        " are wrong.",unRecipeFunctions_ERROR_InvalidElements);
   */   return;
    }

    if (!dpExists(deviceType+"_RcpInfo.DpeInfo")) {
      // If the information about what is the DPE containing the range definition doesn't exist,
      // returns the DPE name
	  //DebugTN("The " + deviceType+"_RcpInfo.DpeInfo DPE doesn't exist!");	  
      string dp=unGenericDpFunctions_getDpName(sDpName);
      int pos = strpos(sDpName, dp);
      sElement = substr(sDpName, strlen(dp), strlen(sDpName)-strlen(dp));
      return;
    }
        
    sElement = substr(sDpName, pos, strlen(sDpName)-pos);
    dpGet(deviceType+"_RcpInfo.DpeInfo", dpeInfo);

    for (int i=1; i<=dynlen(dpeInfo); i++){
      string currDpeInfo = dpeInfo[i];
      dyn_string splitInfo = strsplit(currDpeInfo, ":");
      if (dynlen(splitInfo)!=2) {
        fwException_raise(exceptionInfo,"ERROR","The recipe info for the device type: "+deviceType+
                          " is wrong.",unRecipeFunctions_ERROR_InvalidElements);
        return;        
      }
      
      splitInfo[1] = strltrim(splitInfo[1]);
      splitInfo[1] = strrtrim(splitInfo[1]);
      if (splitInfo[1] == sElement) {
        sElement = strltrim(splitInfo[2]);
        sElement = strrtrim(sElement);
        return;
      }
    }
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Get the min. and max. values defined in a range for a DPE.
 * @param sDpe - [IN] Data point element for whom the range values are required.
 * @param sRangeElement - [IN] DPE where the range is defined for the 'sDpe' parameter (e.g.: ".ProcessInput.CurValSt")
 * @param sRangeMin - [OUT] Minimum value of the range definition.
 * @param sRangeMax - [OUT] Maximum value of the range definition.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_getInstanceRange(string sDpe, string sRangeElement, string &sRangeMin, 
                                         string &sRangeMax, dyn_string &exceptionInfo)
{
  string dpName = unGenericDpFunctions_getDpName(sDpe)+sRangeElement;
  if (!dpExists(dpName)){
    fwException_raise(exceptionInfo,"ERROR","The specified data point doesn't exist: "+dpName,"");
    return;        
  }
  
  string min="",max="";
  dpGet(dpName+":_pv_range.._min", min,
        dpName+":_pv_range.._max", max);
    
  if (min=="" && max=="") {
    sRangeMin="";
    sRangeMax="";
  } else {
    string format = dpGetFormat(dpName);
    sRangeMin = unGenericObject_FormatValue(format, min);
    sRangeMax = unGenericObject_FormatValue(format, max);     
  }
}  

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Activate a recipe.
 * @param sRcpInstanceDp - [IN] Data point name of the recipe which activation is triggered.
 * @param exceptionInfo  - [OUT] Standard exception handling variable.
 * @return Integer value to indicate the activation result (see the RCP_ACT_RETVAL_ constants).
 */
int unRecipeFunctions_activateRecipe(string sRcpInstanceDp, dyn_string &exceptionInfo) 
{
  mapping systemDpes, plcDpes;
  dyn_anytype plcNames;
  dyn_dyn_mixed rcpInstance;
  dyn_string sStatusMessage, dsBusyPlcs;
  string sRcpClassName, sRcpInstanceName;
  time t0, t1;
  bool bActive, bCanActivateRecipe;
  int iLen, iActivationResult;

  
  // Check if the recipe DP exists
  if (!dpExists(sRcpInstanceDp)) {
    fwException_raise(exceptionInfo, "ERROR", "There recipe DP doesn't exist: " + sRcpInstanceDp, "");
	return RCP_ACT_RETVAL_BAD_RECIPE_DATA;
  }
  
  // If the sRcpInstanceDp ends with '.', remove the dot
  iLen = strlen(sRcpInstanceDp);
  if (sRcpInstanceDp[(iLen-1)] == '.') {
	sRcpInstanceDp = substr(sRcpInstanceDp, 0, (iLen-1));
  }
  
  // Disables the recipe class combo box, the recipe instances table and the recipe elements table
  _unRecipeFunctions_setRecipeInstancePanelComponentsEnabled(false);
  
  dpSet(sRcpInstanceDp+".ProcessInput.State", UN_RCP_INSTANCE_STATE_ACTIVATE);
  
  dynClear(exceptionInfo);  
  //t0 = getCurrentTime();
   
  if (mappinglen(unRecipe_unRecipe_pidManRegBits)==0 || mappinglen(unRecipe_aalarmManRegBits)==0)
    _unRecipeFunctions_initializeManRegBits();
  
  dpGet(sRcpInstanceDp+".ProcessInput.ClassName", sRcpClassName,
		sRcpInstanceDp+".ProcessInput.InstanceName", sRcpInstanceName);  
  
  fwConfigurationDB_loadRecipeFromCache(sRcpClassName + "/" +sRcpInstanceName, makeDynString(), "", rcpInstance, exceptionInfo);
  if (dynlen(exceptionInfo)) {
    // Enables the recipe class combo box, the recipe instances table and the recipe elements table
	_unRecipeFunctions_setRecipeInstancePanelComponentsEnabled(true);
	dpSet(sRcpInstanceDp+".ProcessInput.State", "");
	// Show the error message  
	DebugTN(exceptionInfo); 
	return RCP_ACT_RETVAL_BAD_RECIPE_DATA; 
  }
  
  // Check if the recipe instance has elements
  if (dynlen(rcpInstance[fwConfigurationDB_RO_DPE_NAME]) == 0) {
	_unRecipeFunctions_writeInRecipeLog("The recipe instance has no elements and it can not be activated.");
	dpSet(sRcpInstanceDp+".ProcessInput.State", "");
	_unRecipeFunctions_setRecipeInstancePanelComponentsEnabled(true);
	return RCP_ACT_RETVAL_ACTIVATION_OK;
  }
  
  // Check if all the PLCs are connected
  unRecipeFunctions_getPlcStatus(rcpInstance[fwConfigurationDB_RO_DPE_NAME], bActive, sStatusMessage, exceptionInfo);
  if (!bActive) {
    fwException_raise(exceptionInfo, "ERROR", "There isn't connection with the PLC(s): " + sStatusMessage, "");
    // Enables the recipe class combo box, the recipe instances table and the recipe elements table
	_unRecipeFunctions_setRecipeInstancePanelComponentsEnabled(true);
	// Show the error message
	_unRecipeFunctions_writeInRecipeLog("There isn't connection with the PLC(s): " + sStatusMessage + 
			". The recipe " + sRcpClassName + " / " + sRcpInstanceName + " can't be activated");
	dpSet(sRcpInstanceDp+".ProcessInput.State", "");
	return RCP_ACT_RETVAL_BAD_PLC_STATUS;
  }
  
  // Gets the recipe elements by PLC
  _unRecipeFunctions_getElementsByPLC(rcpInstance[fwConfigurationDB_RO_DPE_NAME], 
  									  rcpInstance[fwConfigurationDB_RO_VALUE], 
  									  plcDpes, 
  									  exceptionInfo);
  
  if (dynlen(exceptionInfo))  { 
	unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "unRecipeFunctions_activateRecipe", "unRecipeFunctions_activateRecipe", 
					"Error getting the elements by plc"); 
    // Enables the recipe class combo box, the recipe instances table and the recipe elements table
	_unRecipeFunctions_setRecipeInstancePanelComponentsEnabled(true);
	// Show the error message
    _unRecipeFunctions_writeInRecipeLog("Recipe can't be activated: The number of recipe elements is bigger than the recipe buffer size.");
	dpSet(sRcpInstanceDp+".ProcessInput.State", "");
	DebugTN(exceptionInfo); 
	return RCP_ACT_RETVAL_BAD_RECIPE_DATA;
  }
  
  // Check if the recipe buffers are unlocked in all the PLCs
  plcNames = mappingKeys(plcDpes);
  bCanActivateRecipe = _unRecipeFunctions_canActivateRecipeInPlcs(sRcpInstanceDp, plcNames, dsBusyPlcs, exceptionInfo);
  if (false==bCanActivateRecipe) {
    if (dynlen(exceptionInfo)) {
		_unRecipeFunctions_writeInRecipeLog("Error while locking the recipe buffers: " + exceptionInfo[2]);
	} else {
		fwException_raise(exceptionInfo, "ERROR", "Another recipe is being activated in the PLC(s): " + dsBusyPlcs, "");
		fwException_raise(exceptionInfo, "INFO", "Please wait until the recipe activation is completed.", "");
		DebugTN(exceptionInfo);
		// Enables the recipe class combo box, the recipe instances table and the recipe elements table
		_unRecipeFunctions_setRecipeInstancePanelComponentsEnabled(true);
		_unRecipeFunctions_writeInRecipeLog("Another recipe is being activated in the PLC(s): " + dsBusyPlcs);
		_unRecipeFunctions_writeInRecipeLog("Please wait until the recipe activation is completed.");
		dpSet(sRcpInstanceDp+".ProcessInput.State", "");
	}
	return RCP_ACT_RETVAL_PLC_BUSY;
  }
  
  // Create the recipe header buffer for the PLCs 
  iLen = dynlen(plcNames);
  for (int i=1; i<=iLen; i++) {
	_unRecipeFunctions_getHeaderBuffer(plcNames[i]);
  }
  
  unRecipeFunctions_setRecipeStatusInfo(RCP_ACTIVATION_TRIGGERED);
  _unRecipeFunctions_writeInRecipeLog("Activating recipe: " + sRcpClassName + " / " + sRcpInstanceName, TRUE);
  
  // Format all the recipe values and saves the formatted values 
  // This is necessary to avoid problems if the formats have been modified and the recipe values don't have the correct formats
  _unRecipeFunctions_formatValues(rcpInstance[fwConfigurationDB_RO_DPE_NAME], rcpInstance[fwConfigurationDB_RO_VALUE], 
                                     rcpInstance[fwConfigurationDB_RO_VALUE], exceptionInfo);
  
  fwConfigurationDB_saveRecipeToCache(rcpInstance, "", sRcpClassName +"/" + sRcpInstanceName, exceptionInfo);
  if (dynlen(exceptionInfo)) {
  _unRecipeFunctions_unlockRecipeBuffers(plcNames, exceptionInfo);
  _unRecipeFunctions_removeRecipeHeaderBuffers(plcNames);
    // Enables the recipe class combo box, the recipe instances table and the recipe elements table
	_unRecipeFunctions_setRecipeInstancePanelComponentsEnabled(true);
	// Show the error message
	DebugTN(exceptionInfo);
	_unRecipeFunctions_writeInRecipeLog("Error formatting the recipe values: the recipe activation is cancelled.");
	unRecipeFunctions_setRecipeStatusInfo(RCP_ACTIVATION_FAILED);
	dpSet(sRcpInstanceDp+".ProcessInput.State", "");
	return RCP_ACT_RETVAL_BAD_RECIPE_DATA;
  }
    
  // Recipe activation
  iActivationResult = _unRecipeFunctions_activateRecipeByPLC(plcDpes, sRcpInstanceDp, exceptionInfo);  
  
  // Enables the recipe class combo box, the recipe instances table and the recipe elements table
  _unRecipeFunctions_setRecipeInstancePanelComponentsEnabled(true);
  
  _unRecipeFunctions_removeRecipeHeaderBuffers(plcNames);
  bool releaseResult = _unRecipeFunctions_unlockRecipeBuffers(plcNames, exceptionInfo);
  if (releaseResult == false) {
	_unRecipeFunctions_writeInRecipeLog("Error releasing the recipe buffers: " + exceptionInfo[2]);
	// Show the error message
	DebugTN(exceptionInfo);
  }
  
  // Reset the recipe state
  dpSet(sRcpInstanceDp+".ProcessInput.State", "");
  if (iActivationResult<0)  {
	DebugTN(exceptionInfo); 
	return iActivationResult;
  }
    
  //t1 = getCurrentTime();
  //string elapsedTime = formatTime("%S", t1-t0, " milli: %04d");
  
  return RCP_ACT_RETVAL_ACTIVATION_OK;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Remove the recipe header buffers associated to the specified PLC names.
 * @param daPlcNames - [IN] The PLC names which recipe header buffers must be removed.
 */
void _unRecipeFunctions_removeRecipeHeaderBuffers(dyn_anytype daPlcNames) {
  int iLen;
  
  iLen = dynlen(daPlcNames);
  for (int i=1; i<=iLen; i++) {
	mappingRemove(gHeaderMap, daPlcNames[i]);
  }
}
//------------------------------------------------------------------------------------------------------------------------
/**
 * Unlock the recipe buffers used during the recipe activation.
 * @param daPlcNames - [IN] The PLC names which recipe header buffers must be unlocked.
 * @param exceptionInfo - [OUT] Standard exception handling variable. 
 */
bool _unRecipeFunctions_unlockRecipeBuffers(dyn_anytype daPlcNames, dyn_string &exceptionInfo) {
  int iErr;
  int iManNum = myManNum();
  bool bSelect = false;
  dyn_string dsBufferLockedDpName, dsBufferLockedByDpName;
  dyn_string dsLockedValue;
  dyn_int diLockedByValue;

  dynClear(exceptionInfo);
  
  int iLen = dynlen(daPlcNames);
  // Get the recipe buffer DP names for the plcs
  for (int i=1; i<=iLen; i++) {
	string sPlcName = daPlcNames[i];
	string sDpName;
	
	_unRecipeFunctions_getRecipeBuffersName(sPlcName, sDpName, exceptionInfo);
	dynAppend(dsBufferLockedDpName, sDpName + ":_lock._original._locked");
	dynAppend(dsBufferLockedByDpName, sDpName + ":_lock._original._man_nr");
  }
  
  // Get the 'locked' state of the recipe buffers and the manager number that locked the buffers
  iErr = dpGet(dsBufferLockedDpName, dsLockedValue,
			   dsBufferLockedByDpName, diLockedByValue);			   
  if (_unRecipeFunctions_handleDpGetSetError(iErr, exceptionInfo)==true) {
    _unRecipeFunctions_writeInRecipeLog("Failure while getting the lock state of the recipe buffers for the PLCs: '" + daPlcNames + "'", true);
    _unRecipeFunctions_writeInRecipeLog("Recipe buffers cannot be unlocked", true);
    return false;
  }
  
  for (int i=1; i<=iLen; i++) {
    // Check if the recipe buffer is locked by the current manager
    if (dsLockedValue[i] == true) {
      if (diLockedByValue[i] == iManNum) {
        // Release the recipe buffer
	      iErr = dpSetWait(dsBufferLockedDpName[i], 0);
	      bool bError = _unRecipeFunctions_handleDpGetSetError(iErr, exceptionInfo);
        if (bError) {
          _unRecipeFunctions_writeInRecipeLog("Failure while unlocking buffers for PLC: " + daPlcNames[i], true);
        } else {
          _unRecipeFunctions_writeInRecipeLog("Recipe buffers unlocked for PLC: " + daPlcNames[i] , true);
        }
      } else {
        _unRecipeFunctions_writeInRecipeLog("The PLC '" + daPlcNames[i] 
          + "' buffer is locked by a different manager (" + diLockedValue[i] + ") and it cannot be unlocked", true);
      }        
    }
  }
  
  return (dynlen(exceptionInfo)==0);
}
//------------------------------------------------------------------------------------------------------------------------
/**
 * Enable/disable some graphical components of the recipe instance panel (required for the recipe activation).
 * @param value - [IN] The value for the enabled property of the graphical components.
 */
void _unRecipeFunctions_setRecipeInstancePanelComponentsEnabled(bool value) {
  if (shapeExists("RecipeClasses"))
	setValue("RecipeClasses", "enabled", value);
	
  if (shapeExists("RecipeInstances"))
	setValue("RecipeInstances", "enabled", value);
	
  if (shapeExists("RecipeElements"))
	setValue("RecipeElements", "enabled", value);
	
  if (shapeExists("PcoList"))
	setValue("PcoList", "enabled", value);
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Get the recipe elements according to the PLC where the elements are located.
 * @param dsElements - [IN] Recipe data point elements.
 * @param dsValues - [IN] Recipe values.
 * @param plcDpes - [OUT] Map containing the recipe elements and values for each PLC.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */  
void _unRecipeFunctions_getElementsByPLC(dyn_string dsElements, dyn_anytype dsValues, 
                                         mapping &plcDpes, dyn_string &exceptionInfo)
{
  int rc, len, iElements, iBufferLength;
  string sBufferDpname, sPlcName, sSystemName;
  dyn_dyn_string data;
  mappingClear(plcDpes);
  
  int len = dynlen(dsElements);
  if (len!=dynlen(dsValues)) {
    fwException_raise(exceptionInfo, "ERROR", "The number of DPEs and values of the recipe must be equal.", "");
    return;
  }  
  
  // Inserts the DPEs in the mapping according to the PLC name
  for (int i=1; i<=len; i++) {
    sSystemName = unGenericDpFunctions_getSystemName(dsElements[i]);
    sPlcName = unGenericObject_GetPLCNameFromDpName(dsElements[i]);
    
    dynClear(data);
    if (mappingHasKey(plcDpes, sSystemName+sPlcName)) {
      data = plcDpes[sSystemName+sPlcName];
    }

    dynAppend(data[1], dsElements[i]);
    dynAppend(data[2], dsValues[i]);
    plcDpes[sSystemName+sPlcName] = data;
  }    
  
  // Check that the number of elements in the recipe (for each PLC) isn't bigger than the recipe buffers size
  len = mappinglen(plcDpes);
  for (int i=1; i<=len; i++) {
    data = mappingGetValue(plcDpes, i);
	iElements = dynlen(data[1]);
	sPlcName = mappingGetKey(plcDpes, i);
	
	_unRecipeFunctions_getRecipeBuffersName(sPlcName, sBufferDpname, exceptionInfo);
	if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return; }

	rc = dpGet(sBufferDpname + "ProcessInput.BuffersLength", iBufferLength);
	if (iElements > iBufferLength) {
		fwException_raise(exceptionInfo, "ERROR", "The number of recipe elements (" + iElements + ") is bigger than the PLC buffers length (" + iBufferLength + ") for the PLC: " + sPlcName, "");
		DebugTN(exceptionInfo);
		return;
	}
  }
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Check if a recipe can be activated in the specified PLCs.
 * The recipe can't be activated in a PLC if there is other recipe being activated in that PLC (i.e. the recipe buffer is locked).
 * @param dsPlcNames - [IN] List of PLC names.
 * @param dsBusyPlcs - [OUT] List of busy PLCs.  
 * @param exceptionInfo - [OUT] Standard exception handling variable.  
 * @return TRUE if a recipe can be activated in the specified PLCs, otherwise FALSE. 
 */
synchronized bool _unRecipeFunctions_canActivateRecipeInPlcs(string sRcpInstanceDp, dyn_anytype dsPlcNames, dyn_string &dsBusyPlcs, dyn_string &exceptionInfo) {
  dynClear(dsBusyPlcs);
  int iErr;
  int iLen = dynlen(dsPlcNames);
  bool bSelect;
  string sSelectedManager;
  dyn_string dsPlcDpNames, dsPlcIsLockedDp, dsPlcLockedByDp;
  dyn_string dsPlcIsLockedValue, dsLockRequestValue;
  dyn_int diPlcLockedByValue;
  
  // Get the recipe buffer dp names
  for (int i=1; i<=iLen; i++) {
    string sPlcName = dsPlcNames[i];
	string sDpName;
	_unRecipeFunctions_getRecipeBuffersName(sPlcName, sDpName, exceptionInfo);
	dynAppend(dsPlcDpNames, sDpName);
	dynAppend(dsPlcIsLockedDp, sDpName + ":_lock._original._locked");
	dynAppend(dsPlcLockedByDp, sDpName + ":_lock._original._man_nr");
	dynAppend(dsLockRequestValue, "true");
  }
  
  // Get the 'locked' value of the recipe buffers
  iErr = dpGet(dsPlcIsLockedDp, dsPlcIsLockedValue);
  if (_unRecipeFunctions_handleDpGetSetError(iErr, exceptionInfo)==true) {
	return false;
  }
  
  // Check if all the recipe buffers are unlocked
  for (int i=1; i<=iLen; i++) {
	if (dsPlcIsLockedValue[i] == true) {
	  dynAppend(dsBusyPlcs, dsPlcNames[i]);
	}
  }
  
  // If some recipe buffers are busy return
  if (dynlen(dsBusyPlcs))
	return false;
  
  // Try to lock all the required buffers
  iErr = dpSetWait(dsPlcIsLockedDp, dsLockRequestValue);
  if (_unRecipeFunctions_handleDpGetSetError(iErr, exceptionInfo)==true) {
	return false;
  }
  
  // Get the 'locked' value of the recipe buffers
  iErr = dpGet(dsPlcIsLockedDp, dsPlcIsLockedValue,
			   dsPlcLockedByDp, diPlcLockedByValue);
  if (_unRecipeFunctions_handleDpGetSetError(iErr, exceptionInfo)==true) {
	return false;
  }
    
  // Check if all the recipe buffers are locked by the current manager
  for (int i=1; i<=iLen; i++) {
	if (dsPlcIsLockedValue[i] == false || diPlcLockedByValue[i] != myManNum()) {
	  dynAppend(dsBusyPlcs, dsPlcNames[i]);
	}
  }
  
  // If the lock failed in any buffer, unlock the recipe buffers
  if (dynlen(dsBusyPlcs)) {
	_unRecipeFunctions_unlockRecipeBuffers(dsPlcNames);
	return false;
  }
  _unRecipeFunctions_createRecipeActivationInfo(dsPlcNames, sRcpInstanceDp, exceptionInfo);

  return true;
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Handle dpGet/dpSet errors.
 * @param err - [IN] Value returned by dpGet/dpSet
 * @param exceptionInfo - [OUT] Standard exception handling variable.  
 */
bool _unRecipeFunctions_handleDpGetSetError (int err, dyn_string &exceptionInfo) {
  if((err != 0) || (dynlen(getLastError()) > 0)) {
    fwException_raise(exceptionInfo, "ERROR",
                      "Failed to get/set value from/to dpe (dpGet/dpSetWait).",
                      getLastError());
    return true;
  }else
    return false;
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the time when the recipe activation must be cancelled (if the activation wasn't completed).
 * @param plcName - [IN] The list of PLC names where the recipe will be activated.
 * @param tExpirationTime - [OUT] The time when the recipe activation must be cancelled.
 * @param iTimeoutSec - [OUT] Timeout value in seconds. 
 * @param exceptionInfo - [OUT] Standard exception handling variable.  
 */
void unRecipeFunctions_getRecipeActivationTimeout(dyn_anytype plcNames, time &tExpirationTime, int &iTimeoutSec, dyn_string & exceptionInfo) {

  int rc, iLen;
  dyn_int timeouts;
  time tStart;
  string sBuffersAlias, sBuffersName;
  
  iLen = dynlen(plcNames);
  for (int i=1; i<=iLen; i++) {  
	  // Get the recipe buffers name for the current PLC
	  _unRecipeFunctions_getRecipeBuffersName(plcNames[i], sBuffersAlias, exceptionInfo);  
	  if (dynlen(exceptionInfo))  { DebugTN(exceptionInfo); return; }
	  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(sBuffersAlias, sBuffersName);

	  // Get the activation timeout from the recipe buffers
	  rc = dpGet(sBuffersName+"ProcessInput.ActivationTimeout", iTimeoutSec);
	  dynAppend(timeouts, iTimeoutSec);
  }	  

  // The activation timeout is the minimum timeout of all the PLCs
  iTimeoutSec = dynMax(timeouts);
  
  // Activation timeout
  tStart = getCurrentTime();
  tExpirationTime   = makeTime((unsigned)year(tStart), 
                    (unsigned)month(tStart), 
                    (unsigned)day(tStart), 
                    (unsigned)hour(tStart), 
                    (unsigned)minute(tStart), 
                    (unsigned)second(tStart)+(unsigned)iTimeoutSec);
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Triggers the recipe activation in all the PLCs containing recipe elements.
 * @param plcDpes - [IN] DPE name of the PLCs where the recipe must be activated.
 * @param sRcpInstanceDp - [IN] DPE name of the recipe instance.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */ 
int _unRecipeFunctions_activateRecipeByPLC(mapping plcDpes, string sRcpInstanceDp, dyn_string &exceptionInfo)
{  
  bool bIsRecipeActivated;
  int iLen, iTimeoutSec, iProgressBarValue, iActivationResult;
  dyn_string sRecipeBufferDp;
  time tExpirationTime;
  
  unRecipeFunctions_getRecipeActivationTimeout(mappingKeys(plcDpes), tExpirationTime, iTimeoutSec, exceptionInfo);
  _unRecipeFunctions_createRecipeActivationInfo(mappingKeys(plcDpes), sRcpInstanceDp, exceptionInfo);
  
  // Initializes the recipe activation progress bar
  if (shapeExists("ActivationProgressBar")) { 
    unProgressBar_setMax("ActivationProgressBar", iTimeoutSec);
	unProgressBar_setPosition("ActivationProgressBar", iTimeoutSec);
  }  
  
  // Start the recipe activation in all the PLCs (sending the recipe header)
  iLen = mappinglen(plcDpes);
  for (int i=1; i<=iLen; i++) {
    string sPlcName = mappingGetKey(plcDpes, i);
    string sPlcType = dpTypeName(sPlcName); 
    dyn_dyn_string recipeData = mappingGetValue(plcDpes, i);  
    
    if (sPlcType==UN_PLC_DPTYPE || sPlcType==S7_PLC_DPTYPE){
      bool bActivationInitOk = _unRecipeFunctions_activateRecipe(sPlcName, sPlcType, sRcpInstanceDp, recipeData[1], recipeData[2], tExpirationTime, exceptionInfo);
      if (!bActivationInitOk) {
        unRecipeFunctions_cancelRecipeActivation(sRcpInstanceDp, exceptionInfo, exceptionInfo[1]);
        return RCP_ACT_RETVAL_ACTIVATION_ERROR;
      }
    } else {
      // Unknown PLC
      fwException_raise(exceptionInfo, "ERROR", "The PLC type is unknown for the PLC:" + plcName, "");
	  //_unRecipeFunctions_cancelRecipeActivationMultiPlc(mappingKeys(plcDpes), "Timeout");
	  unRecipeFunctions_cancelRecipeActivation(sRcpInstanceDp, exceptionInfo);
      return RCP_ACT_RETVAL_BAD_PLC_TYPE;      
    }
  }
  
  iActivationResult = _unRecipeFunctions_waitUntilActivationIsCompleted(sRcpInstanceDp, tExpirationTime, exceptionInfo);
  if (iActivationResult==RCP_ACT_RETVAL_ACTIVATION_TIMEOUT) {
	// Recipe activation timeout: Cancel the recipe activation in all the PLCs
	unRecipeFunctions_cancelRecipeActivation(sRcpInstanceDp, exceptionInfo);
	return RCP_ACT_RETVAL_ACTIVATION_TIMEOUT;
  }
  return iActivationResult;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Creates a matrix containing the necessary information to activate a recipe and stores it in the map unRecipeActivationInfo.
 *	 Row 1: List of PLCS containing recipe elements.
 *	 Row 2: List containing the names of the PLCs that are ready to activate the recipe.
 *	 Row 3: List of PLC names where the recipe has been activated.
 */
synchronized void _unRecipeFunctions_createRecipeActivationInfo(dyn_anytype plcNames, string sRcpInstanceDp, dyn_string &exceptionInfo) {
	dyn_dyn_anytype recipeActivationInfo;
	dyn_string plcsReadyToActivate;
	dyn_string recipeAcivatedInPLCs;
	
	dynAppend(recipeActivationInfo, plcNames);
	dynAppend(recipeActivationInfo, plcsReadyToActivate);
	dynAppend(recipeActivationInfo, recipeAcivatedInPLCs);
	unRecipe_recipeActivationInfo[sRcpInstanceDp] = recipeActivationInfo;
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Increment the number of PLCs ready to activate the recipe. When all the PLCs are ready the recipe activation is triggered in all of them.
 * @param sPlcName - [IN] The name of the PLC ready to activate the recipe.
 * @param sRcpInstanceDp - [IN] The data point of the recipe instance to be activated.
 * @return TRUE if all the PLCs containing recipe elements are ready to activate, otherwise FALSE.
 */
synchronized bool _unRecipeFunctions_incrementNumPlcsReadyToActivate(string sPlcName, string sRcpInstanceDp) {
	return _unRecipeFunctions_addPlcToRecipeActivationInfo(PLCS_READY_TO_ACTIVATE_INDEX, sPlcName, sRcpInstanceDp);
}
//------------------------------------------------------------------------------------------------------------------------
/**
 * Increment the number of PLCs ready to activate the recipe. When all the PLCs are ready the recipe activation is triggered in all of them.
 * @param sPlcName - [IN] The name of the PLC where the recipe has been activated.
 * @param sRcpInstanceDp - [IN] The data point of the recipe instance activated in the PLC.
 * @return TRUE if the recipe has been activated in all the PLCs containing recipe elements, otherwise FALSE.
 */
synchronized bool _unRecipeFunctions_incrementNumPlcsActivatedRecipe(string sPlcName, string sRcpInstanceDp) {
	return _unRecipeFunctions_addPlcToRecipeActivationInfo(PLCS_ACTIVATED_RECIPE_INDEX, sPlcName, sRcpInstanceDp);
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Adds the PLC name to the unRecipe_recipeActivationInfo in the index specified as a parameter.
 * @param index - [IN] Index where to add the PLC name in the recipe activation info matrix.
 * @param sPlcName - [IN] The PLC name to add in the recipe activation info matrix.
 * @param sRcpInstanceDP - [IN] The data point of the recipe instance.
 * @return TRUE if the number of PLCs added in the matrix row is equal to the number of PLCs containing recipe elements, otherwise FALSE.
 */
synchronized bool _unRecipeFunctions_addPlcToRecipeActivationInfo(int index, string sPlcName, string sRcpInstanceDp) {
	dyn_dyn_anytype recipeActivationInfo;
	dyn_string plcs;
	int iPlcsReady, iPlcsNumber;
	
	recipeActivationInfo = unRecipe_recipeActivationInfo[sRcpInstanceDp];
	plcs = recipeActivationInfo[index];
	if (dynContains(plcs, sPlcName))
		return false;
		
	dynAppend(plcs, sPlcName);
	
	iPlcsReady = dynlen(plcs);
	recipeActivationInfo[index] = plcs;
	unRecipe_recipeActivationInfo[sRcpInstanceDp] = recipeActivationInfo;
	iPlcsNumber = dynlen(recipeActivationInfo[PLCS_REQUIRED_FOR_ACTIVATION_INDEX]);
	
	if (iPlcsNumber == iPlcsReady)
		return true;
	return false;
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Wait until the recipe activation is completed or the timeout expires.
 * @param sRcpInstanceDp - [IN] DPE name of the recipe instance.
 * @param tEnd - [IN] Expiration time for the recipe activation.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
int _unRecipeFunctions_waitUntilActivationIsCompleted(string sRcpInstanceDp, time tEnd, dyn_string & exceptionInfo) {

  bool bActivationProgressBarExists;
  int iSleepTime=1, iProgressBarValue=0;
  string sRecipeState, sRemainingTime;
  time tCurrent;
  
  bActivationProgressBarExists = shapeExists("ActivationProgressBar");
  if (bActivationProgressBarExists) {
    getValue("ActivationProgressBar", "progress", iProgressBarValue);
  }
  
  // Loop until the recipe activation is finished or the timeout has expired  
  while ((tCurrent = getCurrentTime()) < tEnd) {
    dpGet(sRcpInstanceDp+".ProcessInput.State", sRecipeState);
    if (sRecipeState!=UN_RCP_INSTANCE_STATE_ACTIVATE) {
      // Recipe activation finished
      break;
    }

    delay(iSleepTime);
    // Updates the Activation Timeout progress bar
    if (bActivationProgressBarExists) {	
      iProgressBarValue = iProgressBarValue - iSleepTime;
      unProgressBar_setPosition("ActivationProgressBar", iProgressBarValue);
    }	
	
    // if the selection timeout is less than the activation timeout, selects the recipe instance
    dpGet(getSystemName() + "_unSelectDeselect.remainingTime", sRemainingTime);
    if (iProgressBarValue > (int)(strsplit(sRemainingTime, ";")[3])) {
      unGenericObject_ButtonSelectAction($sDpName, true, exceptionInfo);
    }
  } 
  
  // Resets the progress bar
  if (bActivationProgressBarExists) {
	unProgressBar_setPosition("ActivationProgressBar", 0);
  }
  
  // Check if the activation timeout has expired
  if (tCurrent >= tEnd && sRecipeState==UN_RCP_INSTANCE_STATE_ACTIVATE) {
    return RCP_ACT_RETVAL_ACTIVATION_TIMEOUT;
  } 
  
  if (sRecipeState == UN_RCP_INSTANCE_STATE_CANCELLED) {
    return RCP_ACT_RETVAL_PLC_CANCELLATION;
  }
 
  return RCP_ACT_RETVAL_ACTIVATION_OK;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Activates a recipe in a PLC.
 * @param plcName - [IN] PLC name.
 * @param plcType - [IN] PLC type.
 * @param sRcpInstanceDp - [IN] DPE name of the recipe instance.
 * @param dpes - [IN] List of recipe DPEs for the PLC.
 * @param values - [IN] List of values for the recipe DPEs.
 * @param tEnd - [IN] Expiration time for the recipe activation.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 * @return TRUE if the recipe activation was initiated correctly, otherwise FALSE.
 */
bool _unRecipeFunctions_activateRecipe(string plcName, string plcType, string sRcpInstanceDp, dyn_string dpes, 
                                              dyn_string values, time tEnd, dyn_string &exceptionInfo)
{
  string sSystemName, sRecipeState, dpBuffersAlias, dpBuffersName, sRemainingTime, sPlcType;
  bool bIsRemote, bIsConnected, bActivationProgressBarExists, bSelectionProgressBarExists;
  time tStart, tCurrent;
  int rc, iActivationTimeout, iProgressBarValue=0, iSleepTime=1, iTimeOut;
  
  // Check if the current PLC is in a distributed system and if it's connected
  sSystemName = unGenericDpFunctions_getSystemName(plcName);
  unDistributedControl_isRemote(bIsRemote, sSystemName);
  if (bIsRemote) {
	unDistributedControl_isConnected(bIsConnected, sSystemName);
	if (!bIsConnected){
    string sError = "The distributed system '" + sSystemName + "' isn't connected.";
		fwException_raise(exceptionInfo, "ERROR", sError, "");
		return false;
	}
  }
  
  // Get the recipe buffers name for the current PLC
  _unRecipeFunctions_getRecipeBuffersName(plcName, dpBuffersAlias, exceptionInfo);  
  if (dynlen(exceptionInfo)) { 
    DebugTN(exceptionInfo); 
    return false; 
  }
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(dpBuffersAlias, dpBuffersName);			
  
  // if the selection timeout is less than the activation timeout, selects the recipe instance
  dpGet(getSystemName() + "_unSelectDeselect.remainingTime", sRemainingTime,
        getSystemName() + "_unSelectDeselect.timeOut", iTimeOut);
  if (iActivationTimeout > (int)(strsplit(sRemainingTime, ";")[3])) {
	unGenericObject_ButtonSelectAction($sDpName, true, exceptionInfo);
	if (shapeExists("SelectionProgressBar")) {
		unProgressBar_setPosition("SelectionProgressBar", iTimeOut);
	}
  }
 
  _unRecipeFunctions_createBuffersData(plcName, dpes, values, exceptionInfo);
  if (dynlen(exceptionInfo)) {
    DebugTN(exceptionInfo); 
    return false;
  }
  
  if (plcType==S7_PLC_DPTYPE) {
	// If the PLC is SIEMENS creates the recipe header DB numbers
	_unRecipeFunctions_createRecipeHeaderDBNumber(plcName, dpes, exceptionInfo); 
	if (dynlen(exceptionInfo)) {
    DebugTN(exceptionInfo); 
    return false;
  }  
  }
  
  _unRecipeFunctions_writeInRecipeLog("Sending recipe data to the PLC " + plcName);
  _unRecipeFunctions_sendRecipeHeader(plcName, exceptionInfo);

  dpConnect("_unRecipeFunctions_checkRecipeStatusBufferCB", false,
				dpBuffersName+"ProcessInput.Status.RecipeStatus", 
				sRcpInstanceDp+".ProcessInput.State");
  
  return true;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Send the recipe header buffer to the PLC.
 * @param sPlcName - [IN] PLC Name.
 * @param exceptionInfo - [OUT] Standard error handling variable.
 */
void _unRecipeFunctions_sendRecipeHeader(string sPlcName, dyn_string &exceptionInfo)
{
  int iLen;
  string sDpName, sSystemName;
  dyn_string recipeHeaderDpes;
  dyn_int recipeHeaderValues;
  
  _unRecipeFunctions_getRecipeBuffersName(sPlcName, sDpName, exceptionInfo);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_sendRecipeHeader", "_unRecipeFunctions_sendRecipeHeader", "Function START");
  if (strpos(sPlcName, ":") < 0) {
	// Adds the system name to the PLC name
	sSystemName = unGenericDpFunctions_getSystemName(sPlcName);
	sPlcName = sSystemName + sPlcName;
  }
  
  // Get the recipe header values
  recipeHeaderValues = _unRecipeFunctions_getHeaderBuffer(sPlcName);
  
  // Creates the strings to represent the recipe header dpes
  iLen = dynlen(recipeHeaderValues);
  for (int i=1; i<=iLen; i++) {
	recipeHeaderDpes[i] = sDpName + "ProcessOutput.Header[" + i + "]";
  }
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_sendRecipeHeader", "_unRecipeFunctions_sendRecipeHeader", 
		"PLCName: " + sPlcName, "RcpHeaderDPEs: " + dynlen(recipeHeaderDpes), 
		"RcpHeaderValues: " + dynlen(_unRecipeFunctions_getHeaderBuffer(sPlcName)),
		"Recipe Header DPEs: " + recipeHeaderDpes,
		"RcpHeader: " + _unRecipeFunctions_getHeaderBuffer(sPlcName));
  dpSetWait(recipeHeaderDpes, recipeHeaderValues);
 
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_sendRecipeHeader", "_unRecipeFunctions_sendRecipeHeader", "Function END");
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Callback function executed during the recipe activation when the recipe status has changed.
 * @param sRecipeStatusDp - [IN] DP Name of the recipe buffer status.
 * @param recipeStatus - [IN] Value of the recipe buffer status.
 * @param sRecipeState - [IN] DP Name of the recipe state.
 * @param sState - [IN] Value of the recipe state.
 */
void _unRecipeFunctions_checkRecipeStatusBufferCB(string sRecipeStatusDp, int recipeStatus,
												  string sRecipeState, string sState) 
{
  int pos;
  bool checkResult, bPlcsReadyToActivate;
  string sBuffersDpName, plcName, sActivationStatus, errDpe, pvssDpe, sRcpInstanceDp;
  dyn_int recipeHeaderBuffer;
  dyn_string exceptionInfo;
  dyn_dyn_mixed recipeObject; 
    	
  // Get the PLC name
  plcName = unGenericObject_GetPLCNameFromDpName(sRecipeStatusDp);
  if (strpos(plcName, ":")<0)
	plcName = unGenericDpFunctions_getSystemName(sRecipeStatusDp) + plcName;
    
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_checkRecipeStatusBufferCB", "_unRecipeFunctions_checkRecipeStatusBufferCB", 
		"PLC: " + plcName, "sRcpStatusDp: " + sRecipeStatusDp, "sRecipeState: " + sRecipeState);
  
  // Get the PLC buffers dp name
  pos = strpos(sRecipeStatusDp, "ProcessInput");
  if (pos>=0) {
	sBuffersDpName = substr(sRecipeStatusDp, 0, pos);
  }
  
  // Get the recipe timestamp from the header and status buffers
  int headerTimeLow, headerTimeHigh, statusTimeLow, statusTimeHigh;
  dpGet(sBuffersDpName + "ProcessOutput.Header[1]:_online.._value", headerTimeLow,
		sBuffersDpName + "ProcessOutput.Header[2]:_online.._value", headerTimeHigh,
		sBuffersDpName + "ProcessInput.Status[1]:_online.._value", statusTimeLow,
		sBuffersDpName + "ProcessInput.Status[2]:_online.._value", statusTimeHigh);
  
  if (headerTimeLow != statusTimeLow || headerTimeHigh != statusTimeHigh){
    // If the timestamp are not equal means that the callback function is called 
	// after a TSPP update and the status buffer doesn't correspond to the current
	// recipe activation. Ignore.
	return;
  }
  
  // Get the recipe DP name
  pos = strpos(sRecipeState, ".");
  if (pos>=0) {
	sRcpInstanceDp = substr(sRecipeState, 0, pos);
  }
	    
  switch (recipeStatus){
    case RCP_ACT_STATUS_ERROR:
	  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_checkRecipeStatusBufferCB", 
			"_unRecipeFunctions_checkRecipeStatusBufferCB", 
			"Recipe activation - error");
	  _unRecipeFunctions_writeInRecipeLog("Recipe activation error in PLC " + plcName, TRUE);
      dpDisconnect("_unRecipeFunctions_checkRecipeStatusBufferCB", sRecipeStatusDp, sRecipeState);
	  strreplace(sRecipeState, ":_online.._value", "");
	  dpSet(sRecipeState, UN_RCP_INSTANCE_STATE_CANCELLED);
	  unRecipeFunctions_setRecipeStatusInfo(RCP_ACTIVATION_FAILED);
      break;
  
    case RCP_ACT_STATUS_STANDBY:
	  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_checkRecipeStatusBufferCB", 
			"_unRecipeFunctions_checkRecipeStatusBufferCB", "Recipe activation - in standby");
      break;
      
    case RCP_ACT_STATUS_NEW_ID_RECEIVED:
	  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_checkRecipeStatusBufferCB", 
			"_unRecipeFunctions_checkRecipeStatusBufferCB", "Recipe activation - New Id received");
	  checkResult = _unRecipeFunctions_checkRecipeStatusBufferData(sBuffersDpName, exceptionInfo);  
    if (false==checkResult) {
		// Wrong data in the recipe status buffer
		_unRecipeFunctions_writeInRecipeLog("Activation error in PLC " + plcName + ": " + exceptionInfo[2]);
		unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_checkRecipeStatusBufferCB", 
			"_unRecipeFunctions_checkRecipeStatusBufferCB", "The data in the status buffer is Wrong: ", exceptionInfo);
		unRecipeFunctions_cancelRecipeActivation(sRcpInstanceDp, exceptionInfo, "Bad status data");
	  } else {
		// The data in the recipe status buffer is Ok
		// 1. Send the recipe data (ManRegAddr, ManRegVal, ReqAddr and ReqVal buffers)
		_unRecipeFunctions_sendData(plcName, exceptionInfo);
		// 2. Send the recipe header buffer
		recipeHeaderBuffer = _unRecipeFunctions_getHeaderBuffer(plcName);
		recipeHeaderBuffer[RCP_HEADER_SENT_VALUE] = RCP_ACT_HEADER_DATA_SENT;
		gHeaderMap[plcName] = recipeHeaderBuffer;
		_unRecipeFunctions_sendRecipeHeader(plcName, exceptionInfo);
	  }
      break;

    case RCP_ACT_STATUS_CRC_OK:
	  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_checkRecipeStatusBufferCB", 
			"_unRecipeFunctions_checkRecipeStatusBufferCB", "Recipe activation - CRC Ok");
	  checkResult = _unRecipeFunctions_checkRecipeStatusBufferData(sBuffersDpName, exceptionInfo);
	  if (false==checkResult) {
	    // Wrong data in the recipe status buffer
		_unRecipeFunctions_writeInRecipeLog("Activation error in PLC " + plcName + ": " + exceptionInfo[2]);
	    unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_checkRecipeStatusBufferCB", 
			"_unRecipeFunctions_checkRecipeStatusBufferCB", "The data in the status buffer is Wrong: ", exceptionInfo);
		unRecipeFunctions_cancelRecipeActivation(sRcpInstanceDp, exceptionInfo, "Bad status data");
	  } else {
		// Check if all the PLCs are ready to activate the recipe
		bPlcsReadyToActivate = _unRecipeFunctions_incrementNumPlcsReadyToActivate(plcName, sRcpInstanceDp);
		if (bPlcsReadyToActivate) {
			// If all the PLCs are ready to activate the recipe, send the activation order
			dyn_anytype allPlcNames = unRecipe_recipeActivationInfo[sRcpInstanceDp][1];
			for (int i=1; i<=dynlen(allPlcNames); i++) {
				// Send the activation order 
				string sCurrPlcName = allPlcNames[i];
				recipeHeaderBuffer = _unRecipeFunctions_getHeaderBuffer(sCurrPlcName);
				recipeHeaderBuffer[RCP_HEADER_SENT_VALUE] = RCP_ACT_HEADER_ACTIVATE;
				gHeaderMap[sCurrPlcName] = recipeHeaderBuffer;
				_unRecipeFunctions_sendRecipeHeader(sCurrPlcName, exceptionInfo);  
			}
		}
	  }
      break;      

    case RCP_ACT_STATUS_DONE:
	  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_checkRecipeStatusBufferCB", 
			"_unRecipeFunctions_checkRecipeStatusBufferCB", "Recipe activation DONE");
      dpDisconnect("_unRecipeFunctions_checkRecipeStatusBufferCB", sRecipeStatusDp, sRecipeState);
	  
	  // Is enough this delay?
	  delay(1);
	  
	  // Check if the recipe has been activated properly
	  _unRecipeFunctions_getRecipeObjectFromDp(sRecipeState, recipeObject, exceptionInfo);
	  if (dynlen(exceptionInfo)) {DebugTN(exceptionInfo); return;}
	  dyn_string rcpElements = recipeObject[fwConfigurationDB_RO_DPE_NAME];
	  dyn_string rcpValues   = recipeObject[fwConfigurationDB_RO_VALUE];
	  
	  if (_unRecipeFunctions_checkRecipeOnlineValues(rcpElements, rcpValues, errDpe)) {
		// Updates the recipe meta info
		recipeObject[fwConfigurationDB_RO_META_LASTACTIVATIONTIME] = getCurrentTime();
		recipeObject[fwConfigurationDB_RO_META_LASTACTIVATIONUSER] = getUserName();
		_fwConfigurationDB_activateRecipeUpdateMetaInfo(recipeObject, exceptionInfo);
		string sRcpDp = sRecipeState;
		strreplace(sRcpDp, ".ProcessInput.State:_online.._value", "");
	    _unRecipeFunctions_loadPropertiesTableInfo(sRcpDp, recipeObject);
	    _unRecipeFunctions_loadHistoryTableInfo(sRcpDp, recipeObject);
	    _unRecipeFunctions_writeInRecipeLog("Recipe activation completed in PLC: " + plcName, TRUE);
		unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_checkRecipeStatusBufferCB", 
			"_unRecipeFunctions_checkRecipeStatusBufferCB", "Recipe activation completed in PLC: " + plcName);
	  } else {
	    unGenericDpFunctions_convert_PVSSDPE_to_UNICOSDPE(errDpe, errDpe);
		unRecipeFunctions_setRecipeStatusInfo(RCP_ACTIVATION_FAILED);
	    _unRecipeFunctions_writeInRecipeLog("Recipe activation failed in PLC " + plcName + ": Recipe read back values are different than sent values");		
		_unRecipeFunctions_writeInRecipeLog("The first different value corresponds to the data point: " + errDpe);
		if (patternMatch("*.MOutH", errDpe) || patternMatch("*.MOutL", errDpe)) {
		  _unRecipeFunctions_writeInRecipeLog("Please, verify that the recipe value is defined in the controlled object range.");
		} 
		else if (patternMatch("*.MSPH", errDpe) || patternMatch("*.MSPL", errDpe)) {
		  _unRecipeFunctions_writeInRecipeLog("Please, verify that the recipe value is defined in the measured value range.");
		}
	  }
	  
	  // Check if the activation is completed in all the PLCs
	  if (_unRecipeFunctions_incrementNumPlcsActivatedRecipe(plcName, sRcpInstanceDp)) {
		// Resets the recipe instance state
		strreplace(sRecipeState, ":_online.._value", "");
		dpSet(sRecipeState, UN_RCP_INSTANCE_STATE_UNLOCK_BUFFERS);
		_unRecipeFunctions_writeInRecipeLog("Unlocking recipe buffers...", true);
	  }

	  // Remove the recipe buffers for the PLC
	  _unRecipeFunctions_removeRecipeBuffers(plcName);
      break;
	  
	default:
	  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_checkRecipeStatusBufferCB", 
			"_unRecipeFunctions_checkRecipeStatusBufferCB", "Recipe activation - ERROR CODE RECEIVED: " + recipeStatus);
	  _unRecipeFunctions_writeInRecipeLog("Recipe activation error: Unknown recipe status value (" + recipeStatus + ")", TRUE);
  }
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Checks the data in the recipe status buffer to verify:
 * <ul>
 * 	<li>The received data corresponds to the data in the recipe header buffer.</li>
 *	<li>The timestamp received is synchronized with the PLC: PVSS Time +/- PLC Delay = received timestamp
 * </ul>
 * @param sBuffersDpName - [IN] DpName of the recipe buffers.
 * @param exceptionInfo  - [OUT] Standard exception handling variable.
 */
bool _unRecipeFunctions_checkRecipeStatusBufferData(string sBuffersDpName, dyn_string &exceptionInfo)
{
	string sPlcName, sPlcType, sDelayDpName;
	time timeMin, timeMax, currentTime = getCurrentTime();
	dyn_string timestampDpes, statusValueDpes, headerValueDpes;
	dyn_string statusValues, headerValues, sDelay;
	dyn_time timestamps;
	
	// Creates strings containing the DPEs for: 
	//	 - Timestamp of the status buffer data
	//   - Status buffer data
	//   - Header buffer data
	int i=1;
	while (i<=RCP_STATUS_MAX_SIZE && i<=RCP_STATUS_MAX_SIZE) {
		timestampDpes[i]   = sBuffersDpName + "ProcessInput.Status[" + i + "]:_online.._stime";
		statusValueDpes[i] = sBuffersDpName + "ProcessInput.Status[" + i + "]:_online.._value";
		headerValueDpes[i] = sBuffersDpName + "ProcessOutput.Header[" + i + "]:_online.._value";
		i++;
	}
	
	// Get the PLC Delay DPName
	string sPlcName = unGenericObject_GetPLCNameFromDpName(sBuffersDpName);
	sPlcType = dpTypeName(sPlcName);
	if (sPlcType==S7_PLC_DPTYPE) {
		sDelayDpName = "unicosS7PLC_systemIntegrityInfo.config.data";
	} else if (sPlcType==UN_PLC_DPTYPE) {
		sDelayDpName = "unicosPLC_systemIntegrityInfo.config.data";
	} else {
		// Unknown PLC type
		fwException_raise(exceptionInfo, "ERROR", "The PLC type is unknown: " + sPlcType +".", "");
		return false;
	}

	// Get all the necessary values
	delay(2);
	int rc = dpGet(timestampDpes, timestamps, statusValueDpes, statusValues, headerValueDpes, headerValues, sDelayDpName, sDelay);
	if (rc<0) {
		fwException_raise(exceptionInfo, "ERROR", "DpGet failed in _unRecipeFunctions_checkRecipeStatusBufferData() function.", "");
		return false;	
	}
	
	// Check that the received timestamp is in a valid range
	timeMin = makeTime((unsigned)year(currentTime), 
                       (unsigned)month(currentTime), 
                       (unsigned)day(currentTime), 
                       (unsigned)hour(currentTime), 
                       (unsigned)minute(currentTime), 
                       (unsigned)second(currentTime)-(unsigned)sDelay[1]);
	timeMax = makeTime((unsigned)year(currentTime), 
                       (unsigned)month(currentTime), 
                       (unsigned)day(currentTime), 
                       (unsigned)hour(currentTime), 
                       (unsigned)minute(currentTime), 
                       (unsigned)second(currentTime)+(unsigned)sDelay[1]);
	
	if (timestamps[RCP_STATUS_STATUS]<timeMin || timestamps[RCP_STATUS_STATUS]>timeMax){
		fwException_raise(exceptionInfo, "ERROR", "The timestamp of the received data is out of range.", "");
		return false;
	}
		
	// Check that all the 'mirrored' values corresponds to the values in the recipe header buffer
	for (int i=1; i<=RCP_STATUS_MAX_SIZE && i<=RCP_STATUS_MIRROR_DATA_SIZE; i++) {
		if (headerValues[i]!=statusValues[i]) {
			fwException_raise(exceptionInfo, "ERROR", "The status buffer values are different than the header buffer values.", "");
			return false;
		}
	}

	return true;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Cancel the activation of a recipe.
 * @param sPlcName - [IN] Name of the PLC where the recipe is being activated.
 * @param sBuffersDpName - [IN] DP Name of the recipe buffers for the specified PLC.
 * @param sRecipeState
 */
void _unRecipeFunctions_cancelRecipeActivation (string sPlcName, string sBuffersDpName, string sRecipeState, string reason, bool disconnect=true) 
{
	dyn_string exceptionInfo;
	unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_cancelRecipeActivation", 
			"_unRecipeFunctions_cancelRecipeActivation", 
			"Cancelling recipe activation in PLC: " + sPlcName,
			"BuffersDpName: " + sBuffersDpName,
			"sRecipeState: " + sRecipeState);
	
	if (strpos(sPlcName, ":") < 0) {
		// Adds the system name to the PLC name
		sSystemName = unGenericDpFunctions_getSystemName(sPlcName);
		sPlcName = sSystemName + sPlcName;
    }
  
	dyn_int headerBuffer = _unRecipeFunctions_getHeaderBuffer(sPlcName);
	
	if (disconnect) {
		dpDisconnect("_unRecipeFunctions_checkRecipeStatusBufferCB", 
						sBuffersDpName+"ProcessInput.Status.RecipeStatus", 
						sRecipeState);  
	}

	headerBuffer[RCP_HEADER_SENT_VALUE] = RCP_ACT_HEADER_CANCEL;
	gHeaderMap[sPlcName] = headerBuffer;
	_unRecipeFunctions_sendRecipeHeader(sPlcName, exceptionInfo);
	dpSet(sRecipeState, "");
	unRecipeFunctions_setRecipeStatusInfo(RCP_ACTIVATION_FAILED);
		
	_unRecipeFunctions_writeInRecipeLog("Recipe activation cancelled in PLC " + sPlcName + ": " + reason);
  
    // Remove the recipe buffers for the PLC (To unlock the activation of more recipes in the PLC)
	_unRecipeFunctions_removeRecipeBuffers(sPlcName);
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Cancel the activation of a recipe.
 * @param sRecipeDpName - [IN] Datapoint name of the recipe which cancellation is requested.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 * @param sReason 		- [IN] String containing the reason of the cancellation.
 */
synchronized void unRecipeFunctions_cancelRecipeActivation(string sRecipeDpName, dyn_string &exceptionInfo, string sReason="") 
{
	int iLen;
	string sRcpBuffersDp, sPlcName, sRecipeState;
	// If the sRecipeDpName ends with '.', remove the dot character
	if (sRecipeDpName[strlen(sRecipeDpName)-1] == ".") {
		sRecipeDpName = substr(sRecipeDpName, 0, strlen(sRecipeDpName)-1);
	}
	// Check if the recipe is being activated, if not return.
	if (!mappingHasKey(unRecipe_recipeActivationInfo, sRecipeDpName))
        return;
	
  dpSet(sRecipeDpName+".ProcessInput.State", UN_RCP_INSTANCE_STATE_CANCELLED);
  
	dyn_dyn_anytype recipeActivationInfo = unRecipe_recipeActivationInfo[sRecipeDpName];
	dyn_anytype plcNames = recipeActivationInfo[1];
	iLen = dynlen(plcNames);
	for (int i=1; i<=iLen; i++) {
		sPlcName = plcNames[i];
		_unRecipeFunctions_getRecipeBuffersName(sPlcName, sRcpBuffersDp, exceptionInfo);
		sRecipeState = sRecipeDpName + ".ProcessInput.State";
		dpDisconnect("_unRecipeFunctions_checkRecipeStatusBufferCB",
			sRcpBuffersDp+"ProcessInput.Status.RecipeStatus", 
			sRecipeDpName+".ProcessInput.State");
		_unRecipeFunctions_cancelRecipeActivation (sPlcName, sRcpBuffersDp, sRecipeState, sReason, false);
	}
	_unRecipeFunctions_unlockRecipeBuffers(plcNames, exceptionInfo);
	_unRecipeFunctions_removeRecipeHeaderBuffers(plcNames);
}

//------------------------------------------------------------------------------------------------------------------------
// TODO: Deal with the errors (in exceptionInfo variable)
void _unRecipeFunctions_createBuffersData(string sPlcName, dyn_string dpes, dyn_string values, dyn_string &exceptionInfo)
{
  dyn_int dparManRegAddr, wparManRegAddr, aparManRegAddr, aalarmManRegAddr, pidManRegAddr;
  dyn_int dparManRegVal,  wparManRegVal,  aparManRegVal,  aalarmManRegVal,  pidManRegVal;
  dyn_int dparReqAddr, wparReqAddr, aparReqAddr, aalarmReqAddr, pidReqAddr;
  dyn_float dparReqVal,  wparReqVal,  aparReqVal,  aalarmReqVal,  pidReqVal;
  mapping controllerMap, aalarmMap;
  dyn_dyn_string list;
  int manRegAddr, manRegVal, manRegValAct, reqAddr, reqVal;
  float fval;
  int rc, ival, pos, numDPar=0, numWPar=0, numAPar=0, numAAlarm=0, numPid=0;  
  bool bval, bOk;
  time t0, t1;
  
  t0 = getCurrentTime();
  
  if (strpos(sPlcName, ":") < 0) {
	// Adds the system name to the PLC name
	sSystemName = unGenericDpFunctions_getSystemName(sPlcName);
	sPlcName = sSystemName + sPlcName;
  }
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_createBuffersData", 
			"_unRecipeFunctions_createBuffersData", "Function START", "Values: " + values);
			
  int len = dynlen(dpes);
  for (int i=1; i<=len; i++){    
    // Get the DPE device type name
    string dpe = dpes[i];
    string deviceType = dpTypeName(dpe);
    string deviceName = unGenericDpFunctions_getDpName(dpe);
    string dpeName = substr(dpe, strlen(deviceName), strlen(dpe)-strlen(deviceName));

    unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_createBuffersData", 
			"_unRecipeFunctions_createBuffersData", "DPE: " + dpe, "DeviceType: " + deviceType,
			"DeviceName: " + deviceName, "DpName: " + dpeName );
			
    // Get the DPE address
    string addressType, address, temp;
    _unRecipeFunctions_getDpeAddress(dpe, deviceType, address, exceptionInfo);
    if (dynlen(exceptionInfo)) return;
	
    switch (deviceType){
      case "CPC_DigitalParameter":    
        numDPar++;    
        // Get the ManReg01 address
        _unRecipeFunctions_getDpeAddress(deviceName+".ProcessOutput.ManReg01", deviceType, manRegAddr, exceptionInfo);
		if (dynlen(exceptionInfo)) return;
		//Debug("ManRegAddr of : " + deviceName +".ProcessOutput.ManReg01 : " + manRegAddr +"\n");
        // Set the new ManReg01 values 
        bval = (strtoupper(values[i])=="TRUE")? true : false;    
		//DebugTN ("DPAR VALUE: " + bval + "   values[i] : " + values[i]);
		// Set the ManReg values for the ArmRcp
        manRegVal=0;
		_unRecipeFunctions_setBit(manRegVal, DPAR_ARM_BIT, true);
		_unRecipeFunctions_setBit(manRegVal, DPAR_MONR_BIT, bval);
        _unRecipeFunctions_setBit(manRegVal, DPAR_MOFFR_BIT, !bval);  		
        // Insert the address and values in the lists
        dynAppend(dparManRegAddr, manRegAddr);       
        dynAppend(dparManRegVal, manRegVal);
        break;
      case "CPC_WordParameter":
        numWPar++;
        // Get the ManReg01 address
        _unRecipeFunctions_getDpeAddress(deviceName+".ProcessOutput.ManReg01", deviceType, manRegAddr, exceptionInfo);
		if (dynlen(exceptionInfo)) return;
		//Debug("ManRegAddr of : " + deviceName +".ProcessOutput.ManReg01 : " + manRegAddr +"\n");
        // Set the new ManReg01 values for the ArmRcp
		manRegVal=0;
		_unRecipeFunctions_setBit(manRegVal, WPAR_ARM_BIT, true);
		_unRecipeFunctions_setBit(manRegVal, WPAR_NEWMR_BIT, true);
        // Insert the address and values in the lists
        dynAppend(wparManRegAddr, manRegAddr);       
        dynAppend(wparManRegVal, manRegVal);
        // Get the MPosR address
        _unRecipeFunctions_getDpeAddress(deviceName+".ProcessOutput.MPosR", deviceType, reqAddr, exceptionInfo);
		if (dynlen(exceptionInfo)) return;
        dynAppend(wparReqAddr, reqAddr);
		unGenericObject_StringToFloat(values[i], fval, bOk);
        dynAppend(wparReqVal, fval);
        break; 
      case "CPC_AnalogParameter":
        numAPar++;
        // Get the ManReg01 address
        _unRecipeFunctions_getDpeAddress(deviceName+".ProcessOutput.ManReg01", deviceType, manRegAddr, exceptionInfo);
		if (dynlen(exceptionInfo)) return;
		//Debug("ManRegAddr of : " + deviceName +".ProcessOutput.ManReg01 : " + manRegAddr +"\n");
        // Set the new ManReg01 values for the ArmRcp
        manRegVal=0;
		_unRecipeFunctions_setBit(manRegVal, APAR_ARM_BIT, true);
		_unRecipeFunctions_setBit(manRegVal, APAR_NEWMR_BIT, true);
        // Insert the address and values in the lists
        dynAppend(aparManRegAddr, manRegAddr);       
        dynAppend(aparManRegVal, manRegVal);
        // Get the MPosR address
        _unRecipeFunctions_getDpeAddress(deviceName+".ProcessOutput.MPosR", deviceType, reqAddr, exceptionInfo);
		if (dynlen(exceptionInfo)) return;
        dynAppend(aparReqAddr, reqAddr);
		unGenericObject_StringToFloat(values[i], fval, bOk);
        dynAppend(aparReqVal, fval);
        break;        
        
      case "CPC_AnalogAlarm":
		numAAlarm++; 
        dynClear(list);
        // Get the ManReg01 address and ManReg01 value
        if (!mappingHasKey(aalarmMap, deviceName)){                  
          _unRecipeFunctions_getDpeAddress(deviceName+".ProcessOutput.ManReg01", deviceType, manRegAddr, exceptionInfo);
		  if (dynlen(exceptionInfo)) return;
          list[1][1] = manRegAddr;
          list[1][2] = 0;
          aalarmMap[deviceName]=list;
          manRegVal=0;    
        } else {
          list=aalarmMap[deviceName];
          manRegAddr=list[1][1];
          manRegVal = list[1][2];
        }
        
		// Debug("ManRegAddr of : " + deviceName +".ProcessOutput.ManReg01 : " + manRegAddr +"\n");
		 
        // Modifies the ManReg Value
		// Get the MNewMR bit position
        if (!mappingHasKey(unRecipe_aalarmManRegBits, dpeName)){
          fwException_raise(exceptionInfo, "ERROR", "The analog alarm dpe:" + dpeName + 
                            " hasn't a bit possition defined in the ManReg01.", "");
          return;  
        }	
        pos = unRecipe_aalarmManRegBits[dpeName];
		
		// Set the new ManReg01 values for the ArmRcp
		_unRecipeFunctions_setBit(manRegVal, AALARM_ARM_BIT, true); 
		_unRecipeFunctions_setBit(manRegVal, pos, true); 
        list[1][2]=manRegVal;   
        		
        // Set the Req. Address and Req. Value
        _unRecipeFunctions_getDpeAddress(dpe, deviceType, reqAddr, exceptionInfo);
		if (dynlen(exceptionInfo)) return;
        dynAppend(list, reqAddr);
		unGenericObject_StringToFloat(values[i], fval, bOk);
        list[dynlen(list)][2] = fval; 
        
        aalarmMap[deviceName] = list;
        break;
        
      case "CPC_Controller":
	    numPid++; 
        dynClear(list);
        // Get the ManReg01 address and ManReg01 value
        if (!mappingHasKey(controllerMap, deviceName)){     
          _unRecipeFunctions_getDpeAddress(deviceName+".ProcessOutput.ManReg01", deviceType, manRegAddr, exceptionInfo);
		  if (dynlen(exceptionInfo)) return;		  
          list[1][1] = manRegAddr;
          list[1][2] = 0;
          controllerMap[deviceName]=list;
          manRegVal    = 0;          
        } else {
          list=controllerMap[deviceName];
          manRegAddr   = list[1][1];
          manRegVal    = list[1][2];
        }
        
		//Debug("ManRegAddr of : " + deviceName +".ProcessOutput.ManReg01 : " + manRegAddr +"\n");
		
        // Modifies the ManReg Value
        if (!mappingHasKey(unRecipe_unRecipe_pidManRegBits, dpeName)){
          fwException_raise(exceptionInfo, "ERROR", "The CPC_Controller dpe:" + dpeName + 
                            " hasn't a bit possition defined in the ManReg01.", "");
          return;  
        }
        pos = unRecipe_unRecipe_pidManRegBits[dpeName];
		_unRecipeFunctions_setBit(manRegVal, PID_ARM_BIT, true); 
		_unRecipeFunctions_setBit(manRegVal, pos, true); 
        list[1][2]=manRegVal;   
		
        // Set the Req. Address and Req. Value
        _unRecipeFunctions_getDpeAddress(dpe, deviceType, reqAddr, exceptionInfo);
		if (dynlen(exceptionInfo)) return;
        dynAppend(list, reqAddr);
		unGenericObject_StringToFloat(values[i], fval, bOk);
        list[dynlen(list)][2] = fval; 
        
		//DebugTN("Assigning Controller value - Address: " + reqAddr + "  Value: " + values[i] + "   listValue: " + list[dynlen(list)][2] + "\n");
		
        controllerMap[deviceName] = list;
        break;
    }
  }
  
  // Gets all the recipe buffers
  dyn_int recipeHeaderBuffer = _unRecipeFunctions_getHeaderBuffer(sPlcName);
  dyn_int manRegAddrBuffer   = _unRecipeFunctions_getManRegAddrBuffer(sPlcName);
  dyn_int manRegValBuffer    = _unRecipeFunctions_getManRegValBuffer(sPlcName);
  dyn_int reqAddrBuffer      = _unRecipeFunctions_getReqAddrBuffer(sPlcName);
  dyn_float reqValBuffer   	 = _unRecipeFunctions_getReqValBuffer(sPlcName);
  
  
  // Inserts the data in the recipe header buffer 
  // Split the current time in high part and low part
  dynClear(recipeHeaderBuffer);
  bit64 t = getCurrentTime();
  bit32 bLow  = 0x0000FFFFu & t;
  bit32 bHigh = 0x0000FFFFu & (t >> 16);
  // TODO: Fix this!
  recipeHeaderBuffer[RCP_HEADER_ID_LOW] 	= (int)bLow % 32000;
  recipeHeaderBuffer[RCP_HEADER_ID_HIGH] 	= (int)bHigh % 32000;
  recipeHeaderBuffer[RCP_HEADER_NUM_MANREG] = (numDPar + numWPar + numAPar + numAAlarm + numPid);
  recipeHeaderBuffer[RCP_HEADER_NUM_DPAR] 	= numDPar;
  recipeHeaderBuffer[RCP_HEADER_DB_DPAR] 	= 0;
  recipeHeaderBuffer[RCP_HEADER_NUM_WPAR] 	= numWPar;
  recipeHeaderBuffer[RCP_HEADER_DB_WPAR] 	= 0;
  recipeHeaderBuffer[RCP_HEADER_NUM_APAR] 	= numAPar;
  recipeHeaderBuffer[RCP_HEADER_DB_APAR] 	= 0;
  recipeHeaderBuffer[RCP_HEADER_NUM_AALARM] = numAAlarm;
  recipeHeaderBuffer[RCP_HEADER_DB_AALARM]  = 0;
  recipeHeaderBuffer[RCP_HEADER_NUM_CONTR] 	= numPid;
  recipeHeaderBuffer[RCP_HEADER_DB_CONTR] 	= 0;
  recipeHeaderBuffer[RCP_HEADER_SENT_VALUE] = RCP_ACT_HEADER_SENDING_DATA;
  recipeHeaderBuffer[RCP_HEADER_CRC] 		= 0;
  gHeaderMap[sPlcName] 	  = recipeHeaderBuffer;
  
  // Process the controller devices in the recipe to create the ManReg address, ManReg values, ReqAddress and ReqValues
  //DebugTN("Processing the controller values\n");
  _unRecipeFunctions_processMultiDpeDevices(controllerMap, pidManRegAddr, pidManRegVal, pidReqAddr, pidReqVal);
  //DebugTN("PID Req. values: ", pidReqVal);
  
  // Process the analog alarm devices in the recipe to create the ManReg address, ManReg values, ReqAddress and ReqValues
  //DebugTN("Processing the aalarm values\n");
  _unRecipeFunctions_processMultiDpeDevices(aalarmMap, aalarmManRegAddr, aalarmManRegVal, aalarmReqAddr, aalarmReqVal);

  // Inserts the data in the ManReg Address buffer
  dynClear(manRegAddrBuffer);
  dynAppend(manRegAddrBuffer, dparManRegAddr);
  dynAppend(manRegAddrBuffer, wparManRegAddr);
  dynAppend(manRegAddrBuffer, aparManRegAddr);
  dynAppend(manRegAddrBuffer, aalarmManRegAddr);
  dynAppend(manRegAddrBuffer, pidManRegAddr);
  gManRegAddrMap[sPlcName] = manRegAddrBuffer; 
 /*
  DebugTN("ManRegAddr DPAR: ", dparManRegAddr);
  DebugTN("ManRegAddr WPAR: ", wparManRegAddr);
  DebugTN("ManRegAddr APAR: ", aparManRegAddr);
  DebugTN("ManRegAddr AALARM: ", aalarmManRegAddr);
  DebugTN("ManRegAddr PID: ", pidManRegAddr);*/
  //DebugTN("Setting Man Reg Addr for PLC: " + sPlcName, gManRegAddrMap[sPlcName]);
 
  // Inserts the data in the ManReg Values buffer  (ArmRcp values)
  dynClear(manRegValBuffer);
  dynAppend(manRegValBuffer, dparManRegVal);
  dynAppend(manRegValBuffer, wparManRegVal);
  dynAppend(manRegValBuffer, aparManRegVal);
  dynAppend(manRegValBuffer, aalarmManRegVal);
  dynAppend(manRegValBuffer, pidManRegVal);
  gManRegValMap[sPlcName] = manRegValBuffer;
  //DebugTN("Setting Man Reg Val for PLC: " + sPlcName, gManRegValMap[sPlcName]);
  // Inserts the data in the Requested Address buffer
  dynClear(reqAddrBuffer);
  dynAppend(reqAddrBuffer, dparReqAddr);
  dynAppend(reqAddrBuffer, wparReqAddr);
  dynAppend(reqAddrBuffer, aparReqAddr);
  dynAppend(reqAddrBuffer, aalarmReqAddr);
  dynAppend(reqAddrBuffer, pidReqAddr);
  gReqAddrMap[sPlcName] = reqAddrBuffer;
  //DebugTN("Setting Req Addr for PLC: " + sPlcName, gReqAddrMap[sPlcName]);
  // Inserts the data in the Requested Values buffer
  dynClear(reqValBuffer);  
  //dynAppend(reqValBuffer, dparReqVal);
  dynAppend(reqValBuffer, wparReqVal);
  dynAppend(reqValBuffer, aparReqVal);
  dynAppend(reqValBuffer, aalarmReqVal);
  dynAppend(reqValBuffer, pidReqVal);
  // DebugTN("Req Val Buffer: ", reqValBuffer);
  gReqValMap[sPlcName] = reqValBuffer;
  //DebugTN("Setting Req Val for PLC: " + sPlcName, gReqValMap[sPlcName]);
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_createBuffersData", 
			"_unRecipeFunctions_createBuffersData", "PLCName: " + sPlcName, "ManReq Addr: ", manRegAddrBuffer, 
			"ManReq Vals (ArmRcp): ", manRegValBuffer, "Req Addr: ", reqAddrBuffer, "Req Vals: ", reqValBuffer);
  
  //t1 = getCurrentTime();
  
 // string elapsedTime = formatTime("%S", t1-t0, " milli: %04d");
 // Debug ("Elapsed time creating buffers data: " + elapsedTime +"\n");
}

//------------------------------------------------------------------------------------------------------------------------
void _unRecipeFunctions_sendData(string plcName, dyn_string &exceptionInfo)
{  
  bool done = false;
  dyn_string dpeManRegAddr, dpeManRegVal, dpeReqAddr, dpeReqVal;
  dyn_int manRegAddr, manRegVal, reqAddr;
  dyn_float reqVal;
  string sTmp, sBufferDpName;
  
  _unRecipeFunctions_getRecipeBuffersName(plcName, sBufferDpName, exceptionInfo);
  manRegAddr = _unRecipeFunctions_getManRegAddrBuffer(plcName);
  manRegVal  = _unRecipeFunctions_getManRegValBuffer(plcName);
  reqAddr    = _unRecipeFunctions_getReqAddrBuffer(plcName);
  reqVal     = _unRecipeFunctions_getReqValBuffer(plcName);
   
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_sendData", 
			"_unRecipeFunctions_sendData", "Function START", "PLCName: " + plcName, "ManRegAddr: ", manRegAddr,
			"ManRegVal : ", manRegVal, "ReqAddr: ", reqAddr, "ReqVal : ", reqVal);
  
  int count=0;
  int numManReg  = dynlen(manRegAddr);
  int numReqAddr = dynlen(reqAddr);  
  int numStructures = ((numManReg-1)/CPC_CONFIG_RECIPEBUFFERS_MAX_ELEMENTS_WORD_STRUCTURE)+1;
  
//  Debug("1 - Creating " + numStructures + " WORD structure DPES _unRecipeFunctions_sendData\n");
  for (int i=1; i<=numStructures && !done; i++){
    for (int j=1; j<=CPC_CONFIG_RECIPEBUFFERS_MAX_ELEMENTS_WORD_STRUCTURE; j++){
      string dpe = ".Structure"+i+".Value"+(count+1);
      dynAppend(dpeManRegAddr, sBufferDpName+"ProcessOutput.ManRegAddr"+dpe);
      dynAppend(dpeManRegVal, sBufferDpName+"ProcessOutput.ManRegVal"+dpe);
            
      if (count<numReqAddr)
        dynAppend(dpeReqAddr, sBufferDpName+"ProcessOutput.ReqAddr"+dpe);  
      count++;   
      if (count==numManReg){
        done=true;
        break;     
      }
    }
  }
  
  count=0; 
  done=false;
  
  if (numReqAddr>0) {
//    Debug("NumReqAddr: " + numReqAddr + "\n");
	numStructures = ((numReqAddr-1)/CPC_CONFIG_RECIPEBUFFERS_MAX_ELEMENTS_FLOAT_STRUCTURE)+1;
	//Debug("2 - Creating " + numStructures + " FLOAT structure DPES _unRecipeFunctions_sendData\n");
	for (int i=1; i<=numStructures && !done; i++){
	  for (int j=1; j<=CPC_CONFIG_RECIPEBUFFERS_MAX_ELEMENTS_FLOAT_STRUCTURE; j++){
		string dpe = ".Structure"+i+".Value"+(count+1);
		dynAppend(dpeReqVal, sBufferDpName+"ProcessOutput.ReqVal"+dpe);
		count++;
		if (count==numReqAddr){
		  done=true;
		  break;
		}
	  }
	}
  }
  //Debug("3 - _unRecipeFunctions_sendData\n");

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_sendData", 
			"_unRecipeFunctions_sendData", "DpeManRegAddr: ", dpeManRegAddr,
			"DpeManRegVal: ", dpeManRegVal, "DpeReqAddr: ", dpeReqAddr, "DpeReqVal: ", dpeReqVal,
			"ManRegAddr: ", manRegAddr, "ManRegVal: ", manRegVal, "ReqAddr: ", reqAddr, "ReqVal: ", reqVal);
			
  dpSetWait(dpeManRegAddr, manRegAddr);
  dpSetWait(dpeManRegVal, manRegVal);
  if (numReqAddr > 0) {
	dpSetWait(dpeReqAddr, reqAddr);
	dpSetWait(dpeReqVal, reqVal);
  }
  
//  Debug("4 - All recipe data sent to the PLC " + plcName + " _unRecipeFunctions_sendData\n");
  _unRecipeFunctions_writeInRecipeLog("All recipe data sent to the PLC " + plcName);
}

//------------------------------------------------------------------------------------------------------------------------
void _unRecipeFunctions_processMultiDpeDevices(mapping map, dyn_int &manRegAddresses, dyn_int &manRegValues, 
                                               dyn_int &reqAddresses, dyn_float &reqValues)
{
  dynClear(manRegAddresses);
  dynClear(manRegValues);
  dynClear(reqAddresses);
  dynClear(reqValues);  
  
  int len = mappinglen(map);
  dyn_string keys = mappingKeys(map);
  
  // Iterates through all the devices in the map
  for (int i=1; i<=len; i++){
    string deviceName = keys[i];
    dyn_dyn_string data = map[deviceName];
    int manRegAddr   = data[1][1];
    int manRegVal    = data[1][2];
    int datalen = dynlen(data);

	//DebugTN("Data: ", data);
    // Iterates through all the device DPES     
    for (int j=2; j<=datalen; j++){
      int reqAddr = data[j][1];
      float reqVal  = data[j][2];
	//  DebugTN("ManRegAddr: " + manRegAddr + "  ManRegVal: " + manRegVal + "\n");
    //  DebugTN("ReqAddr: " + reqAddr + "  ReqVal: " + reqVal + "\n");
      // Inserts the data in the buffers
      dynAppend(manRegAddresses, manRegAddr);
      dynAppend(manRegValues, manRegVal);
      dynAppend(reqAddresses, reqAddr);
      dynAppend(reqValues, reqVal);
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Get the PLC address of a DPE.
 * @param dpe 			- [IN] The Datapoint element which address will be calculated.
 * @param deviceType 	- [IN] The device type name.
 * @param address 		- [OUT] The PLC address.
 * @param exceptionInfo - [OUT] tandard exception handling variable.
 */
void _unRecipeFunctions_getDpeAddress(string dpe, string deviceType, 
                                     string &address, dyn_string &exceptionInfo)
{
  string addressType, temp;
  dpGet(dpe + ":_address.._drv_ident", addressType);
  string sFunction = "DRV_" + addressType + "_convertToUnicosAddress";
  if(!isFunctionDefined(sFunction)) {
    fwException_raise(exceptionInfo, "ERROR", "The function " +sFunction+ " isn't defined.", "");  
	DebugTN(exceptionInfo);
    return;
  }	

  evalScript(address,	"string main(string sDeviceDpeName, string sDeviceType)"+
  										"{	string temp;"+
  										"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
  										"	return temp;"+
  										"}", makeDynString(), dpe, deviceType);     
  
  // If it's an S7 address, gets the offset
  if (addressType=="S7"){
    string fullAddress = address;
    string offset;
    
    int pos=strpos(fullAddress, ".");
    if(pos<0) {
      //Debug("Invalid address format: " + fullAddress +"\n");
      fwException_raise(exceptionInfo, "ERROR", "The S7 address format is invalid: " + fullAddress, "");
      return;
    }
  
    string temp = substr(fullAddress, pos+1, strlen(fullAddress)-pos);
    int len = strlen(temp);
    int j=0;
    for (int i=0; i<=len; i++){
      if ((temp[i]>='0' && temp[i]<='9') || temp[i]=='\0'){
        offset = offset + temp[i];
        j++;
      }
    }
    address = offset;  
  }
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Set the bit value in a register variable.
 * @param manRegVal - [IN/OUT] Register where the bit value will be modified.
 * @param position  - [IN] Bit position in the register variable.
 * @param value		- [IN] New value for the bit in the register.
 */
void _unRecipeFunctions_setBit(anytype &manRegVal, int position, bool value)
{
  bit64 bitVariable;
  
  bitVariable = manRegVal;
  setBit(bitVariable, position, value);
  manRegVal = bitVariable;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the bit value in a register variable.
 * @param manRegVal - [IN] Register where the bit value will read.
 * @param position  - [IN] Bit position in the register variable.
 * @param value		- [IN] Value of the bit in the register.
 */
void _unRecipeFunctions_getBit(anytype manRegVal, int position, int &value)
{
  bit64 bitVariable;
  bitVariable = manRegVal;
  value = getBit(bitVariable, position);
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the recipe buffers DPE name for an specific PLC.
 * @param plcName - [IN] PLC name.
 * @param dpName - [OUT] Name of the DPE where the recipe buffers of the PLC are located.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_getRecipeBuffersName(string plcName, string &dpName, dyn_string &exceptionInfo)
{
  dpName = plcName;
  
  // Removes the system name of the plcName
  int pos = strpos(plcName, ":");
  if (pos>=0 && strlen(plcName)>pos) {
    plcName = substr(plcName, pos+1);
  }  
  
  string buffersName = dpName + "_" + CPC_CONFIG_RECIPEBUFFERS_DPT_NAME;
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(buffersName, dpName);  

  //DebugTN("BuffersName: " + buffersName + "   DPName: " + dpName);  
  
  if (!dpExists(dpName)) {
    //Debug("DP doesn't exist: " + dpName +"\n");
    fwException_raise(exceptionInfo, "ERROR", "The PLC recipe buffers don't exist: " + dpName, "");
	DebugTN(exceptionInfo);
  }
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "_unRecipeFunctions_getRecipeBuffersName", "_unRecipeFunctions_getRecipeBuffersName", 
			"PlcName: ", plcName, "BuffersName: " + buffersName, "DPName: " + dpName);
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Insert the DB number for each recipe device type in the recipe header buffer (Siemens only).
 * @param sPlcName - [IN] PLC Name.
 * @param dsDpes - [IN] List of DPEs of a recipe.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_createRecipeHeaderDBNumber(string sPlcName, dyn_string dsDpes, dyn_string &exceptionInfo)
{
  int len;
  string dbNum;
  mapping dbMapping;  
  dyn_int recipeHeaderBuffer = _unRecipeFunctions_getHeaderBuffer(sPlcName);   
  
  len=dynlen(dsDpes);
  for (int i=1; i<=len; i++) {
    string deviceType = dpTypeName(dsDpes[i]);
    if (mappingHasKey(dbMapping, deviceType))
      continue;
    
    _unRecipeFunctions_getS7AddressDBNumber(dsDpes[i], deviceType, dbNum, exceptionInfo);
    if (dynlen(exceptionInfo))  return;
       
    dbMapping[deviceType] = dbNum;
    //Debug("DB of " + deviceType + ": " + dbMapping[deviceType] +"\n");
  }
  
  // Insert the data in the recipe header
  if (mappingHasKey(dbMapping, "CPC_DigitalParameter"))
    recipeHeaderBuffer[RCP_HEADER_DB_DPAR] = dbMapping["CPC_DigitalParameter"];
  else 
    recipeHeaderBuffer[RCP_HEADER_DB_DPAR] = 0;
  if (mappingHasKey(dbMapping, "CPC_WordParameter"))
    recipeHeaderBuffer[RCP_HEADER_DB_WPAR] = dbMapping["CPC_WordParameter"];
  else
    recipeHeaderBuffer[RCP_HEADER_DB_WPAR] = 0;
  if (mappingHasKey(dbMapping, "CPC_AnalogParameter"))
    recipeHeaderBuffer[RCP_HEADER_DB_APAR] = dbMapping["CPC_AnalogParameter"];  
  else 
    recipeHeaderBuffer[RCP_HEADER_DB_APAR] = 0;
  if (mappingHasKey(dbMapping, "CPC_AnalogAlarm"))
    recipeHeaderBuffer[RCP_HEADER_DB_AALARM] = dbMapping["CPC_AnalogAlarm"];
  else 
    recipeHeaderBuffer[RCP_HEADER_DB_AALARM] = 0;
  if (mappingHasKey(dbMapping, "CPC_Controller"))
    recipeHeaderBuffer[RCP_HEADER_DB_CONTR] = dbMapping["CPC_Controller"];  
  else
    recipeHeaderBuffer[RCP_HEADER_DB_CONTR] = 0;
  
  gHeaderMap[sPlcName] = recipeHeaderBuffer;
  //Debug("Header Buffer: " + recipeHeaderBuffer + "\n");
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Gets the DB number of a S7 address for a DPE.
 * @param sDpe - [IN] DPE which DB number is required.
 * @param sDeviceType - [IN] Device type name of the dpe.
 * @param sDb - [OUT] DB number of the DPE S7 address.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_getS7AddressDBNumber(string sDpe, string sDeviceType, string &sDb, dyn_string &exceptionInfo)
{
  string fullAddress = DRV_S7_convertToUnicosAddress(sDpe, sDeviceType);

  int pos=strpos(fullAddress, ".");
  if(pos<0) {
    fwException_raise(exceptionInfo, "ERROR", "The S7 address format is invalid: " + fullAddress, "");
    return;
  }
  
  sDb = substr(fullAddress, 0, pos);
  strreplace(sDb, "DB", "");
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Gets the offset of an S7 address where a DPE is located.
 * @param sDpe - [IN] DPE which offset is required.
 * @param sDeviceType - [IN] Device type name of the dpe.
 * @param sOffset - [OUT] Offset of the DPE S7 address.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_getS7AddressOffset(string sDpe, string sDeviceType, string &sOffset, dyn_string &exceptionInfo)
{
  string fullAddress = DRV_S7_convertToUnicosAddress(sDpe, sDeviceType);

  int pos=strpos(fullAddress, ".");
  if(pos<0) {
    fwException_raise(exceptionInfo, "ERROR", "The S7 address format is invalid: " + fullAddress, "");
    return;
  }
  
  string temp = substr(fullAddress, pos+1, strlen(fullAddress)-pos);
  int len = strlen(temp);
  int j=0;
  for (int i=0; i<=len; i++){
    if ((temp[i]>='0' && temp[i]<='9') || temp[i]=='\0'){
      sOffset = sOffset + temp[i];
      j++;
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Gets the recipe elements according to the distributed system.
 * @param elements - [IN] Recipe data point elements.
 * @param values - [IN] Recipe values.
 * @param systemDpes - [OUT] Map containing the recipe elements and values for each distributed system.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */  
void _unRecipeFunctions_getElementsBySystem(dyn_string elements, dyn_anytype values, 
                                            mapping &systemDpes, dyn_string &exceptionInfo)
{
  string pvssDpe;
  dyn_dyn_string data;
  mappingClear(systemDpes);
  
  int len = dynlen(elements);
  if (len!=dynlen(values)) {
    fwException_raise(exceptionInfo, "ERROR", "The number of DPEs and values of the recipe must be equal.", "");
    return;
  }  
  
  // Inserts the DPEs in the mapping according to the system name
  for (int i=1; i<=len; i++) {
    unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(elements[i], pvssDpe);
    string systemName = unGenericDpFunctions_getSystemName(pvssDpe);
    
    dynClear(data);
    if (mappingHasKey(systemDpes, systemName)) {
      data = systemDpes[systemName];
    }

    dynAppend(data[1], pvssDpe);
    dynAppend(data[2], values[i]);
    systemDpes[systemName] = data;
  }  
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the DPEs that contains the online values (ProcessInput) for the specified DPEs (ProcessOutput)
 * @param dpNames - [IN] List of DPEs which online value DPEs are required.
 * @param outputDpes - [OUT] List of online value DPEs.
 */
void _unRecipeFunctions_getOnlineValueDpes (dyn_string dpNames, dyn_string &outputDpes) {
	int len = dynlen(dpNames);
	for (int i=1; i<=len; i++) {
		string dpType = dpTypeName(dpNames[i]);
		
		if (dpType=="CPC_DigitalParameter") {
			strreplace(dpNames[i], "ProcessOutput.ManReg01", "ProcessInput.PosSt");
		} else if (dpType=="CPC_AnalogAlarm") {
			strreplace(dpNames[i], "ProcessOutput", "ProcessInput");
			dpNames[i] += "St";
		} else if (dpType=="CPC_Controller") {
			strreplace(dpNames[i], "ProcessOutput", "ProcessInput");
			if (strpos(dpNames[i], "MSPH") >= 0) {
				strreplace(dpNames[i], "MSPH", "ActSPH");
			} else if (strpos(dpNames[i], "MSPL") >= 0) {
				strreplace(dpNames[i], "MSPL", "ActSPL");
			} else if (strpos(dpNames[i], "MSP") >= 0) {
				strreplace(dpNames[i], "MSP", "MSPSt");
			} else if (strpos(dpNames[i], "MOutH") >= 0) {
				strreplace(dpNames[i], "MOutH", "ActOutH");
			} else if (strpos(dpNames[i], "MOutL") >= 0) {
				strreplace(dpNames[i], "MOutL", "ActOutL");
			} else if (strpos(dpNames[i], "MKc") >= 0) {
				strreplace(dpNames[i], "MKc", "ActKc");
			} else if (strpos(dpNames[i], "MTi") >= 0) {
				strreplace(dpNames[i], "MTi", "ActTi");
			} else if (strpos(dpNames[i], "MTd") >= 0) {
				strreplace(dpNames[i], "MTd", "ActTd");
			} else if (strpos(dpNames[i], "MTds") >= 0) {
				strreplace(dpNames[i], "MTds", "ActTds");
			} 
		} else if (dpType=="CPC_WordParameter" || dpType=="CPC_AnalogParameter") {
			strreplace(dpNames[i], "ProcessOutput.MPosR", "ProcessInput.PosSt");
		} 
	}
	
	outputDpes = dpNames;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Check if the recipe values are the same than the online values.
 * @param dpes - [IN] Recipe elements data points.
 * @param values - [IN] Recipe values.
 * @param errDpe - [OUT] If the recipe values are different than the online values, this variable
 * 				   will contain the first dpe where the values are different.
 * @return TRUE if the recipe values are the same than the online values, otherwise FALSE.
 */
 bool _unRecipeFunctions_checkRecipeOnlineValues(dyn_string dpes, dyn_string values, string &errDpe)
{ 
  bool bOk;
  dyn_string outputDpes;
  dyn_anytype onlineValues;
  dyn_string exceptionInfo;
  string sFormat, sOnlineValue;
  float fOnlineValue, fRcpValue;;
  
  if (dynlen(dpes)==0)
	return true;
  
  // Get the DPEs for the online values
  _unRecipeFunctions_getOnlineValueDpes (dpes, outputDpes);
  dpGet(outputDpes, onlineValues);
  
  int len = dynlen(onlineValues);
  for (int i=1; i<=len; i++) {	  
	  float fval, frcpval, fDiff;
      int ival, ircpval;
      bool bval, brcpval;        
	  
	  string dpType = dpTypeName(outputDpes[i]);
	  if (dpType=="CPC_DigitalParameter") {
		bval = onlineValues[i];
		brcpval = values[i];
		if (bval!=brcpval) {
			errDpe = dpes[i];
			return false;
		}
		continue;	
	  }
		
      int elementType = dpElementType(outputDpes[i]); 
      switch(elementType){
        case DPEL_FLOAT:
			sFormat = dpGetFormat(outputDpes[i]);
			// Format the online value (e.g. values like '10.579999923706' will be converted to '10.580')
			sOnlineValue = unGenericObject_FormatValue(sFormat, onlineValues[i]);
			//DebugTN("sOnlineValue: " + onlineValues[i] + "  sFormatted: " + sOnlineValue);
			// Get the online value as float
			unGenericObject_StringToFloat(sOnlineValue, fOnlineValue, bOk);
			//DebugTN("fOnlineValue: " + fOnlineValue);
			// Get the recipe value as float
			unGenericObject_StringToFloat(values[i], fRcpValue, bOk);
			// Compare the float values
			//DebugTN("Rcp Value: " + fRcpValue + "  Online Value: " + fOnlineValue);
			if (fRcpValue != fOnlineValue) {
				errDpe = dpes[i];
				return false;
			}
          break;
        case DPEL_INT:
          ircpval = (int)values[i];
          if (onlineValues[i]!=ircpval) {
			errDpe = dpes[i];
            return false;
          }             
          break;        
        case DPEL_BOOL:
          brcpval = (bool)values[i];
          if (onlineValues[i]!=brcpval) {
			errDpe = dpes[i];
            return false;
          }  
          break;        
        default:
          value = (string)values[j];           
      }
  }
  
  return true;
} 

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the status of the recipe instance (Activated, inactive, no plc connection,...).
 * @param recipeObject  - [IN] Recipe instance data from fwConfigDB.
 * @param lastActivated - [IN] String containing the name of the last activated recipe of the class.
 * @param status 	    - [OUT] Recipe status.
 * @param errMsg        - [OUT] Error message for the user.
 * @param exceptionInfo - [OUT] Standard error handling variable.
 */ 
void unRecipeFunctions_getRecipeStatus(dyn_dyn_mixed recipeObject, string lastActivated, int &status, 
                                       dyn_string &errMsg, dyn_string &exceptionInfo)
{
  bool active;
  dyn_string statusMessage;
  string errDpe;
  string rcpName 		 = recipeObject[fwConfigurationDB_RO_META_ORIGNAME];
  dyn_string rcpElements = recipeObject[fwConfigurationDB_RO_DPE_NAME];
  dyn_string rcpValues   = recipeObject[fwConfigurationDB_RO_VALUE];
  dyn_string rcpTypes	 = recipeObject[fwConfigurationDB_RO_ELEMENT_TYPE];
  
  // Check the PLCs status
  unRecipeFunctions_getPlcStatus(rcpElements, active, errMsg, exceptionInfo);
  if(active==false){
    status = RCP_NO_PLC_CONNECTION;
    return;
  }
  
  // Check if all the DPEs used in the recipe exist
  unRecipeFunctions_checkRecipeStructure(rcpName, rcpElements, rcpValues, rcpTypes, exceptionInfo);
  if (dynlen(exceptionInfo)){
    status = RCP_BAD_TYPES;
    return;
  }  
  
  // Check if the last activated recipe of this type is the current one
  if (lastActivated!=rcpName){
    status = RCP_INACTIVE;
    return;
  }      
  
  //Debug("unRecipeFunctions_getRecipeStatus - Getting online values \n");
  if(false == _unRecipeFunctions_checkRecipeOnlineValues(rcpElements, rcpValues, errDpe)){
    status = RCP_INACTIVE;  
    return;  
  }  

  status = RCP_ACTIVATED;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Check if the recipe structure is coherent:
 *   - The number of DPEs and the number of values is the same
 *   - The values match the expected data type.
 * @param sRcpName - [IN] 
 * @param dsRcpElements - [IN] 
 * @param dsRcpValues - [IN]
 * @param dsRcpTypes - [IN] 
 * @param exceptionInfo - [OUT] Standard error handling variable.
 */
void unRecipeFunctions_checkRecipeStructure(string sRcpName, dyn_string dsRcpElements, dyn_string dsRcpValues, dyn_string dsRcpTypes,
                                            dyn_string &exceptionInfo)
{
  float fval;
  int ival, elementType;
  bool bval;
  dyn_string exception;
  string deviceType = "", lowerValue;
  
  dynClear(exceptionInfo);
  
  // Check if the number of elements is equal to the number of values
  if (dynlen(dsRcpElements)!=dynlen(dsRcpValues)){
    fwException_raise(exceptionInfo, "ERROR", "unRecipeFunctions_checkRecipeStructure: " +
                      "The number of elements is different than the number of values: " + sRcpName, "");
    return;
  }
  
  // Check if the recipe value types match the data point element types
  int numElements = dynlen(dsRcpElements);
  for (int i=1; i<=numElements; i++){    
	deviceType = dpTypeName(dsRcpElements[i]);
	// TODO: Avoid the type hardcoding (?)
	if (deviceType=="CPC_DigitalParameter") 
		elementType = DPEL_BOOL;
	else 
		elementType = dsRcpTypes[i];
    dynClear(exception);
	
    switch(elementType){
      case DPEL_FLOAT:       
        unConfigGenericFunctions_checkFloat(dsRcpValues[i], fval, exception); 
        if(dynlen(exception)){ 
          dynAppend(exceptionInfo, exception);  
        }
        break;
      case DPEL_INT:
        unConfigGenericFunctions_checkInt(dsRcpValues[i], ival, exception); 
        if(dynlen(exception)) {
          dynAppend(exceptionInfo, exception);      
        }
        break;
      case DPEL_BOOL:
		lowerValue = strtolower(dsRcpValues[i]);
		if (lowerValue!="true" && lowerValue!="false")
			fwException_raise(exceptionInfo, "ERROR", "unRecipeFunctions_checkRecipeStructure: " +
			          "Wrong boolean value: " + dsRcpValues[i], "");
        break;
      default:
        fwException_raise(exceptionInfo, "ERROR", "unRecipeFunctions_checkRecipeStructure: " +
                          "Unknown DPType: " + dpTypeName(dsRcpElements[i]), "");
        return;
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Check if the recipe values are valid according to the defined ranges.
 * @param recipeObject - [IN] Recipe object.
 * @param values - [IN/OUT] Recipe values.
 * @param errorLines - [OUT] Positions where the values are wrong according to the defined ranges.
 * @param exceptionInfo - [OUT] Standard exception handling routine.
 */
void unRecipeFunctions_checkRecipeValues (dyn_dyn_mixed recipeObject, dyn_anytype &values, 
                                          dyn_int &errorLines, dyn_string &exceptionInfo)
{
  string min, max, range, sVal;
  dyn_string exception;
  dynClear(errorLines);
  dynClear(exceptionInfo);
  int retVal;
  int len = dynlen(recipeObject[fwConfigurationDB_RO_DPE_NAME]);
  
  // Check if the number of elements in the recipe matches 
  // the number of elements in the table.
  if (len != dynlen(values)) {
    fwException_raise(exceptionInfo, "ERROR", "The number of elements in the recipe " + 
                      "is different than the number of values.", "");
    return;
  }
  
  for (int i=1; i<=len; i++){
	// Check if the recipe value is valid
    retVal = unRecipeFunctions_checkRecipeValue (recipeObject[fwConfigurationDB_RO_DPE_NAME][i], values[i], exception);

	if (dynlen(exception)) {
      dynAppend(exceptionInfo, exception);
      dynClear(exception);
    }	  
	  
	if (retVal == -1) {
      dynAppend(errorLines, i);
    } 
  }
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Check if the recipe value is valid according to the defined range.
 * @param sDpName - [IN] Data point name of the recipe element.
 * @param values  - [IN] Value of the recipe element.
 * @param exceptionInfo - [OUT] Standard exception handling routine.
 * @return 0 if the value is in the defined range or if no range is defined, otherwise -1.
 */
int unRecipeFunctions_checkRecipeValue (string sDpName, anytype value, dyn_string &exceptionInfo)
{
	string sVal, range, min, max;
	
	range = unRecipeFunctions_getDeviceRange(sDpName, exceptionInfo);	
	if (dynlen(exceptionInfo)) {
		return -1;
	}
	
	// No range defined => Value is valid
	if (range=="")
		return 0;		
		
	// Check if the range is FALSE, TRUE
	if (range=="[FALSE, TRUE]") {
		if (value=="0" || value=="1" || strtoupper(value)=="TRUE" || strtoupper(value)=="FALSE")
			return 0;
		return -1;
    }	
		
    _unRecipeFunctions_getRangeValues(range, min, max, exceptionInfo);
	
    if (dynlen(exceptionInfo)) {
      return -1;
    } 
	
    float fmin, fmax, fval;
    fmin = (float)min;
    fmax = (float)max;
	
	// Removes the white space in the exponential value (1.5 e1 => 1.5e1) before converting to float
	sVal = value;
	strreplace(sVal, " ", "");	
    fval = (float)sVal;

    if (fmin>fval || fval>fmax){
      return -1;
    } 
	
	return 0;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the min. and max. values defined in a range string
 * @param range - [IN] String containing a range definition (i.e.: [0.0, 100.0])
 * @param min - [OUT] Minimum value defined in the range.
 * @param max - [OUT] Maximum value defined in the range.
 * @param exceptionInfo - [OUT] Standard exception handling routine.
 */
void _unRecipeFunctions_getRangeValues(string range, string &min, string &max, dyn_string &exceptionInfo)
{  
  string rangeElement, deviceType, rangeMin, rangeMax;

  min = "";
  max = "";
  
  string tmpRange = range;
  strreplace(tmpRange, "[", "");
  strreplace(tmpRange, "]", "");
  strreplace(tmpRange, " ", "");
  

  dyn_string rangeSplit = strsplit(tmpRange, ",");
  if (dynlen(rangeSplit)!=2)  {
    fwException_raise(exceptionInfo, "ERROR", "The specified range format is incorrect: " + range, "");
    return;
  }

  min = rangeSplit[1];
  max = rangeSplit[2];
  
  strreplace(min, " ", "");
  strreplace(max, " ", "");
  
  min = strrtrim(min);
  min = strltrim(min);
  max = strrtrim(max);
  max = strltrim(max);
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Gets the PLC status of a recipe.
 * @param recipeElements - [IN] List of DPEs.
 * @param bActive - [OUT] True if the PLCs are active, otherwise false.
 * @param statusmessage - [OUT] PLC status message.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_getPlcStatus(dyn_string recipeElements, bool &bActive, 
                                    dyn_string &statusMessage, dyn_string &exceptionInfo)
{
  int iRes;
  bActive = true;
	
  // Get the list of PLCs used in the recipe
  dyn_string plcList;
  for (int i=1; i<=dynlen(recipeElements); i++){
     string plc = unGenericObject_GetPLCNameFromDpName(recipeElements[i]);
     if (!dynContains(plcList, plc)){
       dynAppend(plcList, plc);
     }
  }
	
  //check PLC status
  for (int i=1; i<=dynlen(plcList); i++){
    //sPlcName - corresponding dp name in "_Mod_Plc" internal DPT
    int iSystemIntegrityAlarmValue = 0;
    bool bSystemIntegrityAlarmEnabled = false;
    string sPlcName = plcList[i];	
    
    iRes = dpGet(c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm", iSystemIntegrityAlarmValue);
    //DebugTN("Checking system integrity: " + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm\n");
    if (iRes < 0)	{
      dynAppend(exceptionInfo, "unRecipeFunctions_getPlcStatus: failed to get current state of plc - " + sPlcName +"\n");
  		 return;
    }
	
    iRes = dpGet(c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled", bSystemIntegrityAlarmEnabled);
    //DebugTN("Checking system integrity 2: " + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled\n");
    if (iRes < 0)	{
     	dynAppend(exceptionInfo, "unRecipeFunctions_getPlcStatus: failed to get current state of plc - " + sPlcName+"\n");
  		 return;
    }	
    
    if((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value)) {
      bActive = false;
      dynAppend(statusMessage, sPlcName);
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Callback function executed when the PLC status has changed.
 * The properties table of the recipe instance will be reloaded.
 * @param sAlarmDpe - [IN] DP Name of the PLC alarm state.
 * @param iSystemIntegrityAlarmValue - [IN] Value of the PLC alarm state.
 * @param sEnabledDpe - [IN] DP Name of the system integrity alarm enabled.
 * @param bSystemIntegrityAlarmEnabled - [IN] Value of the system integrity alarm enabled.
 */
void _unRecipeFunctions_plcStatusCB (string sAlarmDpe, int iSystemIntegrityAlarmValue,
									 string sEnabledDpe, bool bSystemIntegrityAlarmEnabled)
{	
	if (!shapeExists("LoadedRecipeDp")) {
		return;
	}

	_unRecipeFunctions_loadPropertiesTableInfo();
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Disconnect from the PLC status of a recipe instance.
 * @param sRcpInstanceDp - [IN] DP Name of the recipe instance.
 * @param exceptionInfo  - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_disconnectPlcStatus(string sRcpInstanceDp, dyn_string &exceptionInfo)
{
  dyn_string dsRecipeElements, dsPlcList;
  dyn_dyn_mixed recipeObject;
  string sPlcName;
  int rc;

  _unRecipeFunctions_getRecipeObjectFromDp(sRcpInstanceDp, recipeObject, exceptionInfo);
  if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return; }  
  dsRecipeElements = recipeObject[fwConfigurationDB_RO_DPE_NAME];
  
  // Get the list of PLCs used in the recipe
  for (int i=1; i<=dynlen(dsRecipeElements); i++){
     sPlcName = unGenericObject_GetPLCNameFromDpName(dsRecipeElements[i]);
     if (!dynContains(dsPlcList, sPlcName)){
       dynAppend(dsPlcList, sPlcName);
     }
  }
  
  // connect to PLC status
  for (int i=1; i<=dynlen(dsPlcList); i++){
    sPlcName = dsPlcList[i];	
	//DebugTN("DpDisconnect: " + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm" );
    rc = dpDisconnect("_unRecipeFunctions_plcStatusCB", 
			  c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm", 
			  c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled" );
  }	
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Establish a DP connection to the PLC status of all the PLCs containing devices of the recipe instance.
 * @param sRcpInstanceDp - [IN] DP Name of the recipe instance.
 * @param exceptionInfo  - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_connectPlcStatus(string sRcpInstanceDp, dyn_string &exceptionInfo)
{
  dyn_string dsRecipeElements, dsPlcList;
  dyn_dyn_mixed recipeObject;
  string sPlcName;

  _unRecipeFunctions_getRecipeObjectFromDp(sRcpInstanceDp, recipeObject, exceptionInfo);
  if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return; }
  
  dsRecipeElements = recipeObject[fwConfigurationDB_RO_DPE_NAME];
  
  // Get the list of PLCs used in the recipe
  for (int i=1; i<=dynlen(dsRecipeElements); i++){
     sPlcName = unGenericObject_GetPLCNameFromDpName(dsRecipeElements[i]);
     if (!dynContains(dsPlcList, sPlcName)){
       dynAppend(dsPlcList, sPlcName);
     }
  }
  
  // connect to PLC status
  for (int i=1; i<=dynlen(dsPlcList); i++){
    //DebugTN("DpConnect: " + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm" );
    sPlcName = dsPlcList[i];	
	dpConnect("_unRecipeFunctions_plcStatusCB", 
			  c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm", 
			  c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled");
  }
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Load the recipe classes linked to the PCO sPcoAlias.
 * @param sPcoAlias - [IN] The PCO alias which recipe classes muat be loaded.
 * @param exceptionInfo - [OUT] Standard exception handling routine.
 * @param bLoadFirstClass - [IN] TRUE if the data of the first recipe class must be loaded in the panel. 
 * @return The number of recipe classes added.
 */
int unRecipeFunctions_loadRecipeClasses(string sPcoAlias, dyn_string &exceptionInfo, bool bLoadFirstClass=TRUE)
{
  int iLines = 0;
  string sType, sDeleteName, sAppendName;
  dyn_string recipeClassNames, recipeClassDps;
  
  if (!shapeExists("RecipeClasses")) {
    fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
    return;
  }
  
  unRecipeFunctions_getRecipeClasses(recipeClassNames, recipeClassDps, sPcoAlias);  
  
  // Get the type of the RecipeClasses element
  getValue("RecipeClasses", "type", sType);
  if (sType=="Table") {
	iLines = _unRecipeFunctions_loadRecipeClassesInTable(recipeClassNames, recipeClassDps, bLoadFirstClass);
  } else if (sType=="ComboBox") {
	iLines = _unRecipeFunctions_loadRecipeClassesInComboBox(recipeClassNames);
  } else {
	_unRecipeFunctions_writeInRecipeLog("Error: The type of the 'RecipeClasses' shape is unknown.");
  }
  
  return iLines;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Load the recipe class list in a table.
 * @param rcpClassNames   - [IN] List containing the recipe class names.
 * @param rcpClassDps     - [IN] List containing the recipe class datapoints.
 * @param bLoadFirstClass - [IN] TRUE if the data of the first recipe class must be loaded in the panel.
 * @return The number of recipe classes added.
 */
int _unRecipeFunctions_loadRecipeClassesInTable(dyn_string rcpClassNames, dyn_string rcpClassDps, bool bLoadFirstClass=TRUE) {
  int i, iLines;
  dyn_int lines;
  string sRcpClass, sRcpDp, sPvssDpe;
  dyn_string exceptionInfo;

  iLines = dynlen(rcpClassNames);
  setValue("RecipeClasses", "deleteAllLines");
  if (iLines > 0) {
    setValue("RecipeClasses", "appendLines", iLines, 
			 "RecipeClass", rcpClassNames,
			 "RecipeClassDp", rcpClassDps);
	// Select the first recipe class in the table
	if (bLoadFirstClass==true) {
		unRecipeFunctions_selectRecipeClass(rcpClassDps[1], exceptionInfo, TRUE);
	}
  } else {
	setValue("RecipeClasses", "appendLine", "RecipeClass", "-None-");
	unRecipeFunctions_clearRecipeClassData();
  }
  
  return iLines;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Load the recipe class list in a combo box.
 * @param rcpClassNames - [IN] List containing the recipe class names.
 * @return The number of recipe classes added.
 */
int _unRecipeFunctions_loadRecipeClassesInComboBox(dyn_string rcpClassNames) {
  int i, iLines;
  
  iLines = dynlen(rcpClassNames);
  setValue("RecipeClasses", "deleteAllItems");
  if (iLines > 0) {
    for (int i=1; i<=iLines; i++) {
	  setValue("RecipeClasses", "appendItem", rcpClassNames[i]);
	}
  } else {
	setValue("RecipeClasses", "appendItem", "-None-");
  }
  
  return iLines;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Clear all the data from the recipe class panel.
 */
void unRecipeFunctions_clearRecipeClassData() {
  if (shapeExists("RecipeClass")) RecipeClass.text = "";
	
  if (shapeExists("RecipeType")) RecipeType.text = "";
	
  if (shapeExists("RecipeClassComment")) RecipeClassComment.text = "";
	
  if (shapeExists("LastModifiedTime")) LastModifiedTime.text = "";
	
  if (shapeExists("LastModifiedUser")) LastModifiedUser.text = "";
	
  if (shapeExists("LastActivatedRecipe")) LastActivatedRecipe.text = "";
  
  if (shapeExists("LastActivationTime")) LastActivationTime.text = "";
  
  if (shapeExists("LastActivationUser")) LastActivationUser.text = "";
  
  if (shapeExists("LoadedRcpDp")) LoadedRcpDp.text = "";
	
  if (shapeExists("RCCElements.RecipeElementsTable")) 
	setValue("RCCElements.RecipeElementsTable", "deleteAllLines");
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Create a predefined recipe instance to store the default privileges.
 * @param sClassName - [IN] Recipe class name.
 * @param sAppName   - [IN] Application name.
 * @param exceptionInfo - [OUT] Standar exception handling variable.
 */
void _unRecipeFunctions_createPredefinedRecipeInstance(string sClassName, string sAppName, dyn_string &exceptionInfo) {
  bool bEvent;
  int iDeviceNumber;
  string sDpName, sOperatorPrivileges, sExpertPrivileges, sAdminPrivileges, sPvssDpe, sPrefix, sFrontEndName;
  dyn_string dsOperatorPrivileges, dsExpertPrivileges, dsAdminPrivileges;
  
  // Check if the predefined recipe instance already exists
  sDpName = _unGenericDpFunctions_dpAliasToName(sClassName + "/" + UN_RCP_PREDEFINED_NAME);
  if (sDpName != "") {
    // The predefined recipe of the class already exists
	return;
  }
  
  // Get the recipe front end name
  unImportDevice_initialize();
  unImportDevice_getApplicationCharacteristics(bEvent, sPrefix);
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(unRecipeFunctions_RcpFEAlias, sPvssDpe);
  if (sPvssDpe==unRecipeFunctions_RcpFEAlias) {
	// The RCP_FE doesn't exist, create it
	bool bOk = unRecipeFunctions_createRecipeFrontEnd(sAppName, sPrefix, bEvent, exceptionInfo);
	if (bOk == false) {
		fwException_raise(exceptionInfo, "ERROR", "The " + unRecipeFunctions_RcpFEAlias + " SOFT_FE couldn't be created.", "");
		return;
	}
	unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(unRecipeFunctions_RcpFEAlias, sPvssDpe);
  }

  _unRecipeFunctions_createNewRecipeDp(UN_CONFIG_INTERNAL_UNRCPINSTANCE_DPT_NAME, unRecipeFunctions_RcpFEAlias, sAppName, sDpName, iDeviceNumber); 
  dpCreate(sDpName, UN_CONFIG_INTERNAL_UNRCPINSTANCE_DPT_NAME);
  dpSetAlias(sDpName + ".", sClassName + "/" + UN_RCP_PREDEFINED_NAME);
  
  // Set the default privileges for the default recipe instance
  _unRecipeFunctions_getDeviceDefaultPrivileges(UN_CONFIG_UNRCPINSTANCE_DPT_NAME, sOperatorPrivileges, sExpertPrivileges, 
		sAdminPrivileges, exceptionInfo);
		
  dpSet(sDpName + ".accessControl.operator", sOperatorPrivileges,
	    sDpName + ".accessControl.expert", sExpertPrivileges,
		sDpName + ".accessControl.admin", sAdminPrivileges);
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Delete the predefined recipe instance of a recipe class.
 * @param sClassName - [IN] Recipe class name.
 * @return TRUE if the predefined recipe instance was deleted correctly, otherwise FALSE.
 */
bool _unRecipeFunctions_deletePredefinedRecipeInstance(string sClassName) {
  int iRetVal;
  string sDpName, sPvssDpe;
  
  // Check if the predefined recipe instance already exists
  sDpName = _unGenericDpFunctions_dpAliasToName(sClassName + "/" + UN_RCP_PREDEFINED_NAME);
  if (sDpName == "") {
    // The predefined recipe of the class doesn't exist
    return false;
  }
  
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(sDpName, sPvssDpe);
  iRetVal = dpDelete(sPvssDpe);
  
  return (iRetVal==0);
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the default privileges of a recipe device (recipe class or recipe instance).
 * @param sDpType 			  - [IN] Datapoint type which default privileges are required.
 * @param sOperatorPrivileges - [OUT] Default privileges for the operator.
 * @param sExpertPrivileges   - [OUT] Default privileges for the expert.
 * @param sAdminPrivileges    - [OUT] Default privileges for the administrator.
 * @param exceptionInfo       - [OUT] Standard exception handling variable.
 */
_unRecipeFunctions_getDeviceDefaultPrivileges(string sDpType, string & sOperatorPrivileges, string & sExpertPrivileges, 
		string & sAdminPrivileges, dyn_string exceptionInfo) 
{
  string sFunction;
  dyn_string dsOperatorPrivileges, dsExpertPrivileges, dsAdminPrivileges;
  
  if (sDpType == UN_CONFIG_UNRCPCLASS_DPT_NAME) {
	sFunction = "_unRecipeFunctions_getDefaultClassPrivileges";
  } else if (sDpType == UN_CONFIG_UNRCPINSTANCE_DPT_NAME) {
    sFunction = "_unRecipeFunctions_getDefaultInstancePrivileges";
  } else {
    fwException_raise(exceptionInfo, "ERROR", "_unRecipeFunctions_getDeviceDefaultPrivileges - The DPType " +sDpType+ 
		" is unknown." ,"");
	DebugTN(exceptionInfo);
	return;
  }
  
  string sFunctionDefinition = "dyn_string main(string sRoleType)"+
  										"{	dyn_string temp;"+
  										"	"+sFunction+"(sRoleType, temp);"+
  										"	return temp;"+
  										"}";
										
  evalScript(dsOperatorPrivileges, sFunctionDefinition, makeDynString(), "Operator");     
  evalScript(dsExpertPrivileges, sFunctionDefinition, makeDynString(), "Expert");  
  evalScript(dsAdminPrivileges, sFunctionDefinition, makeDynString(), "Admin");  
  fwGeneral_dynStringToString(dsOperatorPrivileges, sOperatorPrivileges, ",");
  fwGeneral_dynStringToString(dsExpertPrivileges, sExpertPrivileges, ",");
  fwGeneral_dynStringToString(dsAdminPrivileges, sAdminPrivileges, ",");
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Update the recipe class privileges.
 * @param sDpName - [IN] Recipe class DP name.
 */
void unRecipeFunctions_updateRecipeClassPrivileges(string sDpName) {
  int iPos;
  string sDeviceOperatorPrivileges, sDeviceExpertPrivileges, sDeviceAdminPrivileges;
  string sDefaultOperatorPrivileges, sDefaultExpertPrivileges, sDefaultAdminPrivileges;
  dyn_string dsDeviceOperatorPrivileges, dsDeviceExpertPrivileges, dsDeviceAdminPrivileges;
  dyn_string dsDefaultOperatorPrivileges, dsDefaultExpertPrivileges, dsDefaultAdminPrivileges;
  
  // Get the default privileges for the recipe class
  _unRecipeFunctions_getDefaultClassPrivileges("operator", dsDefaultOperatorPrivileges);
  _unRecipeFunctions_getDefaultClassPrivileges("expert", dsDefaultExpertPrivileges);
  _unRecipeFunctions_getDefaultClassPrivileges("admin", dsDefaultAdminPrivileges);
  fwGeneral_dynStringToString(dsDefaultOperatorPrivileges, sDefaultOperatorPrivileges, ",");
  fwGeneral_dynStringToString(dsDefaultExpertPrivileges, sDefaultExpertPrivileges, ",");
  fwGeneral_dynStringToString(dsDefaultAdminPrivileges, sDefaultAdminPrivileges, ",");
  
  // Get the device privileges
  dpGet(sDpName + ".statusInformation.accessControl.operator", sDeviceOperatorPrivileges,
	    sDpName + ".statusInformation.accessControl.expert", sDeviceExpertPrivileges,
		sDpName + ".statusInformation.accessControl.admin", sDeviceAdminPrivileges);
		  		  
  if (sDeviceOperatorPrivileges=="" && sDeviceExpertPrivileges=="" && sDeviceAdminPrivileges=="") {
	// Apply the default privileges
	dpSet(sDpName + ".statusInformation.accessControl.operator", sDefaultOperatorPrivileges,
		  sDpName + ".statusInformation.accessControl.expert", sDefaultExpertPrivileges,
		  sDpName + ".statusInformation.accessControl.admin", sDefaultAdminPrivileges);
  } else {
	// There are some privileges for the device, add the new privileges
	dsDeviceOperatorPrivileges = strsplit(sDeviceOperatorPrivileges, ",");
	dsDeviceExpertPrivileges   = strsplit(sDeviceExpertPrivileges, ",");
	dsDeviceAdminPrivileges    = strsplit(sDeviceAdminPrivileges, ",");
		
	// Replace the 'Apply' privilege by 'ApplyChanges' (from version 1.1.x to 1.2.0)
	if ( (iPos = dynContains(dsDeviceOperatorPrivileges, "Apply")) > 0)
		dsDeviceOperatorPrivileges[iPos] = "ApplyChanges";
	else if ( (iPos = dynContains(dsDeviceExpertPrivileges, "Apply")) > 0)
		dsDeviceExpertPrivileges[iPos] = "ApplyChanges";
	else if ( (iPos = dynContains(dsDeviceAdminPrivileges, "Apply")) > 0)
		dsDeviceAdminPrivileges[iPos] = "ApplyChanges";
			
	_unRecipeFunctions_addPrivileges(dsDefaultOperatorPrivileges, dsDeviceOperatorPrivileges, dsDeviceExpertPrivileges, dsDeviceAdminPrivileges);
	_unRecipeFunctions_addPrivileges(dsDefaultExpertPrivileges, dsDeviceExpertPrivileges, dsDeviceOperatorPrivileges, dsDeviceAdminPrivileges);
	_unRecipeFunctions_addPrivileges(dsDefaultAdminPrivileges, dsDeviceAdminPrivileges, dsDeviceOperatorPrivileges, dsDeviceExpertPrivileges);
		
	// Remove the operator obsolete privileges 
	_unRecipeFunctions_removeObsoletePrivileges(dsDeviceOperatorPrivileges, dsDefaultOperatorPrivileges,
			dsDefaultExpertPrivileges, dsDefaultAdminPrivileges);

	// Remove the expert obsolete privileges 
	_unRecipeFunctions_removeObsoletePrivileges(dsDeviceExpertPrivileges, dsDefaultOperatorPrivileges,
			dsDefaultExpertPrivileges, dsDefaultAdminPrivileges);
				
	// Remove the admin obsolete privileges 
	_unRecipeFunctions_removeObsoletePrivileges(dsDeviceAdminPrivileges, dsDefaultOperatorPrivileges, 
				dsDefaultExpertPrivileges, dsDefaultAdminPrivileges);	
				
	fwGeneral_dynStringToString(dsDeviceOperatorPrivileges, sDeviceOperatorPrivileges, ",");
	fwGeneral_dynStringToString(dsDeviceExpertPrivileges, sDeviceExpertPrivileges, ",");
	fwGeneral_dynStringToString(dsDeviceAdminPrivileges, sDeviceAdminPrivileges, ",");
		
	// Apply the device privileges
	dpSet(sDpName + ".statusInformation.accessControl.operator", sDeviceOperatorPrivileges,
		  sDpName + ".statusInformation.accessControl.expert", sDeviceExpertPrivileges,
		  sDpName + ".statusInformation.accessControl.admin", sDeviceAdminPrivileges);
  }
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Update the recipe instances privileges.
 * @param sDpName - [IN] DP name of the recipe instance.
 */
void unRecipeFunctions_updateRecipeInstancePrivileges(string sDpName) {
  string sDeviceOperatorPrivileges, sDeviceExpertPrivileges, sDeviceAdminPrivileges;
  string sDefaultOperatorPrivileges, sDefaultExpertPrivileges, sDefaultAdminPrivileges;
  dyn_string dsDefaultOperatorPrivileges, dsDefaultExpertPrivileges, dsDefaultAdminPrivileges;
  dyn_string dsDeviceOperatorPrivileges, dsDeviceExpertPrivileges, dsDeviceAdminPrivileges;

  // Get the default privileges for the recipe instance
  _unRecipeFunctions_getDefaultInstancePrivileges("operator", dsDefaultOperatorPrivileges);
  _unRecipeFunctions_getDefaultInstancePrivileges("expert", dsDefaultExpertPrivileges);
  _unRecipeFunctions_getDefaultInstancePrivileges("admin", dsDefaultAdminPrivileges);
  fwGeneral_dynStringToString(dsDefaultOperatorPrivileges, sDefaultOperatorPrivileges, ",");
  fwGeneral_dynStringToString(dsDefaultExpertPrivileges, sDefaultExpertPrivileges, ",");
  fwGeneral_dynStringToString(dsDefaultAdminPrivileges, sDefaultAdminPrivileges, ",");

  // Get the device privileges
  dpGet(sDpName + ".statusInformation.accessControl.operator", sDeviceOperatorPrivileges,
	    sDpName + ".statusInformation.accessControl.expert", sDeviceExpertPrivileges,
		sDpName + ".statusInformation.accessControl.admin", sDeviceAdminPrivileges);
		  		  
	if (sDeviceOperatorPrivileges=="" && sDeviceExpertPrivileges=="" && sDeviceAdminPrivileges=="") {
	  // Apply the default privileges
	  int iRes = dpSet(
		sDpName + ".statusInformation.accessControl.operator", sDefaultOperatorPrivileges,
		sDpName + ".statusInformation.accessControl.expert", sDefaultExpertPrivileges,
		sDpName + ".statusInformation.accessControl.admin", sDefaultAdminPrivileges);

	  if (iRes<0) {
		dyn_errClass err = getLastError();
			DebugTN(err);
	  }
	} else {
	  // There are some privileges for the device, add the new privileges
	  dsDeviceOperatorPrivileges = strsplit(sDeviceOperatorPrivileges, ",");
	  dsDeviceExpertPrivileges   = strsplit(sDeviceExpertPrivileges, ",");
	  dsDeviceAdminPrivileges    = strsplit(sDeviceAdminPrivileges, ",");
			
	  // Add the operator default privileges
	  _unRecipeFunctions_addPrivileges(dsDefaultOperatorPrivileges, dsDeviceOperatorPrivileges,
			dsDeviceExpertPrivileges, dsDeviceAdminPrivileges);
				
	  // Add the expert default privileges
	  _unRecipeFunctions_addPrivileges(dsDefaultExpertPrivileges, dsDeviceExpertPrivileges,
			dsDeviceOperatorPrivileges, dsDeviceAdminPrivileges);
				
	  // Add the admin default privileges
	  _unRecipeFunctions_addPrivileges(dsDefaultAdminPrivileges, dsDeviceAdminPrivileges,
			dsDeviceOperatorPrivileges, dsDeviceExpertPrivileges);
		
	  // Remove the operator obsolete privileges 
	  _unRecipeFunctions_removeObsoletePrivileges(dsDeviceOperatorPrivileges, dsDefaultOperatorPrivileges,
			dsDefaultExpertPrivileges, dsDefaultAdminPrivileges);

	  // Remove the expert obsolete privileges 
	  _unRecipeFunctions_removeObsoletePrivileges(dsDeviceExpertPrivileges, dsDefaultOperatorPrivileges,
			dsDefaultExpertPrivileges, dsDefaultAdminPrivileges);
				
	  // Remove the admin obsolete privileges 
	  _unRecipeFunctions_removeObsoletePrivileges(dsDeviceAdminPrivileges, dsDefaultOperatorPrivileges, 
				dsDefaultExpertPrivileges, dsDefaultAdminPrivileges);				
		
	  fwGeneral_dynStringToString(dsDeviceOperatorPrivileges, sDeviceOperatorPrivileges, ",");
	  fwGeneral_dynStringToString(dsDeviceExpertPrivileges, sDeviceExpertPrivileges, ",");
	  fwGeneral_dynStringToString(dsDeviceAdminPrivileges, sDeviceAdminPrivileges, ",");
		
	  // Apply the device privileges
	  int iRes = dpSet(
		  sDpName + ".statusInformation.accessControl.operator", sDeviceOperatorPrivileges,
		  sDpName + ".statusInformation.accessControl.expert", sDeviceExpertPrivileges,
		  sDpName + ".statusInformation.accessControl.admin", sDeviceAdminPrivileges);
	}
}


//------------------------------------------------------------------------------------------------------------------------
/** 
 * Auxiliary function to add the default privileges to the privileges list.
 * If none of the dyn_strings (dsDevicePrivileges1, dsDevicePrivileges2, dsDevicePrivileges3) contains the privileges
 * included in dsDefaultPrivileges, the privilege will be added to dsDevicePrivileges1. Otherwise (the privilege is included
 * in any dyn_string) no actions will be done.
 * @param dsDefaultPrivileges - [IN] The list of privileges to add.
 * @param dsDevicePrivileges1 - [OUT] The list where add the privileges if they don't exist in any dyn_string.
 * @param dsDevicePrivileges2 - [IN] Device privileges list.
 * @param dsDevicePrivileges3 - [IN] Device privileges list.
 */
void _unRecipeFunctions_addPrivileges(dyn_string dsDefaultPrivileges, dyn_string &dsDevicePrivileges1, 
					dyn_string dsDevicePrivileges2, dyn_string dsDevicePrivileges3)
{
  int i, iLen;
  string sPrivilege;
  
  iLen = dynlen(dsDefaultPrivileges);
  for (i=1; i<=iLen; i++) {
    sPrivilege = dsDefaultPrivileges[i];
	
	if (dynContains(dsDevicePrivileges1, sPrivilege))
		continue;
		
	if (dynContains(dsDevicePrivileges2, sPrivilege))
		continue;
		
	if (dynContains(dsDevicePrivileges2, sPrivilege))
		continue;
		
	// The current privilege is not assigned to any role,
	// add it to the default role.
	dynAppend(dsDevicePrivileges1, sPrivilege);
  }
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Auxiliary function to remove the obsolete privileges from the privileges list.
 * @param dsDevicePrivileges - [OUT] List where the obsolete parameters will be removed.
 * @param dsDefaultPrivileges1 - [IN] Default privileges list 1 (e.g. operator).
 * @param dsDefaultPrivileges2 - [IN] Default privileges list 2 (e.g. expert). 
 * @param dsDefaultPrivileges3 - [IN] Default privileges list 3 (e.g. admin).
 */
void _unRecipeFunctions_removeObsoletePrivileges(dyn_string & dsDevicePrivileges, dyn_string dsDefaultPrivileges1, 
				dyn_string dsDefaultPrivileges2, dyn_string dsDefaultPrivileges3) 
{
	int i, iLen;
	
	iLen = dynlen(dsDevicePrivileges);
	for (i=1; i<=iLen; i++) {
	  string sPrivilege = dsDevicePrivileges[i];
	  
	  if (dynContains(dsDefaultPrivileges1, sPrivilege))
		continue;
		
	  if (dynContains(dsDefaultPrivileges2, sPrivilege))
		continue;
		
	  if (dynContains(dsDefaultPrivileges3, sPrivilege))
		continue;
		
	  // Remove the obsolete privilege
	  dynRemove(dsDevicePrivileges, i);
	  iLen--;
	  i--;
	}
}
//------------------------------------------------------------------------------------------------------------------------
/**
 * Get the device mapping from a recipe class.
 * @param rcpClassObject - [IN] The recipe class object from the JCOP recipe class definition.
 * @param rcpDeviceDpes  - [OUT] Mapping where: Key=device name, Value=list of device DPEs included in the recipe.
 */
void _unRecipeFunctions_getRecipeClassDeviceMapping(dyn_mixed rcpClassObject, mapping & rcpDeviceDpes) {
  int i, iLen;
  dyn_string rcpElements;
  
  // For each device in the recipe, get all the DPEs included in the recipe and
  // store them in the mapping rcpDeviceDpes as a string (comma separated)
  rcpElements = rcpClassObject[fwConfigurationDB_RCL_ELEMENTS];
  iLen = dynlen(rcpElements);
  for (i=1; i<=iLen; i++) {
	string sRcpElement = rcpElements[i];
    string sAlias = substr(sRcpElement, 0, strpos(sRcpElement, "."));
	dyn_string rcpElementSplit = strsplit(sRcpElement, ".");
	string rcpAliasDpes = "";
	if (mappingHasKey(rcpDeviceDpes, sAlias)) {
	  rcpAliasDpes = rcpDeviceDpes[sAlias];
	}
	
	if (strlen(rcpAliasDpes))
	  rcpAliasDpes += ", ";
	rcpAliasDpes += rcpElementSplit[dynlen(rcpElementSplit)];
	rcpDeviceDpes[sAlias] = rcpAliasDpes;
  }
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Load the recipe class data in the recipe class panel.
 * @param sRcpClassName - [IN] Name of the recipe class which data will be loaded in the panel.
 */
void unRecipeFunctions_loadRecipeClassData(string sRcpClassName)
{
  int i, j, iLen;
  dyn_mixed recipeClassInfo;
  dyn_string rcpElements, exceptionInfo;
  mapping rcpDeviceDpes;
  
  unRecipeFunctions_getRecipeClassObject(sRcpClassName, recipeClassInfo, exceptionInfo);
  
  RecipeClass.text         = recipeClassInfo[fwConfigurationDB_RCL_CLASSNAME];
  RecipeType.text          = recipeClassInfo[fwConfigurationDB_RCL_RECIPETYPE];
  RecipeClassComment.text  = recipeClassInfo[fwConfigurationDB_RCL_DESCRIPTION];
  LastModifiedTime.text    = recipeClassInfo[fwConfigurationDB_RCL_LAST_MODIFIED_TIME];
  LastModifiedUser.text    = recipeClassInfo[fwConfigurationDB_RCL_LAST_MODIFIED_USER];
  LastActivatedRecipe.text = recipeClassInfo[fwConfigurationDB_RCL_LAST_ACTIVATED_RECIPE];
  LastActivationTime.text  = recipeClassInfo[fwConfigurationDB_RCL_LAST_ACTIVATED_TIME];
  LastActivationUser.text  = recipeClassInfo[fwConfigurationDB_RCL_LAST_ACTIVATED_USER];
  
  // For each device in the recipe, get all the DPEs included in the recipe and
  // store them in the mapping rcpDeviceDpes as a string (comma separated)
  _unRecipeFunctions_getRecipeClassDeviceMapping(recipeClassInfo, rcpDeviceDpes);
  
  // Add the recipe elements to the table
  setValue("RCCElements.RecipeElementsTable", "deleteAllLines");
  dyn_string devices = recipeClassInfo[fwConfigurationDB_RCL_DEVICES];
  int lines = dynlen(devices);
  if (lines>0) {
    int i;    
	setValue("RCCElements.RecipeElementsTable", "enabled", FALSE);
    for (i=1; i<=dynlen(devices); i++) {
      string deviceType, pvssDpe, sAlias;
      unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(devices[i], pvssDpe);
      deviceType = dpTypeName(pvssDpe);
	  sAlias = devices[i];
      setValue("RCCElements.RecipeElementsTable", "appendLine",
			"DeviceType", deviceType, 
			"DeviceName", sAlias, 
			"DeviceElements", rcpDeviceDpes[sAlias]);
    }   
    setValue("RCCElements.RecipeElementsTable", "enabled", TRUE);	
  }
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Select the specified recipe class in the recipe class panel.
 * @param sRcpClassDp   - [IN] Datapoint name of the recipe class.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 * @param bForceReload  - [IN] Boolean value to force the reload of the recipe class data.
 */ 
bool unRecipeFunctions_selectRecipeClass(string sRcpClassDp, dyn_string &exceptionInfo, bool bForceReload=FALSE)
{
  int iPos, iRes;  
  string sClassName, sButtonPanel, sLoadedRcpDp;
  dyn_string classNames;
  
  // If the sRcpClassDp ends with ".", remove the dot from the DP
  if (sRcpClassDp[(strlen(sRcpClassDp)-1)] == ".") {
	sRcpClassDp = substr(sRcpClassDp, 0, strlen(sRcpClassDp)-1);
  }
 
  sClassName = unGenericDpFunctions_getAlias(sRcpClassDp + ".");
  getMultiValue("RecipeClasses", "getColumnN", 0, classNames,
				"LoadedRcpDp", "text", sLoadedRcpDp);
  
  iPos = dynContains(classNames, sClassName);
  if (iPos<=0) {
	//DebugTN("The recipe class " + sClassName + " doesn't exist in the table.");
	//DebugTN(classNames);
	return;
  }

  // Check if the recipe class is already loaded or the reload has been forced
  if (sRcpClassDp != sLoadedRcpDp || bForceReload)
  {	
    RecipeClasses.selectLineN(iPos-1);
	
	if (shapeExists("RCCElements")) {
	  removeSymbol(myModuleName(), myPanelName(), "RCCElements");
	}

    addSymbol(myModuleName(), myPanelName(), "vision/unRecipe/UnRcpClass/unUnRcpClass_ClassElements.pnl",
				"RCCElements", makeDynString("$sDpName:"+sRcpClassDp), 190, 220, 0, 1, 1);
	
	unRecipeFunctions_loadRecipeClassData(sClassName);
	setValue("LoadedRcpDp", "text", sRcpClassDp);		
  }
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the recipe class data from a recipe instance datapoint.
 * @param sRcpInstanceDp - [IN] Datapoint name of the recipe instance.
 * @param sRcpClassName  - [OUT] Name of the recipe class related to the recipe instance.
 * @param sRcpClassDp    - [OUT] Datapoint name of the recipe class related to the recipe instance.
 * @param sPcoLink       - [OUT] Device link of the recipe class.
 */
void _unRecipeFunctions_getRecipeClassDataFromInstance(string sRcpInstanceDp, string & sRcpClassName, string & sRcpClassDp,
		string & sPcoLink) 
{
  // Get the recipe class name of the recipe instance
  //DebugTN("_unRecipeFunctions_getRecipeClassDataFromInstance: " + sRcpInstanceDp + ".ProcessInput.ClassName");
  dpGet(sRcpInstanceDp + ".ProcessInput.ClassName", sRcpClassName);
  
  // Get the recipe class datapoint name and the recipe class pco link 
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(sRcpClassName, sRcpClassDp);
  dpGet(sRcpClassDp + "statusInformation.deviceLink:_online.._value", sPcoLink);
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Selects the specified recipe instance in the recipe instance panel.
 * @param sRcpInstanceDp - [IN] Datapoint of the recipe instance which will be selected.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
bool unRecipeFunctions_selectRecipeInstance(string sRcpInstanceDp, dyn_string &exceptionInfo)
{
  int iPos, iSelected, pcoPos;
  dyn_int selectedLines;
  bool bInitial, bExists;
  string sFaceplatePanel, sButtonPanel, sClassName, sInstanceName, sLoadedRcpDp, sPreviousRcpDp, sState;
  string sRcpClassDp, sPcoLink, sSelectedPco, sJcopRcpClassDp;
  dyn_string rcpClassesLoaded, rcpInstancesDp, pcoItems;  
  
  //DebugTN("START = unRecipeFunctions_selectRecipeInstance " + dpGetAlias(sRcpInstanceDp + ".") + "\n");
  
  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeClasses") || !shapeExists("RecipeInstances") || 
      !shapeExists("RecipeClassLabel") || !shapeExists("RecipeInstanceLabel")) 
  {
	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	return false;
  }

  // Check if the DP name exists
  if (!dpExists(sRcpInstanceDp)) {
    fwException_raise(exceptionInfo, "ERROR", "The DP name " + sRcpInstanceDp + " doesn't exist.","");
	return;
  }
  
  // Get the recipe class data from the recipe instance datapoint
  _unRecipeFunctions_getRecipeClassDataFromInstance(sRcpInstanceDp, sClassName, sRcpClassDp, sPcoLink);
  
  // Get the recipe instance name and recipe instance state
  dpGet(sRcpInstanceDp+".ProcessInput.InstanceName", sInstanceName,
		sRcpInstanceDp+".ProcessInput.State", sState);
  
  // Selects the PCO link in the combo box (if necessary)
  getValue("PcoList", "selectedText", sSelectedPco);
  if (sPcoLink != sSelectedPco) {  
    string sPcoDp;
    unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(sPcoLink, sPcoDp);
	unRecipeFunctions_selectPcoRecipes(sPcoDp, exceptionInfo, false);
  } 
  
  // Get the recipe classes loaded in the combo box
  getValue("RecipeClasses", "items", rcpClassesLoaded);
  iPos = dynContains(rcpClassesLoaded, sClassName);  
  if (iPos<=0) {
    // The recipe class of the recipe instance doesn't exist!
    iPos = dynContains(rcpClassesLoaded, "-None-");
    if (iPos<=0) {
      setValue("RecipeClasses", "appendItem", "-None-");
      setValue("RecipeClasses", "selectedPos", dynlen(rcpClassesLoaded)+1);
    } else {
      setValue("RecipeClasses", "selectedPos", iPos);
    }
    return false;
  } 
  
  getMultiValue("RecipeClasses", "items", rcpClassesLoaded,
			    "RecipeClasses", "selectedPos", iSelected);
  iPos = dynContains(rcpClassesLoaded, sClassName);
  
  // Selects the recipe class in the combo box
  if (iPos!=iSelected) {   
    setValue("RecipeClasses", "selectedPos", iPos);
    _unRecipeFunctions_loadRecipeInstances(sClassName);
  } 
    
  // Selects the recipe instance in the table
  getMultiValue("RecipeInstances", "getSelectedLines", selectedLines,
				"RecipeInstances", "getColumnN", 1, rcpInstancesDp,
				"LoadedRecipeDp", "text", sLoadedRcpDp);
  iPos = dynContains(rcpInstancesDp, sRcpInstanceDp);
  if (iPos<=0) { 
	// The required recipe doesn't exist, an error occurred in the dynContains method
	// or there aren't selected lines in the table
	return false;
  } 
  
  if (dynlen(selectedLines)<=0 || (iPos-1)!=selectedLines[1] || sLoadedRcpDp=="" || sLoadedRcpDp!=sRcpInstanceDp) {
	  setValue("RecipeInstances", "selectLineN", (iPos-1));
   
	  // add  device action
	  if (shapeExists("buttonActivate")) {
		removeSymbol(myModuleName(), myPanelName(), "RCPButton");
		removeSymbol(myModuleName(), myPanelName(), "RCPElementsTable");
	  }
			
	  // get the button panel of the device from UNICOS configuration
	  unGenericDpFunctions_getContextPanel(sRcpInstanceDp, sButtonPanel, exceptionInfo);
	  if(dynlen(exceptionInfo)) {DebugTN(exceptionInfo); return;}

	  addSymbol(myModuleName(), myPanelName(), sButtonPanel, "RCPButton", makeDynString("$sDpName:"+sRcpInstanceDp), 95, 620, 0, 1, 1);
	  addSymbol(myModuleName(), myPanelName(), "vision/unRecipe/UnRcpInstance/unUnRcpInstance_RecipeElements.pnl", 
				"RCPElementsTable", makeDynString(), 190, 193, 0, 1, 1);

	  unRecipeFunctions_loadRecipeInstanceData(); 
	  setMultiValue("RecipeClassLabel", "text", sClassName,
					"RecipeInstanceLabel", "text", sInstanceName,
					"LoadedRecipeDp", "text", sRcpInstanceDp);	
  } 
	  
  // Forces the reload of the recipe state to animate the Select button
  dpSet(sRcpInstanceDp+".ProcessInput.State", sState);

  // Animate the Selection Progress Bar
  startThread("unRecipeFunctions_animateSelectionProgressBar", sRcpInstanceDp);
  
  //DebugTN("END = unRecipeFunctions_selectRecipeInstance " + dpGetAlias(sRcpInstanceDp + ".") + "\n");
  
  return true;
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Callback function executed when the recipe class elements have been modified.
 * The function will enable a warning message to indicate that the recipe must be reloaded.
 */
void _unRecipeFunctions_dpConnectRcpClassElementsCB(string sRcpClassElementsDp, dyn_string dsValues) {
  _unRecipeFunctions_enableRecipeInstanceWarning(true);
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Function to show/hide the recipe instance warning message.
 * @param bEnable - [IN] Flag to specify the visibility of the warning message.
 */
void _unRecipeFunctions_enableRecipeInstanceWarning(bool bEnable) {
  if (shapeExists("WarningIcon"))
	setValue("WarningIcon", "visible", bEnable);
	
  if (shapeExists("WarningIcon"))
	setValue("WarningText", "visible", bEnable);	
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Function to select the PCO recipes in the recipe instance panel.
 * @param sPcoDp - [IN] DP Name of the PCO.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 * @param bSelectFirstInstance - [IN] Flag to specify if the first recipe must be selected (TRUE by default).
 */
void unRecipeFunctions_selectPcoRecipes(string sPcoDp, dyn_string &exceptionInfo, bool bSelectFirstInstance=true) {
  int iPos, iNumRecipes;
  string sPcoAlias, sRcpClassName, sRcpInstanceDp;
  dyn_string dsPcoList, dsRcpClasses, dsRcpInstances;
  
  if (!shapeExists("PcoList") || !shapeExists("RecipeInstances") || !shapeExists("PcoLabel")) {
    fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	return;
  }
    
  if (sPcoDp[(strlen(sPcoDp)-1)]!=".")
  	sPcoDp = sPcoDp + ".";	
	
  sPcoAlias = unGenericDpFunctions_getAlias(sPcoDp);
  PcoLabel.text = sPcoAlias;
  //DebugTN("PCO Dp: " + sPcoDp, "PCO Alias: " + sPcoAlias);
 
  // Get the PCOs loaded in the combo box and select the sPcoAlias if it exists
  _unRecipeFunctions_loadPcoList();
  getValue("PcoList", "items", dsPcoList);
  iPos = dynContains(dsPcoList, sPcoAlias);
  if (iPos>0) {
	setValue("PcoList", "selectedPos", iPos);
  } else if (dynlen(dsPcoList)>1) {
	// The selected PCO doesn't exist, select the first PCO in the Combo box
	setValue("PcoList", "selectedPos", 1);
	sPcoAlias = dsPcoList[1];
  } else {
	// There are no PCOs loaded in the combo box, return
	return;
  }
  
  // Load the recipe classes for the selected PCO
  unRecipeFunctions_loadRecipeClasses(sPcoAlias, exceptionInfo);
  
  // Load the recipe instances of the selected recipe class
  getValue("RecipeClasses", "items", dsRcpClasses);  
  sRcpClassName = dsRcpClasses[1];
  if (sRcpClassName == "-None-") {
	//DebugTN("Deleting all lines in the RecipeInstances table");
	setValue("RecipeInstances", "deleteAllLines");
	return; 
  } 
  
  _unRecipeFunctions_loadRecipeInstances(sRcpClassName);

  // Select the first recipe instance (if it exists)
  getValue("RecipeInstances", "lineCount", iNumRecipes);
  if (bSelectFirstInstance && iNumRecipes>0) {
	getValue("RecipeInstances", "cellValueRC", 0, "RecipeInstancesDP", sRcpInstanceDp);
	//DebugTN("Select recipe instance: " + sRcpInstanceDp);
	unRecipeFunctions_selectRecipeInstance(sRcpInstanceDp, exceptionInfo);
  }
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Method used to animate the selection progress bar in the recipe instance panel.
 * @param sDpName - [IN] Data point name of the selected recipe instance.
 */
void unRecipeFunctions_animateSelectionProgressBar (string sDpName) 
{
  int iTimeOut;
  dyn_string exceptionInfo;
  string sRemainingTime, sSelectedManager, sSelectedRecipeDp;
  bool bSelect;

  // Get the selected recipe instance in the panel
  unRecipeFunctions_getSelectedRecipeDp(sSelectedRecipeDp, exceptionInfo); 
  if (sDpName != sSelectedRecipeDp) {
	unProgressBar_setPosition("SelectionProgressBar", 0);  
	return;
  }
	
  /** Selection Progress Bar animation **/
  if (shapeExists("SelectionProgressBar") && shapeExists("buttonSelect")) {
    delay (0, 50);
    dpGet(sDpName + ".statusInformation.selectedManager", sSelectedManager);
    bSelect = (sSelectedManager != "");
    if (bSelect) {
      dpGet(getSystemName() + "_unSelectDeselect.remainingTime", sRemainingTime,
		         getSystemName() + "_unSelectDeselect.timeOut", iTimeOut);      
      
      unProgressBar_setMax("SelectionProgressBar", iTimeOut);
      unProgressBar_setPosition("SelectionProgressBar", iTimeOut);

	  while (strsplit(sRemainingTime, ";")[3] != "0" && bSelect) {
		unRecipeFunctions_getSelectedRecipeDp(sSelectedRecipeDp, exceptionInfo);
		if (sDpName != sSelectedRecipeDp) {
			return;
		}
        dpGet(getSystemName() + "_unSelectDeselect.remainingTime", sRemainingTime);
        unProgressBar_setPosition("SelectionProgressBar", strsplit(sRemainingTime, ";")[3]);
		delay(5);
		// If the DP was removed, return
		if (!dpExists(sDpName + ".statusInformation.selectedManager"))
			return;
		dpGet(sDpName + ".statusInformation.selectedManager", sSelectedManager);
		bSelect = (sSelectedManager != "");
	  }
    }
    
	unRecipeFunctions_getSelectedRecipeDp(sSelectedRecipeDp, exceptionInfo);
	if (sDpName != sSelectedRecipeDp) {
		return;
	}
	unProgressBar_setPosition("SelectionProgressBar", 0);     
  } 
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Load the PCO List in the PcoList combo box.
 */
void _unRecipeFunctions_loadPcoList() {
	int i;
	dyn_string pcoNames;
	
	// Check that all the necessary shapes exist
	if (!shapeExists("PcoList")) {
		fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
		return;
	}
	
	PcoList.deleteAllItems();
	unRecipeFunctions_getPcoList(pcoNames);	
	PcoList.appendItem("*");
	for (i=1; i<=dynlen(pcoNames); i++) {
		PcoList.appendItem(pcoNames[i]);
	}
} 
 
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Method used to load the recipe class list associated to a PCO
 * @param sPcoLink Alias of the PCO which recipe classes will be loaded or "*" for all the available classes.
 */ 
void _unRecipeFunctions_loadRecipeClassList(string sPcoLink="*")
{
	dyn_dyn_anytype resultTable;
	dyn_string recipeClassNames, recipeClassDps, recipeClassComments, exceptionInfo;
	dynClear(recipeClassNames);
	dynClear(recipeClassComments);

	// Check that all the necessary shapes exist
	if (!shapeExists("RecipeClasses")) {
		fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
		return;
	}	
	
	unRecipeFunctions_getRecipeClasses(recipeClassNames, recipeClassDps, sPcoLink);
	if (dynlen(recipeClassNames)==0) {
		setValue("RecipeClasses", "appendItem", "-None-");
	} else {
		for (int i=1; i<=dynlen(recipeClassNames); i++) {
			setValue("RecipeClasses", "appendItem", recipeClassNames[i]);
		}
	}
} 

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the list of PCO aliases.
 * @param dsPcoNames - [OUT] List of PCO aliases.
 */
void unRecipeFunctions_getPcoList(dyn_string & dsPcoNames) 
{
  dyn_string dsSelectedDeviceTypes, dsRetDpNames;
  dynClear(dsPcoNames);
	
  dsSelectedDeviceTypes = makeDynString(UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME);
	
  _unRecipeFunctions_getFilteredDevices(dsSelectedDeviceTypes, dsRetDpNames, dsPcoNames);
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the list of devices filtering the applications and the device types specified.
 * @param dsSelectedDevType - [IN] List of selected device types.
 * @param dsRetDpName 	    - [OUT] List of DP Names found.
 * @param dsRetAlias 		- [OUT] List of devices aliases found.
 */
void _unRecipeFunctions_getFilteredDevices(dyn_string dsSelectedDevType, dyn_string &dsRetDpName, dyn_string &dsRetAlias) 
{
  bit32 b32SelectState;
  dyn_string dsSelectedSystem, dsSelectedFE, dsSelectedAppl, dsSelectedSubsystem1, dsSelectedSubsystem2, dsAliasSelected ;
  dyn_string dsExcludedAliasSelected, dsDpName, dsAlias, dsFE, dsFEDpName, dsFEDPType, dsAppl, dsDevType, dsSubsystem1, dsSubsystem2;
  dyn_string dsRetFE, dsRetFEDpName, dsRetFEDPType, dsRetAppl, dsRetDevType, dsRetSubsystem1, dsRetSubsystem2;
  dyn_dyn_string dsSystemData;
  
  // Set the bits of the SelectState variable
  setBit(b32SelectState, FILTER_APPLICATION, TRUE);
  setBit(b32SelectState, FILTER_DEVICETYPE, TRUE);

  dsSelectedSystem = makeDynString("*");
  dsSelectedFE = makeDynString("*");
  dsSelectedSubsystem1 = makeDynString("*");
  dsSelectedSubsystem2 = makeDynString("*");
  dsAliasSelected = makeDynString("*");
  dsExcludedAliasSelected = makeDynString();
  
  // Get the system data
  dsSystemData = g_m_ddsTreeDeviceOverviewDeviceList[getSystemName()];
  dsDpName = dsSystemData[TREE_DEVICE_DPNAME];
  dsAlias = dsSystemData[TREE_DEVICE_ALIAS];
  dsFE = dsSystemData[TREE_DEVICE_FE];
  dsFEDpName = dsSystemData[TREE_DEVICE_FEDPNAME];
  dsFEDPType = dsSystemData[TREE_DEVICE_FEDPTYPE];
  dsAppl = dsSystemData[TREE_DEVICE_FEAPPLICATION];
  dsDevType = dsSystemData[TREE_DEVICE_TYPE];
  dsSubsystem1 = dsSystemData[TREE_DEVICE_SUBSYSTEM1];
  dsSubsystem2 = dsSystemData[TREE_DEVICE_SUBSYSTEM2];
  
  // Get the visible applications
  _unRecipeFunctions_getVisibleApplicationsList(dsSelectedAppl);
  
  CtrlTreeDeviceOverview_getSelectedDeviceList(
    b32SelectState, getSystemName(), dsSelectedSystem, dsSelectedFE, dsSelectedAppl,
	dsSelectedDevType, dsSelectedSubsystem1, dsSelectedSubsystem2, dsAliasSelected, dsExcludedAliasSelected,
	dsDpName, dsAlias, dsFE, dsFEDpName, dsFEDPType, dsAppl, dsDevType, dsSubsystem1, dsSubsystem2,
	dsRetDpName, dsRetAlias, dsRetFE, dsRetFEDpName, dsRetFEDPType, dsRetAppl, dsRetDevType, dsRetSubsystem1, dsRetSubsystem2);
	
  // Remove the DPNames and Alias that don't exist (the list may not be synchronized)
  int iLen = dynlen(dsRetDpName);
  for (int i=1; i<=iLen; i++) {
	if (!dpExists(dsRetDpName[i])) {
	  dynRemove(dsRetDpName, i);
	  dynRemove(dsRetAlias, i);
	  iLen--;
	  i--;
	}
  } 
}
//------------------------------------------------------------------------------------------------------------------------
/**	 
 * Get the list of recipe class names and recipe class DPs linked to a PCO.
 * @param dsRecipeClassNames - [OUT] List of recipe class names.
 * @param dsRecipeClassDps   - [OUT] List of recipe class DPs.
 * @param sPcoLink		   - [IN] PCO alias which recipe classes are requested.
 */
void unRecipeFunctions_getRecipeClasses(dyn_string &dsRecipeClassNames, dyn_string &dsRecipeClassDps, string sPcoLink="*")
{
	int iLen;
	string pvssDpe, sRcpClassPcoLink;
	dyn_string dsSelectedDevType, exceptionInfo;
	dyn_dyn_anytype resultTable;
	
	dynClear(dsRecipeClassNames);
	dynClear(dsRecipeClassDps);
	
	// Get the list of recipe classes
	dsSelectedDevType = makeDynString(UN_CONFIG_UNRCPCLASS_DPT_NAME);	
	_unRecipeFunctions_getFilteredDevices(dsSelectedDevType, dsRecipeClassDps, dsRecipeClassNames);
				
	if (sPcoLink!="*") {
	  // Removes all the recipe classes which are not linked with the selected PCO
	  iLen = dynlen(dsRecipeClassDps);
	  for (int i=1; i<=dynlen(dsRecipeClassDps); i++) {
	    if (!dpExists(dsRecipeClassDps[i] + ".statusInformation.deviceLink:_online.._value"))
		  continue;
	    dpGet(dsRecipeClassDps[i] + ".statusInformation.deviceLink:_online.._value", sRcpClassPcoLink);
		if (sRcpClassPcoLink!=sPcoLink) {
		  dynRemove(dsRecipeClassDps, i);
		  dynRemove(dsRecipeClassNames, i);
		  iLen--;
		  i--;
		}
	  }
	}
	
	// Sort alphabetically the recipe class names and the recipe class DP Names in a consistent way
	_unRecipeFunctions_sortAscDynLists(dsRecipeClassNames, dsRecipeClassDps, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Get the list of visible applications.
 * @param dsApplications - [OUT] List of visible applications.
 */
void _unRecipeFunctions_getVisibleApplicationsList(dyn_string &dsApplications) synchronized(g_unicosHMI_dsApplicationList) { 
  dynClear(dsApplications);
  
  // Check if the application filter is being used
  if (false == g_unicosHMI_bApplicationFilterInUse) {
	dynAppend(dsApplications, "*");
	return;
  }
  
  fwGeneral_stringToDynString(g_unicosHMI_dsApplicationList, dsApplications);
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Load the recipe instances in the table (recipe instances panel).
 * @param sRecipeClass - [IN] Recipe class name which instances will be loaded.
 */
synchronized void _unRecipeFunctions_loadRecipeInstances(string sRecipeClass="")
{
  dyn_string dsRcpNames, dsRcpDps, exceptionInfo;
  dyn_dyn_anytype resultTable;
    	
  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeInstances") || !shapeExists("RecipeClasses")) 
  {
	  fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	  return;
  }  	
	
  if (sRecipeClass=="") {
    // Get the name of the recipe class selected in the combo box
	getValue("RecipeClasses", "text", sRecipeClass); 
	if (sRecipeClass=="") return;
  }
  
  setValue("RecipeInstances", "deleteAllLines");   
  _unRecipeFunctions_getRecipeInstancesOfClass(sRecipeClass, dsRcpNames, dsRcpDps); 
  _unRecipeFunctions_sortAscDynLists(dsRcpNames, dsRcpDps, exceptionInfo);
  setValue("RecipeInstances", "appendLines", dynlen(dsRcpNames),
           "RecipeInstances", dsRcpNames, 
           "RecipeInstancesDP", dsRcpDps);
} 
 
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get all the recipe instances of a class.
 * @param sClassName 		    - [IN] Recipe class name.
 * @param dsRcpInstanceNames    - [OUT] Recipe instance names.
 * @param dsRcpInstanceDps      - [OUT] Recipe instance datapoint elements.
 */
void _unRecipeFunctions_getRecipeInstancesOfClass(string sClassName, dyn_string & dsRcpInstanceNames, 
			dyn_string &dsRcpInstanceDps) 
{
  int iLen;
  string sInstClassName, sInstName;
  dyn_dyn_anytype resultTable;
  dyn_string dsSelectedDevType;
  
  dsSelectedDevType = makeDynString(UN_CONFIG_UNRCPINSTANCE_DPT_NAME);
  _unRecipeFunctions_getFilteredDevices(dsSelectedDevType, dsRcpInstanceDps, dsRcpInstanceNames);
  
  if (sClassName=="*")
    return;
	
  // Check if the recipe instances belong to the specified recipe class
  iLen = dynlen(dsRcpInstanceDps);
  for (int i=1; i<=iLen; i++) {
    dpGet(dsRcpInstanceDps[i] + ".ProcessInput.ClassName", sInstClassName,
		  dsRcpInstanceDps[i] + ".ProcessInput.InstanceName", sInstName);
	
	if (sInstClassName!=sClassName) {
	  dynRemove(dsRcpInstanceDps, i);
	  dynRemove(dsRcpInstanceNames, i);
	  iLen--;
	  i--;
	  continue;
	} 
	
	dsRcpInstanceNames[i] = sInstName;
  }
}
//------------------------------------------------------------------------------------------------------------------------ 
/** 
 * Auxiliary function to order alphabetically a list (list1) and reorder the second list (list2) in a consistent way.
 * @param list1 - [IN/OUT] List to be ordered alphabetically.
 * @param list2 - [IN/OUT] List to be ordered in the same way as the list1.
 * @param exceptionInfo - [OUT] Standard error handling variable.
 */
void _unRecipeFunctions_sortAscDynLists(dyn_string &list1, dyn_anytype &list2, dyn_string &exceptionInfo) {
  int iLen, iPos;
  dyn_string cpyList1;
  dyn_anytype cpyList2;
  
  if (dynlen(list1) != dynlen(list2)) {
    fwException_raise(exceptionInfo, "ERROR", "The number of elements in the list must be equal","");
	return;
  }  
  
  cpyList1 = list1;    
  dynSortAsc(cpyList1);
  
  iLen = dynlen(cpyList1);
  for (int i=1; i<=iLen; i++) {
    // Check the position of the curren element in the original list1
	iPos = dynContains(list1, cpyList1[i]);
	
	// Place the element of the second list in the proper order
	cpyList2[i] = list2[iPos];
  }
  
  list1 = cpyList1;
  list2 = cpyList2;
}
//------------------------------------------------------------------------------------------------------------------------ 
/**
 * Load the data of the recipe instance properties table.
 * @param sDpName - [IN] Datapoint element of the recipe instance.
 * @param recipeObject - [IN] Recipe instance data from fwConfigDB.
 */
synchronized void _unRecipeFunctions_loadPropertiesTableInfo(string sDpName="", dyn_dyn_mixed recipeObject=makeDynMixed())
{
  int status;
  bool bActive;
  dyn_int lines;
  dyn_string message, exceptionInfo;
  dyn_mixed recipeClassObject;

  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeProperties")) 
  {
	  fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	  return;
  }     
  
  // Get the selected recipe DP name
  if (sDpName=="") {
	unRecipeFunctions_getSelectedRecipeDp(sDpName, exceptionInfo);
	if (dynlen(exceptionInfo) || sDpName=="") { DebugTN(exceptionInfo); return; }
  }
  
  // Loads the recipe object data
  if (dynlen(recipeObject)==0) {
	_unRecipeFunctions_getRecipeObjectFromDp (sDpName, recipeObject, exceptionInfo); 
	if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return; }
  }

  // Loads the recipe class info
  fwConfigurationDB_getRecipeClassInfo(recipeObject[fwConfigurationDB_RO_META_CLASSNAME], recipeClassObject, exceptionInfo);
  if (dynlen(exceptionInfo))  { DebugTN(exceptionInfo); return; }
  
  // Fill the data in the recipe properties table
  setValue("RecipeProperties", "deleteAllLines");
  dyn_string names = makeDynString("Class:", "Last Activated:", "Status:", "Class Desc.:", "Rcp. Desc.:");
  dyn_string values= makeDynString(recipeClassObject[fwConfigurationDB_RCL_CLASSNAME], 
                                   recipeClassObject[fwConfigurationDB_RCL_LAST_ACTIVATED_RECIPE], 
                                   "", 
                                   recipeClassObject[fwConfigurationDB_RCL_DESCRIPTION], 
								   recipeObject[fwConfigurationDB_RO_META_COMMENT]);
  setValue("RecipeProperties", "appendLines", 5,
           "Name", names,
           "Value", values);
  
  // Get the recipe instance status
  unRecipeFunctions_getRecipeStatus(recipeObject, 
                                    recipeClassObject[fwConfigurationDB_RCL_LAST_ACTIVATED_RECIPE],
                                    status, 
                                    message, 
                                    exceptionInfo);
									
  unRecipeFunctions_setRecipeStatusInfo(status, message);
} 
 
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Set the recipe status in the RecipeProperties table.
 * @param status  - [IN] New status of the recipe instance.
 * @param message - [IN] Contains a message to display when there is no PLC connection.
 */
synchronized void unRecipeFunctions_setRecipeStatusInfo(int status, dyn_string message=makeDynString())
{
  int len;
  string sCurrentStatus;
  dyn_string exceptionInfo;
  
  // Check that all the necessary shapes exist
  if (!shapeExists("StatusColor") || !shapeExists("buttonActivate") || !shapeExists("RecipeProperties")) 
  {
	  fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	  return;
  }     

  StatusColor.visible(true);

  switch(status){
    case RCP_NO_PLC_CONNECTION:
      StatusColor.backCol("{255,0,0}");
      setValue("RecipeProperties", "cellValueRC", 2, "Value", "     [No connection to PLCs: "+message+"]");
      break;
    case RCP_BAD_STRUCTURE:
      StatusColor.backCol("{255,0,0}");
      setValue("RecipeProperties", "cellValueRC", 2, "Value", "     [Wrong recipe structure]");
      break;     
    case RCP_BAD_TYPES:
      StatusColor.backCol("{255,0,0}");
      setValue("RecipeProperties", "cellValueRC", 2, "Value", "     [Wrong value types]");
	  _unRecipeFunctions_setInvalidRows(message);
      break;       
    case RCP_INACTIVE:
      StatusColor.backCol("{251,255,72}");
      setValue("RecipeProperties", "cellValueRC", 2, "Value", "     [Inactive]");
      break;
    case RCP_ACTIVATED:
      StatusColor.backCol("{0,245,0}");
      setValue("RecipeProperties", "cellValueRC", 2, "Value", "     [Active]");
      break;
    case RCP_ACTIVATION_FAILED:
      StatusColor.backCol("{255,0,0}");
      setValue("RecipeProperties", "cellValueRC", 2, "Value", "     [Activation failed]");
      break;	  
	case RCP_ACTIVATION_TRIGGERED:
      StatusColor.backCol("{251,255,72}");
      setValue("RecipeProperties", "cellValueRC", 2, "Value", "     [Activating]");  	
	  break;
  }
}
 
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Load the recipe instance data in the panel.
 */
synchronized void unRecipeFunctions_loadRecipeInstanceData()
{
  string sRecipeInstance, sRecipeClass, sDpName;
  dyn_int diIndex;
  dyn_string exceptionInfo, dsDeviceDps, dsDeviceAlias, dsDescriptions, dsUnits, dsRanges;  
  dyn_dyn_mixed recipeObject;
  dyn_anytype formattedValues;
  mapping typeMapping;	 
				 
  // Get the selected recipe DP name
  unRecipeFunctions_getSelectedRecipeDp(sDpName, exceptionInfo);
  if (dynlen(exceptionInfo) || sDpName=="") { 
    if (dynlen(exceptionInfo))
      _unRecipeFunctions_writeInRecipeLog("Error loading the recipe: " + exceptionInfo[2]);
	else 
	  _unRecipeFunctions_writeInRecipeLog("Error loading the recipe: The DP Name is empty");
	DebugTN(exceptionInfo); 
	return; 
  }
  
  _unRecipeFunctions_getRecipeObjectFromDp(sDpName, recipeObject, exceptionInfo);
  if (dynlen(exceptionInfo)) { 
	_unRecipeFunctions_writeInRecipeLog("Error loading the recipe: " +exceptionInfo[2]);
	DebugTN(exceptionInfo); 
	return; 
  }
	  
  // Load the RecipeProperties table
  _unRecipeFunctions_loadPropertiesTableInfo(sDpName, recipeObject);
  
  // Load the History table
  _unRecipeFunctions_loadHistoryTableInfo(sDpName, recipeObject);

  // Inserts the recipe elements in the table
  int len = dynlen(recipeObject[fwConfigurationDB_RO_DPE_NAME]);
  setMultiValue("RecipeElements", "enabled", FALSE,
				"RecipeElements", "deleteAllLines");
  
  for (int i=1; i<=len; i++) {  
	dyn_string tmp;
    string dpName, alias, deviceType, description, dpe, value, unit="", range="";    
   
    dpName = recipeObject[fwConfigurationDB_RO_DPE_NAME][i];
	alias = unGenericDpFunctions_getAlias(unGenericDpFunctions_getDpName(dpName));
    deviceType = dpTypeName(dpName);
	tmp = strsplit(dpName, ".");
	dpe = tmp[dynlen(tmp)];
    description = unGenericDpFunctions_getDescription(unGenericDpFunctions_getDpName(dpName)) + " : " + dpe;

    range = unRecipeFunctions_getDeviceRange(recipeObject[fwConfigurationDB_RO_DPE_NAME][i], exceptionInfo);
    unit = unRecipeFunctions_getDpeUnit(dpName, exceptionInfo);
    
	dynAppend(diIndex, i);
	dynAppend(dsDeviceDps, dpName);
	dynAppend(dsDeviceAlias, alias);
	dynAppend(dsDescriptions, description);
	dynAppend(dsUnits, unit);
	dynAppend(dsRanges, range);
  }
  
  if (len>0) {
	  setValue("RecipeElements", "appendLines", len,
				 "Index", diIndex,
				 "DeviceDp", dsDeviceDps,
				 "Alias", dsDeviceAlias,
				 "Description", dsDescriptions,
				 "Value",  recipeObject[fwConfigurationDB_RO_VALUE],
				 "Unit", dsUnits,
				 "Range", dsRanges);
  }
  
  string sInitial = recipeObject[fwConfigurationDB_RO_META_PREDEFINED];
  setValue("RecipeInitialLabel", "text", sInitial);
  _unRecipeFunctions_resetRecipeElementsBgColor();
  
  setMultiValue("RecipeElements", "columnHeader", 0, "Index",
				"RecipeElements", "enabled", TRUE,
				"RecipeElements", "adjustColumn", 2);
				
  _unRecipeFunctions_enableRecipeInstanceWarning(false);
}

//------------------------------------------------------------------------------------------------------------------------ 
 /** 
  * Load the recipe instance data in the history table.
  * @param sDpName - [IN] Datapoint element of the recipe instance.
  * @param recipeObject - [IN] Contents of the recipe instance from fwConfigDB.
  */
synchronized void _unRecipeFunctions_loadHistoryTableInfo(string sDpName="", dyn_dyn_mixed recipeObject=makeDynMixed())
{
  string recipeClass, recipeInstance;
  dyn_int lines;
  dyn_string exceptionInfo;

  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeInstances") || !shapeExists("History") || !shapeExists("RecipeClasses")) {
	  fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	  return;
  }     

  // Get the selected recipe DP name
  if (sDpName=="") {
	unRecipeFunctions_getSelectedRecipeDp(sDpName, exceptionInfo);
	if (dynlen(exceptionInfo) || sDpName=="") { DebugTN(exceptionInfo); return; }
  }
  
  // Loads the recipe object data
  if (dynlen(recipeObject)==0) {
	_unRecipeFunctions_getRecipeObjectFromDp (sDpName, recipeObject, exceptionInfo); 
	if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return; }
  }
  
  
  setValue("History", "deleteAllLines");
  dyn_string col1 = makeDynString("Creator:", "Last modifier:", "Last activator:");
  dyn_string col2 = makeDynString(recipeObject[fwConfigurationDB_RO_META_AUTHOR], 
                                  recipeObject[fwConfigurationDB_RO_META_LASTMODIFICATIONUSER], 
                                  recipeObject[fwConfigurationDB_RO_META_LASTACTIVATIONUSER]);
  dyn_string col3 = makeDynString("Creation time:", "Last modification time:", "Last activation time:");
  dyn_string col4 = makeDynString(recipeObject[fwConfigurationDB_RO_META_CREATIONTIME], 
                                  recipeObject[fwConfigurationDB_RO_META_LASTMODIFICATIONTIME], 
                                  recipeObject[fwConfigurationDB_RO_META_LASTACTIVATIONTIME]);  
  setValue("History", "appendLines", 3,
           "#1", col1,
           "#2", col2,
           "#3", col3,
           "#4", col4);
}  

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Remove the recipe instance data from the panel.
 */
synchronized void _unRecipeFunctions_clearInstanceTables() {
  setMultiValue("RecipeProperties", "deleteAllLines",
				"History", "deleteAllLines",
				"LoadedRecipeDp", "text", "",
				"StatusColor", "visible", false,
				"RecipeClassLabel", "text", "",
				"RecipeInstanceLabel", "text", "",
				"RecipeInitialLabel", "text", ""
				);
  
  if (shapeExists("RecipeElements"))
	setValue("RecipeElements", "deleteAllLines");				
}

//--------------------------------------------------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//		Methods to edit a recipe instance
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Put the recipe instance in Edit Mode so the recipe values can be modified.
 * @param sRecipeDp 	- [IN] DataPoint name of the recipe instance.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 * @param row			- [IN] Optional variable. It indicates the row number that is being edited.
 * @param column		- [IN] Optional variable. It indicates the name of the column that is being edited.
 */
void unRecipeFunctions_editRecipeInstance(string sRecipeDp, dyn_string &exceptionInfo, int row=-1, string column="")
{  
  string sSelectedDpName, sRecipeState;
  
  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeElements") || !shapeExists("RecipeClasses") || !shapeExists("RecipeInstances")) {
	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	return;
  }
  
  unRecipeFunctions_getSelectedRecipeDp(sSelectedDpName, exceptionInfo);
  if (sSelectedDpName!=sRecipeDp) {
	  // If the instance selection fails, returns
	  if (!unRecipeFunctions_selectRecipeInstance(sRecipeDp, exceptionInfo)) 
		return;
  }
	
  // Recipe instance in EDIT mode
  dpSet(sRecipeDp+".ProcessInput.State", UN_RCP_INSTANCE_STATE_EDIT);
	  
  // Modifies the color of the recipe values
  _unRecipeFunctions_setTableColumnBGColor("RecipeElements", "Value", CELL_EDIT_COLOR);
  setMultiValue("PcoList", "enabled", false,
				"RecipeClasses", "enabled", false,
				"RecipeInstances", "enabled", false,
				"RecipeElements", "namedColumnEditable", "Value", true);
  
  g_editingRowNumber = row;
  g_editingColName = column;
  _unRecipeFunctions_copyRecipeInstanceToHiddenTable();
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Copy the RecipeElements table (Recipe instance) to a hidden table.
 */
void _unRecipeFunctions_copyRecipeInstanceToHiddenTable() {
  int i, iCols;
  dyn_int diIndex;
  dyn_dyn_string dsData;
  
  getValue("RecipeElements", "columnCount", iCols);
  
  for (i=0; i<iCols; i++) {
    if (i==0) {
	  getValue("RecipeElements", "getColumnN", i, diIndex);
	} else {
      getValue("RecipeElements", "getColumnN", i, dsData[i]);
	}
  }
  
  setValue("RecipeElementsCopy", "deleteAllLines");
  setValue("RecipeElementsCopy", "appendLines", dynlen(dsData[1]), 
		   "Index", diIndex,
		   "DeviceDp", dsData[1],
		   "Alias", dsData[2],
		   "Description", dsData[3],
		   "Value", dsData[4],
		   "Unit", dsData[5],
		   "Range", dsData[6]);

  setValue("RecipeElementsCopy", "sort", true, "Index");  
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Save the values of the recipe instance.
 * @param sRecipeDp - [IN] The DP of the recipe instance.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 * @param bShowConfirmationMessage - [IN] Flag to specify if the confirmation message must be displayed (TRUE by default)
 */ 
void unRecipeFunctions_saveRecipeInstance(string sRecipeDp, dyn_string &exceptionInfo, bool bShowConfirmationMessage=TRUE)
{
  int colCount, iValColIndex = -1, iDpColIndex = -1, status;
  bool bAreEqual;
  string colName;
  dyn_float df;
  dyn_string ds, message, dsDpNames, dsIndex;
  dyn_int errorLines;
  dyn_anytype values;
  dyn_dyn_mixed recipeObject;
  
  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeElements") || !shapeExists("RecipeElementsCopy") || !shapeExists("RecipeClasses") || !shapeExists("RecipeInstances")) {
	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	DebugTN(exceptionInfo);
	return;
  } 
  
  // Copy the table data to a hidden table (to order by index)
  _unRecipeFunctions_copyRecipeInstanceToHiddenTable();
  
  // Get the column index for the values
  getMultiValue("RecipeElementsCopy", "nameToColumn", "Value", iValColIndex,
				"RecipeElementsCopy", "nameToColumn", "DeviceDp", iDpColIndex);  
  if (iValColIndex<0) {
	fwException_raise(exceptionInfo, "ERROR", "The 'RecipeElements' table doesn't have a column named 'Value'.","");
	return;	
  }
  getMultiValue("RecipeElementsCopy", "getColumnN", iValColIndex, values,
				"RecipeElementsCopy", "getColumnN", iDpColIndex, dsDpNames);
  
  _unRecipeFunctions_getRecipeObjectFromDp(sRecipeDp, recipeObject, exceptionInfo);
  if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return;}
  
  // Check if the DPEs of the recipe are coherent (comparing the ones on the table and the ones saved)
  bAreEqual = _unRecipeFunctions_areListsEqual(recipeObject[fwConfigurationDB_RO_DPE_NAME], dsDpNames);
  if (!bAreEqual) {
    _unRecipeFunctions_writeInRecipeLog("Error: The DP names in the table don't match the recipe DP names. Please reload the recipe.");
    return;
  }
  
  // Check the recipe values
  unRecipeFunctions_checkRecipeValues(recipeObject, values, errorLines, exceptionInfo);
  if (dynlen(exceptionInfo)) {
    _unRecipeFunctions_writeInRecipeLog("Error: " + exceptionInfo[2]);
	return;
  } 
  
  // check if the range of any value is wrong
  if (dynlen(errorLines)) {
    int len=dynlen(errorLines);
    _unRecipeFunctions_writeInRecipeLog("The value of some recipe elements is out of range");
	
	dyn_string dsElementsIndex;
	getValue("RecipeElements", "getColumnN", 0, dsElementsIndex);
    for (int i=1; i<=len; i++) {
      int index = (int)errorLines[i];
	  int iPos = dynContains(dsElementsIndex, index);
	  if (iPos>0) {
		setValue("RecipeElements", "cellBackColRC", (iPos-1), "Value", CELL_VALUE_OUT_OF_RANGE_COLOR);
	  }
    }
    return;
  }
  
  // Asks for user confirmation
  if (bShowConfirmationMessage) {
	ChildPanelOnCentralReturn("fwGeneral/fwOkCancel.pnl",
	 						  "Save recipe values",
							  makeDynString("$sTitle:Save recipe values ",
											  "$text: Do you want to save the recipe values?"  ),
							  df,
							  ds);
	  
	  if (df[1]<=0) return;
  }
  
  _unRecipeFunctions_saveRecipeValues(recipeObject, values, exceptionInfo); 
  if (dynlen(exceptionInfo)) {
    _unRecipeFunctions_writeInRecipeLog("Error: " + exceptionInfo[2]);
    return;
  }
  recipeObject[fwConfigurationDB_RO_VALUE] = values;

  getMultiValue("RecipeElements", "getColumnN", 1, dsDpNames,
				"RecipeElements", "getColumnN", 0, dsIndex);
  // sets the formatted values in the table
  for (int i=1; i<=dynlen(dsIndex); i++) {
    int index = dsIndex[i];
	setValue("RecipeElements", "cellValueRC", (i-1), "Value", values[index]);
  }
  _unRecipeFunctions_resetRecipeElementsBgColor();
  _unRecipeFunctions_loadHistoryTableInfo(sRecipeDp, recipeObject);
  _unRecipeFunctions_loadPropertiesTableInfo(sRecipeDp, recipeObject);
  
  setMultiValue("PcoList", "enabled", true,
				"RecipeClasses", "enabled", true,
				"RecipeInstances", "enabled", true,
				"RecipeElements", "namedColumnEditable", "Value", false);
  
  dpSet(sRecipeDp+".ProcessInput.State", "");
  g_editingRowNumber = -1;
  g_editingColName = "";
  _unRecipeFunctions_writeInRecipeLog("The recipe " + recipeObject[fwConfigurationDB_RO_META_ORIGNAME] + " has been saved.", TRUE);
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Save the values of the selected recipe in the initial recipe of the class.
 * @param sRecipeDp - [IN] The DP of the recipe instance which values must be save in the initial recipe.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */ 
void unRecipeFunctions_saveAsInitialRecipe(string sRecipeDp, dyn_string &exceptionInfo)
{
  int iCol;
  string sRcpClassName, sRcpInstanceName, sInitialRcpInstanceDp;
  dyn_dyn_mixed recipeObject, initialRecipeObject;
  dyn_string recipeValues;
  dyn_int errorLines;

  // Get the recipe class of the selected recipe instance
  _unRecipeFunctions_getRecipeObjectFromDp(sRecipeDp, recipeObject, exceptionInfo);
  if (dynlen(exceptionInfo)) {
	DebugTN("Error getting the data of the recipe instance with DP: " + sRecipeDp);
	DebugTN(exceptionInfo); 
	return;
  }
  sRcpClassName = recipeObject[fwConfigurationDB_RO_META_CLASSNAME];
  
  // Get the initial recipe of the recipe class
  _unRecipeFunctions_getInitialRecipeOfClass(sRcpClassName, sInitialRcpInstanceDp, exceptionInfo);
  if (dynlen(exceptionInfo)) {
    _unRecipeFunctions_writeInRecipeLog("Error: " + exceptionInfo[2]);
    DebugTN(exceptionInfo);
    return;
  }
  
  // Get the values of the selected recipe
  _unRecipeFunctions_copyRecipeInstanceToHiddenTable();
  getMultiValue("RecipeElementsCopy", "nameToColumn", "Value", iCol,
				"RecipeElementsCopy", "getColumnN", iCol, recipeValues);
  
  // Get the initial recipe object
  _unRecipeFunctions_getRecipeObjectFromDp(sInitialRcpInstanceDp, initialRecipeObject, exceptionInfo);
  if (dynlen(exceptionInfo)) {
    _unRecipeFunctions_writeInRecipeLog("Error getting the data of the recipe instance with DP: " + sRecipeDp);
	DebugTN(exceptionInfo); 
	return;
  }
  
  // Check if the recipe values are in the valid range
  unRecipeFunctions_checkRecipeValues (initialRecipeObject, recipeValues, errorLines, exceptionInfo);
  if (dynlen(errorLines)) {
	_unRecipeFunctions_writeInRecipeLog("Error: The recipe values can't be saved. Some recipe elements are out of range.");
	return;
  } 
  
  if (dynlen(exceptionInfo)) {
    _unRecipeFunctions_writeInRecipeLog("Error: " + exceptionInfo[2]);
	return;
  }
  
  // Save the new values of the initial recipe
  dpGet(sInitialRcpInstanceDp + ".ProcessInput.InstanceName", sRcpInstanceName);
  initialRecipeObject[fwConfigurationDB_RO_VALUE] = recipeValues;  
  fwConfigurationDB_saveRecipeToCache(initialRecipeObject, "", sRcpClassName + "/" + sRcpInstanceName, exceptionInfo);
  
  if (dynlen(exceptionInfo)) {
    _unRecipeFunctions_writeInRecipeLog("Error: " + exceptionInfo[2]);
	//DebugTN("Error saving the data of the recipe instance with DP: " + sInitialRcpInstanceDp);
	//DebugTN(exceptionInfo); 
	return;
  }
  
  _unRecipeFunctions_writeInRecipeLog("The values of the recipe " + recipeObject[fwConfigurationDB_RO_META_ORIGNAME] + 
				" have been saved in the initial recipe: " + sRcpClassName + "/" + sRcpInstanceName);
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the DP name of the initial recipe of the class 'sClassName'.
 * @param sClassName - [IN] Name of the recipe class which initial recipe is required.
 * @param sInitialRcpDp - [OUT] DP Name of the initial recipe of the specified class.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 * @return 0 if the initial recipe was found, otherwise returns a negative value
 */
int _unRecipeFunctions_getInitialRecipeOfClass(string sClassName, string & sInitialRcpDp, dyn_string & exceptionInfo) {
  
  int iRetVal, iPos;
  string sRcpInstanceName;
  dyn_dyn_anytype resultTable;
  
  // Get the initial recipe of the recipe class 'sClassName'
  iRetVal = dpQuery("SELECT '.RecipeName:_online.._value' FROM '*' WHERE '.RecipeName:_online.._value' LIKE \"" + sClassName + "/*\" AND '.MetaInfo.Predefined:_online.._value' = \"TRUE\"  AND _DPT=\"_FwRecipeCache\" ", resultTable);   
 
  // Check if there isn't an initial recipe for the recipe class 
  if (dynlen(resultTable)<=1) {
    fwException_raise(exceptionInfo,"ERROR", "The recipe class '" + sClassName + "' doesn't have an initial recipe. ","");
	return RCP_INITIAL_NO_DEFINED;	   
  }

  // Check if there is more than one initial recipe instance
  if (dynlen(resultTable)>2) {
    fwException_raise(exceptionInfo, "ERROR", "The recipe class '" + sClassName + "' has more than one initial recipe instance. ","");
	return RCP_INITIAL_TOO_MANY;	   
  }
  
  // Get the recipe instance name (Recipe name: rcpClassName/rcpInstanceName)
  iPos = strpos(resultTable[2][2], "/");
  if (iPos<0) {
	fwException_raise(exceptionInfo, "ERROR", "The recipe name is wrong ('/' character is missing).");
	return RCP_INITIAL_UNDEFINED;
  }
  
  // Get the UNICOS recipe DP name
  sRcpInstanceName = substr(resultTable[2][2], iPos+1);
  iRetVal = dpQuery("SELECT '.ProcessInput.ClassName:_online.._value' FROM '*' WHERE '.ProcessInput.ClassName:_online.._value' = \"" + sClassName + "\" AND '.ProcessInput.InstanceName:_online.._value' = \"" + sRcpInstanceName + "\"  AND _DPT=\"" + UN_CONFIG_UNRCPINSTANCE_DPT_NAME + "\" ", resultTable);   
  
  if (dynlen(resultTable)!=2) {
    fwException_raise(exceptionInfo, "ERROR", "The recipe instance '" + sClassName + "/" + sRcpInstanceName + "' doesn't exist. ","");
	return RCP_INITIAL_UNDEFINED;	 
  }
  
  // Get the PVSS DP name of the recipe instance
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(resultTable[2][1], sInitialRcpDp);
  
  return RCP_INITIAL_OK;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get all the PCOs that are children of the sPco device (recursive method, get all the PCOs in the hierarchy).
 * @param sPcoDpName - [IN] Name of the PCO which children are requested.
 * @param children - [OUT] Contains the alias of all the PCO children of the sPcoDp device.
 * @param exceptionInfo - [OUT] Standard exception handling variable. 
 */
void _unRecipeFunctions_getPcoChildren(string sPcoDpName, dyn_string &children, dyn_string & exceptionInfo) {
  string sCurrentPcoAlias;
  dyn_string all_children;

  if (sPcoDpName[strlen(sPcoDpName)-1]==".")
	sPcoDpName = substr(sPcoDpName, 0, strlen(sPcoDpName)-1);
  
  all_children = cpcGenericObject_getLinkedObjects(sPcoDpName, UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME, exceptionInfo);
  
  for (int i = 1; i <= dynlen(all_children); i++) {
    sCurrentPcoAlias = all_children[i];
    dynAppend(children, sCurrentPcoAlias);
	_unRecipeFunctions_getPcoChildren(sCurrentPcoAlias, children, exceptionInfo);    
  }
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Open a new window to display the initial recipes of a PCO.
 * @param sPcoDp - [IN] DataPoint name of the PCO which initial recipes will be displayed.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_activateInitialRecipesFromPcoHierarchy(string sPcoDp, dyn_string & exceptionInfo) {
  dyn_string ds;
  dyn_float df;
  
  ChildPanelOnCentralReturn("vision/unRecipe/UnRcpInstance/unFaceplateUnRcpInstance.pnl",
			"Create Recipe Instance",
			makeDynString("$sAction:" + UN_RCP_ACTION_ACTIVATE_INITIAL_RECIPES, "$sDpName:" + sPcoDp),
			df,ds);
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the DP name of the last activated recipe of the class 'sClassName'.
 * @param sClassName - [IN] Name of the recipe class which last activated recipe is required.
 * @param sLastActivatedRcpDp - [OUT] DP Name of the last activated recipe of the specified class.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 * @return 0 if the last activated recipe was found, otherwise returns a negative value
 */
int _unRecipeFunctions_getLastActivatedRecipeDp(string sClassName, string & sLastActivatedRcpDp, dyn_string & exceptionInfo) {
  
  int iRetVal, iPos;
  string sRcpInstanceName;
  dyn_dyn_anytype resultTable;
  
  // Get the last activated recipe of the recipe class 'sClassName'
  iRetVal = dpQuery("SELECT '.RecipeName:_online.._value', '.MetaInfo.LastActivationTime:_online.._value' FROM '*' WHERE '.RecipeName:_online.._value' LIKE \"" + sClassName + "/*\" AND _DPT=\"_FwRecipeCache\" SORT BY 2 DESC", resultTable);   
 
  // Check if there aren't recipes matching the criteria
  if (dynlen(resultTable)<=1) {
    fwException_raise(exceptionInfo,"ERROR", "The recipe class '" + sClassName + "' doesn't have any recipe instance. ","");
	return -1;	   
  }
  
  // Get the recipe instance name (Recipe name: rcpClassName/rcpInstanceName)
  iPos = strpos(resultTable[2][2], "/");
  if (iPos<0) {
	fwException_raise(exceptionInfo, "ERROR", "The recipe name is wrong ('/' character is missing).");
	return -1;
  }
  
  // Get the UNICOS recipe DP name
  sRcpInstanceName = substr(resultTable[2][2], iPos+1);
  iRetVal = dpQuery("SELECT '.ProcessInput.ClassName:_online.._value' FROM '*' WHERE '.ProcessInput.ClassName:_online.._value' = \"" + sClassName + "\" AND '.ProcessInput.InstanceName:_online.._value' = \"" + sRcpInstanceName + "\"  AND _DPT=\"" + UN_CONFIG_UNRCPINSTANCE_DPT_NAME + "\" ", resultTable);   
  
  if (dynlen(resultTable)!=2) {
    fwException_raise(exceptionInfo, "ERROR", "The recipe instance '" + sClassName + "/" + sRcpInstanceName + "' doesn't exist. ","");
	return -1;	 
  }
  
  // Get the PVSS DP name of the recipe instance
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(resultTable[2][1], sLastActivatedRcpDp);
  
  return 0;
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Open a new window to display the last activated recipes of a PCO.
 * @param sPcoDp - [IN] DataPoint name of the PCO which initial recipes will be displayed.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_activateLastActivatedRecipesFromPcoHierarchy(string sPcoDp, dyn_string & exceptionInfo) {
  dyn_string ds;
  dyn_float df;
  
  ChildPanelOnCentralReturn("vision/unRecipe/UnRcpInstance/unFaceplateUnRcpInstance.pnl",
			"Create Recipe Instance",
			makeDynString("$sAction:" + UN_RCP_ACTION_ACTIVATE_LAST_ACTIVATED_RECIPES, "$sDpName:" + sPcoDp),
			df,ds);
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the last activated recipe of each recipe class for the current PCO.
 * @param sPcoDp - [IN] The PCO datapoint element which last activated recipes are required.
 * @param recipeList - [OUT] Datapoint elements of the last activated recipes of the PCO.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_getLastActivatedPcoRecipes (string sPcoDp, dyn_string &recipeList, dyn_string &exceptionInfo) {
  int iRetVal, iLen, i;
  string sPcoAlias, sClassName, sLastActivatedRcpDp;
  dyn_string dsRcpClasses, dsRcpClassDps;
  dyn_dyn_anytype resultTable;
  
  // Check if the PCO DP exists
  if (!dpExists(sPcoDp)) {
	fwException_raise(exceptionInfo, "ERROR", "The PCO DP '" + sPcoDp + "' doesn't exist", "");
	return;
  }
  
  // Get the PCO UNICOS alias
  sPcoAlias = unGenericDpFunctions_getAlias(sPcoDp);
  
  // Get all the recipe classes linked to the PCO
  unRecipeFunctions_getRecipeClasses(dsRcpClasses, dsRcpClassDps, sPcoAlias);
  
  if (!dynlen(dsRcpClasses)) {
    // The selected PCO doesn't have any recipe class linked, return.
	//DebugTN("There aren't recipe classes linked to the PCO");
	return;
  }
    
  iLen = dynlen(dsRcpClasses);
  for (i=1; i<=iLen; i++) {
	// Get the last activated recipe of the recipe class
	sClassName = dsRcpClasses[i];	
	iRetVal = _unRecipeFunctions_getLastActivatedRecipeDp(sClassName, sLastActivatedRcpDp, exceptionInfo);	
	if (iRetVal < 0)
		continue;
	
	dynAppend(recipeList, sLastActivatedRcpDp);
  }
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the initial recipe of each recipe class for the current PCO.
 * @param sPcoDp - [IN] The PCO datapoint element which initial recipes are required.
 * @param recipeList - [OUT] Datapoint elements of the initial recipes of the PCO.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_getInitialPcoRecipes (string sPcoDp, dyn_string &recipeList, dyn_string &exceptionInfo) {
  int iRetVal, iLen, i;
  string sPcoAlias, sClassName, sInitialRcpDp;
  dyn_string dsRcpClasses, dsRcpClassDps;
  dyn_dyn_anytype resultTable;
  
  // Check if the PCO DP exists
  if (!dpExists(sPcoDp)) {
	fwException_raise(exceptionInfo, "ERROR", "The PCO DP '" + sPcoDp + "' doesn't exist", "");
	return;
  }
  
  // Get the PCO UNICOS alias
  sPcoAlias = unGenericDpFunctions_getAlias(sPcoDp);
  
  // Get all the recipe classes linked to the PCO
  unRecipeFunctions_getRecipeClasses(dsRcpClasses, dsRcpClassDps, sPcoAlias);
  
  if (!dynlen(dsRcpClasses)) {
    // The selected PCO doesn't have any recipe class linked, return.
	//DebugTN("There aren't recipe classes linked to the PCO");
	return;
  }  
  
  iLen = dynlen(dsRcpClasses);
  for (i=1; i<=iLen; i++) {
	// Get the initial recipe of the recipe class
	sClassName = dsRcpClasses[i];	
	iRetVal = _unRecipeFunctions_getInitialRecipeOfClass(sClassName, sInitialRcpDp, exceptionInfo);
	if (iRetVal < 0)
		continue;
	
	dynAppend(recipeList, sInitialRcpDp);
  }
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the last activated recipe of each recipe class for the PCO hierarchy.
 * @param sPcoDp - [IN] The PCO datapoint element which last activated recipes are required.
 * @param recipeList - [OUT] Datapoint elements of the last activated recipes of the PCO hierarchy.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_getLastActivatedRecipesFromPcoHierarchy(string sPcoDp, dyn_string &recipeList, dyn_string &exceptionInfo) {
  int i, iLen;
  dyn_string allPcos;

  dynAppend(allPcos, sPcoDp);
  
  // Get all the PCO that are children of the sPcoDp device
  _unRecipeFunctions_getPcoChildren(sPcoDp, allPcos, exceptionInfo);
  
  iLen = dynlen(allPcos);
  for (i=1; i<=iLen; i++) {
    string sCurrentPco = allPcos[i];
    _unRecipeFunctions_getLastActivatedPcoRecipes (sCurrentPco, recipeList, exceptionInfo);
  } 
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the initial recipe of each recipe class for the PCO hierarchy.
 * @param sPcoDp - [IN] The PCO datapoint element which initial recipes are required.
 * @param recipeList - [OUT] Datapoint elements of the initial recipes of the PCO hierarchy.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_getInitialRecipesFromPcoHierarchy(string sPcoDp, dyn_string &recipeList, dyn_string &exceptionInfo) {
  int i, iLen;
  dyn_string allPcos;
  
  dynAppend(allPcos, sPcoDp);
  
  // Get all the PCO that are children of the sPcoDp device
  _unRecipeFunctions_getPcoChildren(sPcoDp, allPcos, exceptionInfo);
  iLen = dynlen(allPcos);
  for (i=1; i<=iLen; i++) {
    string sCurrentPco = allPcos[i];
    _unRecipeFunctions_getInitialPcoRecipes (sCurrentPco, recipeList, exceptionInfo);
  } 
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Write a message in the recipe log.
 * @param sMessage 			 - [IN] The message to write in the recipe log.
 * @param bWriteInGeneralLog - [IN] Flag to specify if the message must be written in the UNICOS HMI log (false by default).
 * @param bWriteInLogViewer  - [IN] Flag to specify if the message must be written in the WinCC OA log viewer (false by default).
 */
void _unRecipeFunctions_writeInRecipeLog(string sMessage, bool bWriteInGeneralLog=false, bool bWriteInLogViewer=false) {
  int iElements;
  dyn_string exceptionInfo;
 
  // Check that all the necessary shapes exist
  if (shapeExists("RecipeLog")) {
	string sTime = formatTime("[%Y.%m.%d %H:%M:%S]", getCurrentTime());
	RecipeLog.appendItem(sTime + " " + sMessage);
	getValue("RecipeLog", "itemCount", iElements);
	setValue("RecipeLog", "bottomPos", iElements);
  } 

  if (bWriteInGeneralLog) {
    unSendMessage_toAllUsers(sMessage, exceptionInfo);
  }
  
  if (bWriteInLogViewer) {
	DebugTN(sMessage);
  }
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Cancel the recipe instance edition.
 * @param sRecipeDp - [IN] DP Name of the recipe instance that was being edited.
 */
void unRecipeFunctions_cancelEditRecipeInstance(string sRecipeDp) { 
  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeElements") || !shapeExists("RecipeClasses") || !shapeExists("RecipeInstances") ) {
	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	return;
  } 
  
  dpSet(sRecipeDp+".ProcessInput.State", "");
  
  setMultiValue("PcoList", "enabled", true,
				"RecipeClasses", "enabled", true,
				"RecipeInstances", "enabled", true,
				"RecipeElements", "namedColumnEditable", "Value", false);
  _unRecipeFunctions_resetRecipeElementsBgColor(); 
  unRecipeFunctions_loadRecipeInstanceData();
  
  g_editingRowNumber = -1;
  g_editingColName = "";
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the recipe instance object.
 * @param sRecipeDp     - [IN] Datapoint name of the recipe instance.
 * @param recipeObject  - [OUT] Recipe instance object from fwConfigDB.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_getRecipeObjectFromDp (string sRecipeDp, dyn_dyn_mixed &recipeObject, dyn_string &exceptionInfo)
{
  int pos;
  string recipeClass, recipeInstance;
  
  if ((pos=strpos(sRecipeDp, "."))>=0) {
	sRecipeDp = substr(sRecipeDp, 0, pos);
  }
  
  if (!dpExists(sRecipeDp+".ProcessInput.ClassName") || 
      !dpExists(sRecipeDp+".ProcessInput.InstanceName"))
  {
	fwException_raise(exceptionInfo, "ERROR", "The recipe DP '" + sRecipeDp + "' doesn't exist.","");
	return;
  }  
  
  dpGet(sRecipeDp+".ProcessInput.ClassName", recipeClass,
		sRecipeDp+".ProcessInput.InstanceName", recipeInstance);
		
  fwConfigurationDB_loadRecipeFromCache(recipeClass+"/"+recipeInstance, makeDynString(), "", recipeObject, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Get the recipe class object from the JCOP library.
 * @param sRcpClassName	  - [IN] Recipe class name.
 * @param recipeClassInfo - [OUT] Recipe class object returned from the JCOP library.
 * @param exceptionInfo   - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_getRecipeClassObject(string sRcpClassName, dyn_mixed &recipeClassInfo, dyn_string &exceptionInfo)
{
  fwConfigurationDB_getRecipeClassInfo(sRcpClassName, recipeClassInfo, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Add new devices to the recipe class that is being displayed in the recipe class panel.
 * @param sRcpClassDp - [IN] Datapoint element of the recipe class that will be modified.
 */
void unRecipeFunctions_addRecipeClassDevices(string sRcpClassDp) {
  dyn_string exceptionInfo;
  dyn_string types;
  dyn_dyn_string fwDevices;
  int i,n;
  string sPreviousState;
  dyn_string deviceList, existingDevices;
  dyn_dyn_string rcpTypeData;
  dyn_float df;
  dyn_string ds;
  mapping mRcpTypeMap, mRcpTypeMapString;
  

  _fwConfigurationDB_getRecipeTypeData(UN_CONFIG_UNRCPTYPE_DPT_NAME, rcpTypeData, exceptionInfo);  
  if (dynlen(exceptionInfo)) {
	fwExceptionHandling_display(exceptionInfo);
	DebugTN(exceptionInfo);
	return;
  }
  
  _unRecipeFunctions_getTypeMapping(UN_CONFIG_UNRCPTYPE_DPT_NAME, mRcpTypeMap, exceptionInfo);
  dyn_string mappingKeys = mappingKeys(mRcpTypeMap);
  for (i=1; i<=dynlen(mappingKeys); i++) {
    string sCurrentType = mappingKeys[i];
	dyn_string dsTypeDpes = mRcpTypeMap[sCurrentType];
	dynSortAsc(dsTypeDpes);	
	string sDeviceTypeDpes = "";
	fwGeneral_dynStringToString(dsTypeDpes, sDeviceTypeDpes, ", ");
	mRcpTypeMapString[sCurrentType] = sDeviceTypeDpes;
  }

  dyn_dyn_string allfwDeviceTypeInfo;
  dyn_string allfwDpTypes,allfwDevTypes;
  fwDevice_getAllTypes(allfwDeviceTypeInfo,exceptionInfo);
  if (dynlen(exceptionInfo)) {
	fwExceptionHandling_display(exceptionInfo);
	return;
  }
  allfwDevTypes=allfwDeviceTypeInfo[1];
  allfwDpTypes=allfwDeviceTypeInfo[2];
  
  bool allowDevsWithNoAlias=true;
  if (isDollarDefined("$allowDevsWithNoAlias")) allowDevsWithNoAlias=$allowDevsWithNoAlias;
  
  dyn_string allDevDps,allDevAliases, rcpDeviceTypes;
  //dpGetAllAliases(allDevDps,allDevAliases,"*","*.");
  
  for (int i=1; i<=dynlen(rcpTypeData); i++) {
    string sCurrTypeName = rcpTypeData[i][2];
    dynAppend(rcpDeviceTypes, sCurrTypeName);
  }
  _unRecipeFunctions_getFilteredDevices(rcpDeviceTypes, allDevDps, deviceList);
  
  // Get the current recipe state
  dpGet(sRcpClassDp + ".ProcessInput.State", sPreviousState);
  if (sPreviousState == "") {
	// Set the recipe State to Edit mode
	dpSet(sRcpClassDp + ".ProcessInput.State", UN_RCP_CLASS_STATE_EDIT);
  }
  
  // Get the list of existing devices from the RecipeElements table
  getValue("RCCElements.RecipeElementsTable", "getColumnN", 2, existingDevices);
  
  // Create a String with the allowed device types for the recipe type
  string sRcpTypeList;
  dynSortAsc(mappingKeys);
  fwGeneral_dynStringToString(mappingKeys, sRcpTypeList, ",");
  
  ChildPanelOnCentralModalReturn("vision/unRecipe/UnRcpClass/unUnRcpClass_SelectionList.pnl",
		"Select Devices",
		makeDynString("$sDpName:" + sRcpClassDp,
					  "$text:Select devices:",
					  "$items:"+deviceList,
					  "$types:"+sRcpTypeList,
        			  "$AliasSelector:FALSE",
        			  "$DPSelector:FALSE",
					  "$selectMultiple:"+TRUE,
					  "$disabledItems:"+existingDevices),
		df,ds);
		
  // Insert the selected elements in the table
  for (int i=1; i<=dynlen(ds); i++) {
    string dpName,devName,alias;
    // firstly check if this is h/w or logical device
    if (strpos(ds[i],":")>0){
      alias="";
      dpName=ds[i];
    } else {
      alias=ds[i];
	  int idx=dynContains(deviceList,alias);
      if (idx<1) {
        continue;
      }
      dpName=dpSubStr(allDevDps[idx],DPSUB_SYS_DP);
    }
    if (!dpExists(dpName)) {
      continue;
    }

    string deviceDpType = dpTypeName(dpName);
    string deviceType=deviceDpType;
	dyn_string deviceElements = mRcpTypeMapString[deviceType];
    setValue("RCCElements.RecipeElementsTable", "appendLine",
             "DeviceDpType",deviceDpType,
             "DeviceType", deviceType,
             "DeviceName", ds[i],
			 "DeviceElements", deviceElements);             
  }
  
  if (dynlen(ds)) {
    _unRecipeFunctions_writeInRecipeLog("The new devices have been added. Please apply the changes to the recipe class.");
	setValue("applyChangesFrame", "visible", true);
  } else if (sPreviousState=="") {
	dpSet(sRcpClassDp + ".ProcessInput.State", "");
  }
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Apply the modifications done in the recipe class elements.
 * @param sRcpClassDp - [IN] DataPoint name of the recipe class where the changes must be applied.
 */
void unRecipeFunctions_applyRecipeClassModifications(string sRcpClassDp) {
  int i, j, iCol;
  string sClassName, sTypeName, sDesc;
  dyn_string deviceAliases, deviceDpes, rcpDpes, exceptionInfo;

  // Get the data from the recipe class panel
  getMultiValue("RecipeClass", "text", sClassName,
				"RecipeType", "text", sTypeName,
				"RecipeClassComment", "text", sDesc,
				"RCCElements.RecipeElementsTable", "nameToColumn", "DeviceName", iCol,
				"RCCElements.RecipeElementsTable", "getColumnN", iCol, deviceAliases,
				"RCCElements.RecipeElementsTable", "nameToColumn", "DeviceElements", iCol,
				"RCCElements.RecipeElementsTable", "getColumnN", iCol, deviceDpes);
  
  for (i=1; i<=dynlen(deviceAliases); i++) {
    dyn_string deviceDpesSplit = strsplit(deviceDpes[i], ",");
	for (j=1; j<=dynlen(deviceDpesSplit); j++) {		
		dynAppend(rcpDpes, deviceAliases[i] + ".ProcessOutput." + strltrim(deviceDpesSplit[j]));
	}
  }
  
  // Updates the JCOP Recipe class DP
  fwConfigurationDB_modifyRecipeClass(sClassName, rcpDpes, sTypeName, sDesc, exceptionInfo);
  
  // Remove the Edit state of the recipe class
  dpSet(sRcpClassDp + ".ProcessInput.State", "");
  _unRecipeFunctions_writeInRecipeLog("The changes have been applied to the recipe class " + sClassName + " and all its instances.", TRUE);
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Remove the selected recipe class devices from the table.
 * @param sRcpClassDp - [IN] Datapoint element of the recipe class to be modified.
 */
void unRecipeFunctions_removeSelectedRecipeClassDevices(string sRcpClassDp) {
  int i, iLen;
  dyn_int diLines;

  getValue("RCCElements.RecipeElementsTable", "getSelectedLines", diLines);
  iLen = dynlen(diLines);
  for (i=iLen; i>=1; i--) {
	setValue("RCCElements.RecipeElementsTable", "deleteLineN", diLines[i]);
  }
  
  if (dynlen(diLines)) {
	dpSet(sRcpClassDp + ".ProcessInput.State", UN_RCP_CLASS_STATE_EDIT);
	setValue("applyChangesFrame", "visible", true);
	_unRecipeFunctions_writeInRecipeLog("The selected devices have been removed. Please apply the changes to the recipe class.");
  }
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Create the recipe class device.
 * @param sClassName    - [IN] Recipe class name.
 * @param sDescription  - [IN] Recipe class description.
 * @param sPcoLink      - [IN] Alias of the PCO where the recipe class will be linked.
 * @param sRcpClassDp   - [OUT] Datapoint of the new recipe class.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_createRecipeClass(string sClassName, string sDescription, string sPcoLink, string &sRcpClassDp, dyn_string &exceptionInfo)
{
  int iDeviceNumber, iDriverNumber, iLen, iNumInstances;
  string sPrefix, sDeviceList = "", sPcoLinkDpe, sPvssDpe, sFrontEndName, sFrontEndDp, sAppName, sOperatorPrivileges, sExpertPrivileges, sAdminPrivileges;
  dyn_mixed recipeClass;
  dyn_string dsConfig, dsRunningArchive, dsSplitDp, dsOnlineDpes, dsRcpNames, dsArchBool, dsArchAnalog, dsArchEvent, dsParameters;
  dyn_string dsOperatorPrivileges, dsExpertPrivileges, dsAdminPrivileges;
  dyn_dyn_anytype resultTable;
  bool bEvent;
  
  _unRecipeFunctions_writeInRecipeLog("Creating recipe class: '" + sClassName + "' ...");
  
  // Check if the recipe class already exists
  dpQuery("SELECT '.ProcessInput.ClassName:_online.._value' FROM '*' WHERE '.ProcessInput.ClassName:_online.._value' = \""+sClassName+"\" AND _DPT=\""+UN_CONFIG_UNRCPCLASS_DPT_NAME+"\"", resultTable); 
  if (dynlen(resultTable)>1) {
    fwException_raise(exceptionInfo,"ERROR", "The recipe class '" + sClassName + "' already exists. ","");
	return;	   
  }
  
  // Get the application parameters from the PCO Link
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(sPcoLink, sPcoLinkDpe);
  sAppName = unGenericDpFunctions_getApplication(sPcoLinkDpe);
  
  // Initialize the fwDevice and value archive global variables
  unImportDevice_initialize();
  unImportDevice_getApplicationCharacteristics(bEvent, sPrefix);
  
  // Get necessary data to create a new recipe instance
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(unRecipeFunctions_RcpFEAlias, sPvssDpe);
  if (sPvssDpe==unRecipeFunctions_RcpFEAlias) {
	// The RCP_FE doesn't exist, create it
	// Get the Front End where the PCO link is defined to get the archive files
	string sPcoLinkDpName;
	unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(sPcoLink, sPcoLinkDpName);
	string sPcoFeName = unGenericObject_GetPLCNameFromDpName(sPcoLinkDpName);
	dyn_string dsAnalog, dsBoolean, dsEvent;
	dpGet(sPcoFeName + ".configuration.archive_analog", dsAnalog,
		  sPcoFeName + ".configuration.archive_bool", dsBoolean,
		  sPcoFeName + ".configuration.archive_event", dsEvent);
	
    bool bOk = unRecipeFunctions_createRecipeFrontEnd(sAppName, sPrefix, bEvent, dsAnalog[1], dsBoolean[1], dsEvent[1], exceptionInfo);
	if (bOk == false) {
		fwException_raise(exceptionInfo, "ERROR", "The " + unRecipeFunctions_RcpFEAlias + " SOFT_FE couldn't be created.", "");
		return;
	}
	if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return; }
	unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(unRecipeFunctions_RcpFEAlias, sPvssDpe);
  }

  sFrontEndName = unGenericObject_GetPLCNameFromDpName(sPvssDpe);
  strreplace(sFrontEndName, SOFT_FE_DPTYPE+"_", "");
  
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(sFrontEndName, sFrontEndDp);
  dpGet(sFrontEndDp + "configuration.archive_bool", dsArchBool,
		sFrontEndDp + "configuration.archive_analog", dsArchAnalog,
		sFrontEndDp + "configuration.archive_event", dsArchEvent,
		sFrontEndDp + "communication.driver_num", iDriverNumber);
  
  // [UCPC-1467] Setting the application in the SOFT FE
  dyn_string dsSoftFeConfig;
  dynAppend(dsSoftFeConfig, SOFT_FE_DPTYPE);
  dynAppend(dsSoftFeConfig, unRecipeFunctions_RcpFEAlias);
  dynAppend(dsSoftFeConfig, sAppName);
  dynAppend(dsSoftFeConfig, "N");
  dynAppend(dsSoftFeConfig, iDriverNumber);
  dynAppend(dsSoftFeConfig, dsArchBool[1]);
  dynAppend(dsSoftFeConfig, dsArchAnalog[1]);
  dynAppend(dsSoftFeConfig, "FALSE");
  dynAppend(dsSoftFeConfig, dsArchEvent[1]);
  unConfigSOFT_FE_setApplication(dsSoftFeConfig, sAppName);
  
  fwConfigurationDB_createRecipeClass(sClassName, UN_CONFIG_UNRCPTYPE_DPT_NAME, sDescription, true, makeDynString(), exceptionInfo);
  if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return; }
  
  // Create a unique recipe class DP that is not existing yet
  _unRecipeFunctions_createNewRecipeDp(UN_CONFIG_UNRCPCLASS_DPT_NAME, sFrontEndName, sAppName, sRcpClassDp, iDeviceNumber);
  
  // Create the dsConfig for the device
  dynAppend(dsConfig, UN_CONFIG_UNRCPCLASS_DPT_NAME);	// Recipe instance DPType name
  dynAppend(dsConfig, iDeviceNumber);					// Device number
  dynAppend(dsConfig, sClassName);						// Device alias
  dynAppend(dsConfig, sDescription);					// Description
  dynAppend(dsConfig, "");								// Diagnostics
  dynAppend(dsConfig, "");								// WWWLink
  dynAppend(dsConfig, "");								// Synoptic
  dynAppend(dsConfig, "");								// Domain
  dynAppend(dsConfig, "");								// Nature
  dynAppend(dsConfig, "UnRcpClassState");				// Widget Type
  dynAppend(dsConfig, UN_CONFIG_UNRCPTYPE_DPT_NAME);	// Recipe Type
  dynAppend(dsConfig, sDeviceList);						// Device List
    
  unImportDevice_import(dsConfig, exceptionInfo, SOFT_FE_DPTYPE, sFrontEndName,
					    sAppName, sPrefix, iDriverNumber,
						dsArchBool[1], dsArchAnalog[1], dsArchEvent[1], bEvent,
						dsRunningArchive);
						
  if (dynlen(exceptionInfo)) {
	DebugTN(exceptionInfo);
	return; 
  }
  
  unImportDevice_triggerUpdate();
  
  // Get the default values of the privileged actions
  _unRecipeFunctions_getDeviceDefaultPrivileges(UN_CONFIG_UNRCPCLASS_DPT_NAME, sOperatorPrivileges, sExpertPrivileges, sAdminPrivileges, exceptionInfo);
  if (dynlen(exceptionInfo)) {
	DebugTN(exceptionInfo);
	return; 
  }
  
  // Set the recipe class data
  dpSet(sRcpClassDp + ".statusInformation.deviceLink", sPcoLink,
		sRcpClassDp + ".ProcessInput.ClassName", sClassName,
		sRcpClassDp + ".ProcessInput.RecipeType", UN_CONFIG_UNRCPTYPE_DPT_NAME,
		sRcpClassDp + ".statusInformation.accessControl.operator", sOperatorPrivileges,
		sRcpClassDp + ".statusInformation.accessControl.expert", sExpertPrivileges,
		sRcpClassDp + ".statusInformation.accessControl.admin", sAdminPrivileges);
		
  _unRecipeFunctions_writeInRecipeLog("The recipe class '" + sClassName + "' has been created.", TRUE);
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Callback function from the recipe class panel. The function is called when the devices list has been updated.
 * @param sCommandDp 	- [IN] DP Name of the command over the list of devices.
 * @param iCommand   	- [IN] Command value over the list of devices.
 * @param sSystemNameDp - [IN] DP Name of the system.
 * @param sSystemName   - [IN] System name.
 */
unRecipeFunctions_deviceUpdatedRcpClassCB(string sCommandDp, int iCommand, string sSystemNameDp, string sSystemName)
{
  int iCol;
  bool bAreEqual;
  string sPcoLink, sDpName, sState, sPanelName;
  dyn_float dfReturn;
  dyn_string dsRcpClasses, dsRcpClassesRet, dsRcpClassesDpRet, exceptionInfo, dsReturn;
  
  switch(iCommand) 
  {
    case DEVUN_ADD_COMMAND:
    case DEVUN_DELETE_COMMAND:
    {
      synchronized(g_dsTreeApplication) 
      {
	    // Get the PCO link selected in the panel
		getValue("PcoList", "selectedText", sPcoLink);
		
		// Get the List of recipe classes loaded in the panel
		getValue("RecipeClasses", "nameToColumn", "RecipeClass", iCol);
		getValue("RecipeClasses", "getColumnN", iCol, dsRcpClasses);
		
		// Get the list of recipe classes linked to the PCO
		unRecipeFunctions_getRecipeClasses(dsRcpClassesRet, dsRcpClassesDpRet, sPcoLink);
		
		// Compare the two recipe classes list
		bAreEqual = _unRecipeFunctions_areListsEqual(dsRcpClasses, dsRcpClassesRet);
		
		if (!bAreEqual) {
		  // The list of recipe classes is different and it should be reloaded to avoid inconsistencies
		  getValue("LoadedRcpDp", "text", sDpName);

		  if (sDpName != "" && sDpName != "_UnRcpClass") {
		    // Check if the current recipe class is being edited
		    dpGet(sDpName + ".ProcessInput.State", sState);
		    if (sState != "") {
			  // The recipe class is being edited, display a confirmation message
			  unGraphicalFrame_ChildPanelOnCentralModalReturn(
				"vision/MessageInfo",
				sPanelName,
				makeDynString("$1:The list of recipe classes has been modified.\nDo you want update the recipe class list?\n" +
						+ "(The changes of the recipe class will be applied)", "$2:Yes","$3:No"), 
				dfReturn, 
				dsReturn);
			  if(dynlen(dfReturn)>0 && dfReturn[1]==1)
			  { // Apply the changes to the recipe class
			    unRecipeFunctions_applyRecipeClassModifications(sDpName);
			  } else {
			    return;
			  }
		    }
			unRecipeFunctions_loadRecipeClasses(sPcoLink, exceptionInfo);
		    unRecipeFunctions_selectRecipeClass(sDpName, exceptionInfo);
		  } else {
		    // There is no recipe class selected, load the recipe classes
		    unRecipeFunctions_loadRecipeClasses(sPcoLink, exceptionInfo);
		  }
		}
      }
      break;
    }
  }
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Callback function from the recipe instance panel. The function is called when the devices list has been updated.
 * @param sCommandDp 	- [IN] DP Name of the command over the list of devices.
 * @param iCommand   	- [IN] Command value over the list of devices.
 * @param sSystemNameDp - [IN] DP Name of the system.
 * @param sSystemName   - [IN] System name.
 */
unRecipeFunctions_deviceUpdatedRcpInstanceCB(string sCommandDp, int iCommand, string sSystemNameDp, string sSystemName)
{
  int iCol;
  bool bAreEqual;
  string sClassName, sLoadedRecipeInstance, sState, sPanelName;
  dyn_string dsRcpInstances, dsRcpInstanceNamesRet, dsRcpInstanceDpsRet, exceptionInfo, dsReturn;
  dyn_float dfReturn;
  
  switch(iCommand) 
  {
    case DEVUN_ADD_COMMAND:
    case DEVUN_DELETE_COMMAND:
    {
      synchronized(g_dsTreeApplication) 
      {	
		// Get the recipe class selected in the panel
		getValue("RecipeClasses", "selectedText", sClassName);

		// Get the list of recipe instances displayed in the panel
		getValue("RecipeInstances", "nameToColumn", "RecipeInstances", iCol);
		getValue("RecipeInstances", "getColumnN", iCol, dsRcpInstances);
		
		// Get the list of recipe instances of the selected class
		_unRecipeFunctions_getRecipeInstancesOfClass(sClassName, dsRcpInstanceNamesRet, dsRcpInstanceDpsRet);

		// Compare the two recipe classes list
		bAreEqual = _unRecipeFunctions_areListsEqual(dsRcpInstances, dsRcpInstanceNamesRet);
		
		if (!bAreEqual) {
		  // The list of recipe instances is different and it should be reloaded to avoid inconsistencies
		  
		  getValue("LoadedRecipeDp", "text", sLoadedRecipeInstance);
		  if (sLoadedRecipeInstance!="") {
  		    // Check if the current recipe has been removed
			  
			if (!dpExists(sLoadedRecipeInstance + ".ProcessInput.State")) {
			  _unRecipeFunctions_writeInRecipeLog("The recipe has been removed.");
			  _unRecipeFunctions_clearInstanceTables();
			  _unRecipeFunctions_loadRecipeInstances(sClassName);
			  return;
			} 
			  
			// Check if the current recipe is being edited
			dpGet(sLoadedRecipeInstance + ".ProcessInput.State", sState);
			if (sState == UN_RCP_INSTANCE_STATE_EDIT) {
			  // The recipe is being edited, display a confirmation message
			  unGraphicalFrame_ChildPanelOnCentralModalReturn(
					"vision/MessageInfo",
					sPanelName,
					makeDynString("$1:The list of recipe instances has been modified.\nDo you want update the recipe instance list?\n" +
							+ "(The changes of the recipe will be applied)", "$2:Yes","$3:No"), 
					dfReturn, 
					dsReturn);
			  if(dynlen(dfReturn)>0 && dfReturn[1]==1)
			  { // Apply the changes to the recipe 
				unRecipeFunctions_saveRecipeInstance($sDpName, exceptionInfo, FALSE);
			  } else {
				return;
			  }
			}
		  }
		  _unRecipeFunctions_loadRecipeInstances(sClassName);
		  unRecipeFunctions_selectRecipeInstance(sLoadedRecipeInstance, exceptionInfo);
		}
      }
      break;
    }
  }
}
//------------------------------------------------------------------------------------------------------------------------
/**
 * Function to check if the items of two lists are equal.
 * @param list1 - [IN] The first list of elements.
 * @param list2 - [IN] The second list of elements.
 * @return TRUE if the lists contain the same elements, otherwise FALSE.
 */
bool _unRecipeFunctions_areListsEqual(dyn_string list1, dyn_string list2) {
  if (dynlen(list1) != dynlen(list2))
    return false;
	
  dynSortAsc(list1);
  dynSortAsc(list2);
  
  int iLen = dynlen(list1);
  for (int i=1; i<=iLen; i++) {
    if (list1[i] != list2[i])
	  return false;
  }
  
  return true;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Open a new window to edit the DPEs of the devices included in the recipe.
 * @param sRcpClassDp - [IN] The recipe class DP.
 */
void unRecipeFunctions_editRecipeClassDeviceElements(string sRcpClassDp) {
  int i, iLen, j, jLen, k, kLen;
  dyn_int diLines;
  dyn_float df;
  dyn_string ds, exceptionInfo;
  string sSelectedDevices = "", sCellValue, sDescription, sAlias, sState;
  
  // Get the recipe class state
  dpGet(sRcpClassDp + ".ProcessInput.State", sState);
  if (sState == "") {
	dpSet(sRcpClassDp + ".ProcessInput.State", UN_RCP_CLASS_STATE_EDIT);
  }
  
  sDescription = dpGetDescription(sRcpClassDp+".");
  getValue("RCCElements.RecipeElementsTable", "getSelectedLines", diLines);
  iLen = dynlen(diLines);
  for (i=1; i<=iLen; i++) {
	getValue("RCCElements.RecipeElementsTable", "cellValueRC", diLines[i], "DeviceName", sCellValue);
	if (i>1) {
	  sSelectedDevices += ",";
	}
	sSelectedDevices += sCellValue;
  }
  
  sAlias = dpGetAlias(sRcpClassDp + ".");

  ChildPanelOnCentralModalReturn(
		"vision/unRecipe/UnRcpClass/unUnRcpClass_EditDeviceElements.pnl",
		sAlias + " - Edit recipe elements",
		makeDynString(
				"$sDpName:" + sRcpClassDp,
				"$sSelectedDevices:" + sSelectedDevices,
				"$sDescription:" + sDescription),
		df, ds);

  // Check if the user pressed the cancel button
  if (dynlen(df)>0 && df[1]==0) {
	dpSet(sRcpClassDp + ".ProcessInput.State", sState);
	return;
  }
		
  // If the user clicked the Ok button, reload the recipe class data
  if(dynlen(df)>0 && df[1]==1) {
	mapping mDeviceTypes;
	mapping mDeviceAlias;
	dyn_string dsDeviceTypeList;
	dyn_string dsDeviceAliasList;
	dyn_string dsDeviceElementList;
	for (i=1; i<=dynlen(ds); i++) {
	  string sRcpElement = ds[i];
	  string pvssDpe;
	  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(sRcpElement, pvssDpe);
	  dyn_string elementSplit = strsplit(sRcpElement, ".");
	  string sType = dpTypeName(pvssDpe);
	  string sDpe  = elementSplit[(dynlen(elementSplit))];  
	  _unRecipeFunctions_mappingInsertValue(mDeviceTypes, sType, elementSplit[1]);
	  _unRecipeFunctions_mappingInsertValue(mDeviceAlias, elementSplit[1], sDpe);
	}
	
	dyn_string dsDeviceTypes = mappingKeys(mDeviceTypes);
	iLen = dynlen(dsDeviceTypes);
	dynSortAsc(dsDeviceTypes);
	for (i=1; i<=iLen; i++) {
	  string sType = dsDeviceTypes[i];
	  // Get the list of devices belonging to the device type sType
	  dyn_string dsDeviceAlias = mDeviceTypes[sType];
	  dynSortAsc(dsDeviceAlias);
	  jLen = dynlen(dsDeviceAlias);
	  for (j=1; j<=jLen; j++) {
		string sAlias = dsDeviceAlias[j];
		dyn_string dsDpes = mDeviceAlias[sAlias];
		string sDpes = "";
		kLen = dynlen(dsDpes);
		for (k=1; k<=kLen; k++) {
		  if (k>1)
			sDpes += ", ";
		  sDpes += dsDpes[k];
		}
		// Append the data to the dynamic list that will be loaded in the table
		dynAppend(dsDeviceTypeList, sType);
	    dynAppend(dsDeviceAliasList, sAlias);
		dynAppend(dsDeviceElementList, sDpes);
	  }
	}
	
	// Set the data in the table
	setValue("RCCElements.RecipeElementsTable", "deleteAllLines");
	setValue("RCCElements.RecipeElementsTable", "appendLines", dynlen(dsDeviceTypeList), 
				"DeviceType", dsDeviceTypeList,
				"DeviceName", dsDeviceAliasList,
				"DeviceElements", dsDeviceElementList);
				
	_unRecipeFunctions_writeInRecipeLog("The recipe class elements have been modified. Please apply the changes to the recipe class.");
	setValue("applyChangesFrame", "visible", true);
  } else {
    // Restore the recipe class state
	dpSet(sRcpClassDp + ".ProcessInput.State", sState);	
  }	
  
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Open a new window to edit the recipe class privileges and parameters.
 * @param sRcpClassDp - [IN] The recipe class DP.
 */
void unRecipeFunctions_editRecipeClassPrivileges(string sRcpClassDp) {
  int i, iLen;
  string sClassName, sDefinedACDomains, sClassACDomain, sClassOperatorAction, sClassExpertAction, sClassAdminAction;
  string sRcpInstanceDp, sInstanceACDomain, sInstanceOperatorAction, sInstanceExpertAction, sInstanceAdminAction;
  dyn_string ds, dsAccessControlDomains, dsFullAccessControlDomain, exceptionInfo, dsRcpInstanceNames, dsRcpInstanceDps;
  dyn_float df;
  
  // Get the recipe class privileges and ACDomain.
  sClassName = dpGetAlias(sRcpClassDp + ".");
  _unRecipeFunctions_getDevicePrivileges(sRcpClassDp, sClassACDomain, sClassOperatorAction, 
			sClassExpertAction, sClassAdminAction);
			
  // Get the recipe instance privileges and ACDomain.
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(sClassName + "/" + UN_RCP_PREDEFINED_NAME, sRcpInstanceDp);

  dpGet(sRcpInstanceDp + "accessControl.accessControlDomain", sInstanceACDomain,
        sRcpInstanceDp + "accessControl.operator", sInstanceOperatorAction,
		sRcpInstanceDp + "accessControl.expert", sInstanceExpertAction,
		sRcpInstanceDp + "accessControl.admin", sInstanceAdminAction);
		
  // Get the list of access control domains
  fwAccessControl_getAllDomains(dsAccessControlDomains, dsFullAccessControlDomain, exceptionInfo);
  if (dynlen(exceptionInfo)) {
    string errMsg = "Error getting the list of access control domains.";
	_unRecipeFunctions_writeInRecipeLog(errMsg);
	DebugTN(errMsg, exceptionInfo);
	return;
  }
  
  // Remove the SYSTEM DOMAIN if it exists
  if ( (i=dynContains(dsAccessControlDomains, "SYSTEM")) > 0) {
	dynRemove(dsAccessControlDomains, i);
  }
  fwGeneral_dynStringToString(dsAccessControlDomains, sDefinedACDomains, ",");

  // Display the panel with the recipe class and recipe instance privileges  
  ChildPanelOnCentralModalReturn(
		"vision/unRecipe/UnRcpClass/unUnRcpClass_Privileges.pnl",
		sClassName + " - Edit recipe privileges",
		makeDynString(
			"$sDpName:" + sRcpClassDp,
			"$sDefinedACDomains:" + sDefinedACDomains,
			"$sClassACDomain:" + sClassACDomain,
			"$sClassOperatorAction:" + sClassOperatorAction,
			"$sClassExpertAction:" + sClassExpertAction,
			"$sClassAdminAction:" + sClassAdminAction,
			"$sInstanceACDomain:" + sInstanceACDomain,
			"$sInstanceOperatorAction:" + sInstanceOperatorAction,
			"$sInstanceExpertAction:" + sInstanceExpertAction,
			"$sInstanceAdminAction:" + sInstanceAdminAction
			),
		df, ds);
		
  // Check if the Ok button has been pressed
  if (dynlen(df) && df[1]>0) {
    // Set the recipe class privileges
	sClassACDomain = ds[1];
	sClassOperatorAction = ds[2];
	sClassExpertAction = ds[3];
	sClassAdminAction = ds[4];	
	_unRecipeFunctions_setDevicePrivileges(sRcpClassDp, sClassACDomain, sClassOperatorAction, 
			sClassExpertAction, sClassAdminAction, exceptionInfo);
	
	// Get the recipe instance privileges
	sInstanceACDomain = ds[5];
	sInstanceOperatorAction = ds[6];
	sInstanceExpertAction = ds[7];
	sInstanceAdminAction = ds[8];

	// Apply the privileges to the default recipe instance
	dpSet(sRcpInstanceDp + "accessControl.accessControlDomain", sInstanceACDomain,
        sRcpInstanceDp + "accessControl.operator", sInstanceOperatorAction,
		sRcpInstanceDp + "accessControl.expert", sInstanceExpertAction,
		sRcpInstanceDp + "accessControl.admin", sInstanceAdminAction);
	
	// Get all the recipe instances of the class
	_unRecipeFunctions_getRecipeInstancesOfClass(sClassName, dsRcpInstanceNames, dsRcpInstanceDps);
	// Set the recipe instances privileges
	iLen = dynlen(dsRcpInstanceDps);
	for (i=1; i<=iLen; i++) {
		_unRecipeFunctions_setDevicePrivileges(dsRcpInstanceDps[i], sInstanceACDomain, sInstanceOperatorAction, 
			sInstanceExpertAction, sInstanceAdminAction, exceptionInfo);
	}
	
	_unRecipeFunctions_writeInRecipeLog("The privileges of the recipe class " + sClassName + 
			" and all its instances have been modified.", TRUE);
  }
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Auxiliary function to get the device privileges.
 * @param sDpName 	   - [IN] Datapoint name of the device.
 * @param sAcDomain	   - [OUT] Access control domain for the device.
 * @param sOperatorAction - [OUT] Privileged actions for the operator.
 * @param sExpertAction   - [OUT] Privileged actions for the expert.
 * @param sAdminAction	  - [OUT] Privileged actions for the admin.
 */
void _unRecipeFunctions_getDevicePrivileges(string sDpName, string & sAcDomain, string & sOperatorAction, 
			string & sExpertAction, string & sAdminAction) 
{
	if (sDpName[(strlen(sDpName)-1)] == ".") {
		sDpName = substr(sDpName, 0, strlen(sDpName)-1);
	}

	dpGet(sDpName + ".statusInformation.accessControl.accessControlDomain", sAcDomain,
 		  sDpName + ".statusInformation.accessControl.operator", sOperatorAction,
		  sDpName + ".statusInformation.accessControl.expert", sExpertAction,
		  sDpName + ".statusInformation.accessControl.admin", sAdminAction);
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Auxiliary function to set the device privileges.
 * @param sDpName 	      - [IN] Datapoint name of the device.
 * @param sAcDomain	      - [IN] Access control domain for the device.
 * @param sOperatorAction - [IN] Privileged actions for the operator.
 * @param sExpertAction   - [IN] Privileged actions for the expert.
 * @param sAdminAction	  - [IN] Privileged actions for the admin.
 * @param exceptionInfo   - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_setDevicePrivileges(string sDpName, string sAcDomain, string sOperatorAction, 
			string sExpertAction, string sAdminAction, dyn_string & exceptionInfo) 
{
	if (sDpName[(strlen(sDpName)-1)] == ".") {
		sDpName = substr(sDpName, 0, strlen(sDpName)-1);
	}
	
	dpSet(sDpName + ".statusInformation.accessControl.accessControlDomain", sAcDomain,
 		  sDpName + ".statusInformation.accessControl.operator", sOperatorAction,
		  sDpName + ".statusInformation.accessControl.expert", sExpertAction,
		  sDpName + ".statusInformation.accessControl.admin", sAdminAction);
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Function used to get the default privileges for the recipe class actions.
 * @param sRoleType    - [IN] Role type (operator, expert or admin).
 * @param dsPrivileges - [OUT] List of default privileged actions for the specified role.
 */
void _unRecipeFunctions_getDefaultClassPrivileges(string sRoleType, dyn_string & dsPrivileges) 
{
  string sRole = strtolower(sRoleType);
  dynClear(dsPrivileges);
  
  if (sRole=="operator") {
    return;
  } else if (sRole=="expert") {
    return;
  } else if (sRole=="admin") {
    dynAppend(dsPrivileges, makeDynString("AddDevices", "ApplyChanges", "Cancel", "Duplicate", "EditElements", "NewClass", 
				"NewInstance", "Privileges", "RemoveClass", "RemoveSelected", UN_FACEPLATE_BUTTON_SELECT));
  }
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Function used to get the default privileges for the recipe instance actions.
 * @param sRoleType    - [IN] Role type (operator, expert or admin).
 * @param dsPrivileges - [OUT] List of default privileged actions for the specified role.
 */
void _unRecipeFunctions_getDefaultInstancePrivileges(string sRoleType, dyn_string & dsPrivileges) 
{
  string sRole = strtolower(sRoleType);
  dynClear(dsPrivileges);
  
  if (sRole=="operator") {
    dynAppend(dsPrivileges, makeDynString("Activate", UN_FACEPLATE_BUTTON_SELECT));
  } else if (sRole=="expert"){
    dynAppend(dsPrivileges, makeDynString("Cancel", "Delete", "Duplicate", "Edit", "InitialRecipes", "NewInstance", 
				"LastActivatedRecipes", "OnlineValues", "Save",  "SaveAsInitial"));
  } else if (sRole=="admin") {
  }
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Auxiliary function to add values to a mapping where the mapping value is a dynamic list.
 * @param m   		   - [OUT] Mapping where to insert the values.
 * @param key 		   - [IN] Key for the new value.
 * @param value        - [IN] The value to insert.
 * @param bUniqueValue - [IN] If TRUE the values can not be repeated for the same key.
 */
void _unRecipeFunctions_mappingInsertValue(mapping & m, string key, string value, bool bUniqueValue=TRUE) 
{
  dyn_string dsValues;
  
  if (mappingHasKey(m, key)) {
    dsValues = m[key];
  }
  
  if (bUniqueValue==FALSE || FALSE==dynContains(dsValues, value))
	dynAppend(dsValues, value);

  m[key] = dsValues;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Create the SOFT_FE for the recipes.
 * @param sAppName - [IN] Application name.
 * @param sPrefix  - [IN] Application prefix.
 * @param bEvent   - [IN] TRUE=16 bit event type, FALSE=32 bit event type
 * @param sAnalog  - [IN] Name of the analog archive file.
 * @param sBool    - [IN] Name of the boolean archive file.
 * @param sEvent   - [IN] Name of the event archive file.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */ 
bool unRecipeFunctions_createRecipeFrontEnd(string sAppName, string sPrefix, bool bEvent, string sAnalog, string sBool, string sEvent, dyn_string & exceptionInfo)
{
  string sPvssDpe;
  
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(unRecipeFunctions_RcpFEAlias, sPvssDpe);
  if (sPvssDpe==unRecipeFunctions_RcpFEAlias) {

	dyn_float df;
	dyn_string ds;
	ChildPanelOnCentralModalReturn(
		"vision/unRecipe/UnRcpFrontEnd/unUnRcpFrontEnd_Create.pnl",
		"Recipe Front End Creation",
		makeDynString(
			"$sAppName:" + sAppName,
			"$sPrefix:" + sPrefix,
			"$bEvent:" + bEvent,
			"$sAnalog:" + sAnalog,
			"$sBool:" + sBool,
			"$sEvent:" + sEvent),
		df, ds);

		if (dynlen(df)==1 && df[1]==1)
			return true;
		return false;
  } 
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Create a new recipe instance of the selected recipe class.
 */
void unRecipeFunctions_createRecipeInstance()
{
  string sAllowInitialRecipe = "FALSE";
  string sRecipeClass, sDpName, sRcpName, sRcpDesc, sInitial, sRecipeClassesType;
  bool bRecipeInstancesTableExist;
  dyn_int lines;
  dyn_string exceptionInfo, ds;
  dyn_float df;
  dyn_dyn_anytype resultTable;
  
  // Check if the recipe instances table exist (if true, the method has been called from the
  // recipe instances panel, otherwise from the recipe class panel
  bRecipeInstancesTableExist = shapeExists("RecipeInstances");
  
  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeClasses")) {
	_unRecipeFunctions_writeInRecipeLog("Error: Some graphical shapes are missing.");
	return;
  } 
  
  // Get the type of the RecipeClasses shape (it can be "Table" or "ComboBox")
  getValue("RecipeClasses", "type", sRecipeClassesType);  
  if (sRecipeClassesType=="Table") {
	// Get the selected recipe class from the table
	getValue("RecipeClasses", "getSelectedLines", lines); 
	if (!dynlen(lines)>0) {
		_unRecipeFunctions_writeInRecipeLog("Error: Select a recipe class before creating a recipe.");
		return;
	}    
	getValue("RecipeClasses", "cellValueRC", lines[1], "RecipeClass", sRecipeClass);  
  } else if (sRecipeClassesType=="ComboBox") {
    // Get the selected recipe class from the combo box
	getValue("RecipeClasses", "selectedText", sRecipeClass);
  } else {
    // The type of the RecipeClasses shape is unknown
	_unRecipeFunctions_writeInRecipeLog("Error: The type of the RecipeClasses shape is unknown.");
	return;
  }
  
  // Check if the recipe class name is empty
  if (sRecipeClass=="" || sRecipeClass=="-None-") {
    _unRecipeFunctions_writeInRecipeLog("Error: Please, select a recipe class before creating a recipe.");
    return;
  }
  
  // Check if the recipe class exists
  sDpName = _unGenericDpFunctions_dpAliasToName(sRecipeClass);
  if (sDpName == "") {
	_unRecipeFunctions_writeInRecipeLog("Error: The recipe class '" + sRecipeClass + "' doesn't exist.");
	return;
  }

  // Check if an initial recipe of the class already exists
  dpQuery("SELECT '.RecipeName:_online.._value' FROM '*' WHERE '.RecipeName:_online.._value' LIKE \"" + sRecipeClass + "/*\" AND '.MetaInfo.Predefined:_online.._value' = \"TRUE\"  AND _DPT=\"_FwRecipeCache\" ", resultTable);   
  // Check if there isn't an initial recipe for the recipe class 
  if (dynlen(resultTable)<=1) {
    sAllowInitialRecipe = "TRUE"; 
  }
  
  ChildPanelOnCentralReturn("vision/unRecipe/UnRcpInstance/unUnRcpInstance_Duplicate.pnl",
			"Create Recipe Instance",
			makeDynString("$sTitle:Create Recipe of class: " + sRecipeClass,
				      "$sDescription: Input the name for the new recipe instance",
					  "$sAllowInitialRecipe:"+sAllowInitialRecipe),
			df,ds);
  
  if (!dynlen(df) || df[1]<0) return;
  if (!dynlen(ds) || ds[1]=="") {
    _unRecipeFunctions_writeInRecipeLog("Error: The recipe name can't be empty. ");
	DebugTN(exceptionInfo);
    return;     
  }  			
  
  sRcpName = ds[1];
  sRcpDesc = ds[2];
  sInitial = ds[3];

  if (!dpIsLegalName(sRcpName)) {
	_unRecipeFunctions_writeInRecipeLog("Error: Wrong recipe name: " + sRcpName);
    return;  
  }
  
  _unRecipeFunctions_writeInRecipeLog("Creating recipe '" + sRcpName + "' of class '" + sRecipeClass + "' ...");
  _unRecipeFunctions_createRecipeInstance(sRecipeClass, sRcpName, sRcpDesc, exceptionInfo, sInitial);
  if (dynlen(exceptionInfo)) { 
    _unRecipeFunctions_writeInRecipeLog("Error: " + exceptionInfo[2]);
	DebugTN(exceptionInfo); 
	return; 
  }
  _unRecipeFunctions_writeInRecipeLog("The recipe '" + sRcpName + "' of class '" + sRecipeClass + "' has been created.", TRUE);
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Create a new recipe instance.
 * @param sClassName 	- [IN] Name of the recipe class.
 * @param sInstanceName - [IN] Name of the new recipe instance.
 * @param sDescription  - [IN] Description for the new recipe instance.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 * @param sInitial		- [IN] Flag to define if the new recipe is the initial recipe of the class 
 * @param dsRcpValues 	- [IN] Recipe values. If the parameter isn't specified, the recipe elements will have the online values.
 */ 
void _unRecipeFunctions_createRecipeInstance(string sClassName, string sInstanceName, string sDescription, 
											 dyn_string &exceptionInfo, string sInitial="FALSE", dyn_string dsRcpValues=makeDynString())
{
  int iDeviceNumber, iDriverNumber, iLen, iNumInstances;
  string sDpName, sPrefix, sDpeValue = "", sPvssDpe, sFrontEndName, sFrontEndDp, sAppName, sDomain, sNature;
  string sOperatorPrivileges, sExpertPrivileges, sAdminPrivileges;
  dyn_mixed recipeClass;
  dyn_string dsConfig, dsRunningArchive, dsSplitDp, dsOnlineDpes, dsRcpNames, dsArchBool, dsArchAnalog, dsArchEvent, dsParameters;
  dyn_string dsOperatorPrivileges, dsExpertPrivileges, dsAdminPrivileges;
  dyn_dyn_anytype resultTable;
  bool bEvent;
  
  //DebugTN("START - _unRecipeFunctions_createRecipeInstance");
  // check if the recipe instance already exists
  // Get all the instances of the recipe class
  dpQuery("SELECT '.ProcessInput.InstanceName:_online.._value' FROM '*' WHERE '.ProcessInput.ClassName:_online.._value' = \""+sClassName+
				"\" AND _DPT=\""+UN_CONFIG_UNRCPINSTANCE_DPT_NAME+"\"", resultTable); 
  
  if (dynlen(resultTable)>1) {
    dynRemove(resultTable, 1);
    for (int i=1; i<=dynlen(resultTable); i++) {
		// If the recipe instance already exists, show an error message and return
		if (resultTable[i][2] == sInstanceName) {
			fwException_raise(exceptionInfo,"ERROR", "The recipe " + sClassName + "/" + sInstanceName +" already exists. ","");
			DebugTN(exceptionInfo);
			return;	
		}
    }
  }

  // Get necessary data to create a new recipe instance
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(sClassName, sPvssDpe);
  sFrontEndName = unGenericObject_GetPLCNameFromDpName(sPvssDpe);
  strreplace(sFrontEndName, SOFT_FE_DPTYPE+"_", "");
  sAppName = unGenericDpFunctions_getApplication(sPvssDpe);
  unGenericDpFunctions_getParameters(sPvssDpe, dsParameters);
  sDomain = dsParameters[4];
  sNature = dsParameters[5];

  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(sFrontEndName, sFrontEndDp);
  dpGet(sFrontEndDp + "configuration.archive_bool", dsArchBool,
		sFrontEndDp + "configuration.archive_analog", dsArchAnalog,
		sFrontEndDp + "configuration.archive_event", dsArchEvent,
		sFrontEndDp + "communication.driver_num", iDriverNumber);
  
  fwConfigurationDB_getRecipeClassInfo(sClassName, recipeClass, exceptionInfo);
  if (dynlen(exceptionInfo)) {	// || dynlen(recipeClass[fwConfigurationDB_RCL_ELEMENTS])==0 ) 
    fwException_raise(exceptionInfo,"ERROR", "Error getting the recipe class data. The recipe can't be created.", "");
	DebugTN(exceptionInfo);
	return;
  }
  
  //DebugTN("Recipe class info: " + sClassName, recipeClass);
  
  // Creates a unique recipe DP that is not existing yet
  _unRecipeFunctions_createNewRecipeDp(UN_CONFIG_UNRCPINSTANCE_DPT_NAME, sFrontEndName, sAppName, sDpName, iDeviceNumber);
  //DebugTN("New recipe instance DP created: " + sDpName);
 
  // Initialize the fwDevice and value archive global variables
  unImportDevice_initialize();
  unImportDevice_getApplicationCharacteristics(bEvent, sPrefix);

  iLen = dynlen(recipeClass[fwConfigurationDB_RCL_ELEMENTS]);
  int i=1;
  if (iLen>0) {	// Execute the block only if the recipe has some elements (Otherwise it's the predefined recipe of the class)
	if (dynlen(dsRcpValues)==0 || dynlen(dsRcpValues)!=iLen) { 
	  // If no values specified, get the online values
	  do {
		//DebugTN("Getting online values for the new recipe instance");
		dsSplitDp = strsplit(recipeClass[fwConfigurationDB_RCL_ELEMENTS][i], ".");
		unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(dsSplitDp[1] + "." + dsSplitDp[dynlen(dsSplitDp)], sPvssDpe);
		_unRecipeFunctions_getOnlineValueDpes (sPvssDpe, dsOnlineDpes);
		dpGet(dsOnlineDpes, dsRcpValues);
		if (i>1) sDpeValue += ",";
		sDpeValue += dsSplitDp[1] + "." + dsSplitDp[dynlen(dsSplitDp)] + "|" + dsRcpValues[1];
		i++;
	  } while (i<=iLen);
	} else {
	  // Set the specified values for the recipe 
	  do {
		dsSplitDp = strsplit(recipeClass[fwConfigurationDB_RCL_ELEMENTS][i], ".");
		if (i>1) sDpeValue += ",";
		sDpeValue += dsSplitDp[1] + "." + dsSplitDp[dynlen(dsSplitDp)] + "|" + dsRcpValues[i];
		i++;
	  } while (i<=iLen);
	}
  }
  
  //DebugTN("sDpeValue: " + sDpeValue);
  //DebugTN("sInitial : " + sInitial);
  
  // Creates the dsConfig for the device
  dynAppend(dsConfig, UN_CONFIG_UNRCPINSTANCE_DPT_NAME);	// Recipe instance DPType name
  dynAppend(dsConfig, iDeviceNumber);						// Device number
  dynAppend(dsConfig, sClassName+"/"+sInstanceName);		// Device alias
  dynAppend(dsConfig, sDescription);						// Description
  dynAppend(dsConfig, "");									// Diagnostics
  dynAppend(dsConfig, "");									// WWWLink
  dynAppend(dsConfig, "");									// Synoptic
  dynAppend(dsConfig, sDomain);								// Domain
  dynAppend(dsConfig, sNature);								// Nature
  dynAppend(dsConfig, "UnRcpInstanceState");				// Widget Type
  dynAppend(dsConfig, sClassName);							// Recipe class
  dynAppend(dsConfig, sInitial);							// Initial recipe
  dynAppend(dsConfig, sDpeValue);							// Recipe DPE-Value definitions

  //DebugTN("dsConfig: ", dsConfig);

  unImportDevice_import(dsConfig, exceptionInfo, SOFT_FE_DPTYPE, sFrontEndName,
					    sAppName, sPrefix, iDriverNumber,
						dsArchBool[1], dsArchAnalog[1], dsArchEvent[1], bEvent,
						dsRunningArchive);
						
  if (dynlen(exceptionInfo)) {
	if (dpExists(sDpName)) {
		dpDelete(sDpName);
	}
	return; 
  }
  
  unImportDevice_triggerUpdate();
  
  // Get the default values of the privileged actions
  _unRecipeFunctions_getDeviceDefaultPrivileges(UN_CONFIG_UNRCPINSTANCE_DPT_NAME, sOperatorPrivileges, sExpertPrivileges, sAdminPrivileges, exceptionInfo);
  
  // Set the recipe instance privileges
  dpSet(sDpName + ".statusInformation.accessControl.operator", sOperatorPrivileges,
		sDpName + ".statusInformation.accessControl.expert", sExpertPrivileges,
		sDpName + ".statusInformation.accessControl.admin", sAdminPrivileges);
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Modify the font type of the edited rows to Bold.
 * @param sDpName - [IN] The data point name of the edited row.
 */
void _unRecipeFunctions_highlightEditedValue(string sDpName)
{
	int retVal;
	string newValue;
	dyn_string exceptionInfo, formattedValues;
	dyn_dyn_mixed recipeObject;
	
	if (g_editingRowNumber < 0 || g_editingColName=="") {
		return;
	}
	
	_unRecipeFunctions_getRecipeObjectFromDp (sDpName, recipeObject, exceptionInfo);
	getValue("RecipeElements", "cellValueRC", g_editingRowNumber, g_editingColName, newValue);
					
	_unRecipeFunctions_formatValues(makeDynString(recipeObject[fwConfigurationDB_RO_DPE_NAME][g_editingRowNumber+1]), 
					makeDynString(newValue), formattedValues, exceptionInfo);

	setValue ("RecipeElements", "cellValueRC", g_editingRowNumber, g_editingColName, formattedValues[1]);
	if (formattedValues[1] != recipeObject[fwConfigurationDB_RO_VALUE][g_editingRowNumber+1]) {
		RecipeElements.rowFontType (g_editingRowNumber, makeDynInt(1));
	}
	
	g_editingRowNumber = -1;
	g_editingColName = "";	
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * The edition of a cell value has been completed.
 * @param sDpName 	- [IN] The data point name of the recipe instance.
 * @param row	  	- [IN] Row number of the edited cell.
 * @param column  	- [IN] Column name of the edited cell.
 * @param iRowIndex - [IN] Row index in the "Index" column of the table (proper order in the JCOP recipe definition).
 */
void _unRecipeFunctions_cellEditionFinished(string sDpName, int row, string column, int iRowIndex)
{
	int retVal;
	string newValue;
	dyn_string exceptionInfo, formattedValues;
	dyn_dyn_mixed recipeObject;
	
	_unRecipeFunctions_getRecipeObjectFromDp (sDpName, recipeObject, exceptionInfo);
	getValue("RecipeElements", "cellValueRC", row, column, newValue);
	
	// Check if the recipe value is in the defined range.
	retVal = unRecipeFunctions_checkRecipeValue (
					recipeObject[fwConfigurationDB_RO_DPE_NAME][iRowIndex], 
					newValue, 
					exceptionInfo);
	
	// If the value is out of range shows an error message and modifies the cell color
	if (retVal == -1) {
	  _unRecipeFunctions_writeInRecipeLog("The new value is out of range");
	  setValue("RecipeElements", "cellBackColRC", (row), "Value", CELL_VALUE_OUT_OF_RANGE_COLOR);
	  return;
	} else {
		// Restore the cell color
		setValue("RecipeElements", "cellBackColRC", (row), "Value", CELL_EDIT_COLOR);
	}
	
	_unRecipeFunctions_formatValues(makeDynString(recipeObject[fwConfigurationDB_RO_DPE_NAME][iRowIndex]), 
								    makeDynString(newValue), formattedValues, exceptionInfo);

	if (dynlen(exceptionInfo)) {
		// The value introduced is wrong
		_unRecipeFunctions_writeInRecipeLog("The introduced value is not valid: '" + newValue + "'.");
		setValue ("RecipeElements", "cellValueRC", row, column, recipeObject[fwConfigurationDB_RO_VALUE][iRowIndex]);	
		RecipeElements.rowFontType (row, makeDynInt(0));
	} else {
		setValue ("RecipeElements", "cellValueRC", row, column, formattedValues[1]);
		if (formattedValues[1] != recipeObject[fwConfigurationDB_RO_VALUE][iRowIndex]) {
			RecipeElements.rowFontType (row, makeDynInt(1));
		} else {
			RecipeElements.rowFontType (row, makeDynInt(0));
		}
	}
	g_editingRowNumber = -1;
	g_editingColName = "";	
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Duplicate the selected recipe instance.
 * @param sRecipeDp - [IN] DP name of the recipe instance to duplicate.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_duplicateRecipeInstance(string sRecipeDp, dyn_string &exceptionInfo)
{
  string sRecipeInstance, sRecipeClass, sRecipeDesc, sInitial;
  dyn_float df;
  dyn_string ds;
  dyn_dyn_mixed recipeObject;  
  
  // Recipe instance in DUPLICATE mode
  dpSet(sRecipeDp+".ProcessInput.State", UN_RCP_INSTANCE_STATE_DUPLICATE);
  
  if (!shapeExists("RecipeClasses") || !shapeExists("RecipeInstances")) {
    dpSet(sRecipeDp+".ProcessInput.State", "");
	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	return;
  }
  
  _unRecipeFunctions_getRecipeObjectFromDp(sRecipeDp, recipeObject, exceptionInfo);
  if (dynlen(exceptionInfo)) { 
	dpSet(sRecipeDp+".ProcessInput.State", "");
	DebugTN(exceptionInfo);	
	return; 
  }
  
  ChildPanelOnCentralReturn("vision/unRecipe/UnRcpInstance/unUnRcpInstance_Duplicate.pnl",
			"Duplicate Recipe Instance",
			makeDynString("$sTitle:Duplicate Recipe Instance",
				      "$sDescription: Input the name for the new recipe instance"),
			df,ds);
  
  if (!dynlen(df) || df[1]<0) {
	dpSet(sRecipeDp+".ProcessInput.State", "");
	return;
  }
  
  if (!dynlen(ds) || ds[1]=="") {
    dpSet(sRecipeDp+".ProcessInput.State", "");
    fwException_raise(exceptionInfo,"ERROR", "The recipe name can't be empty. ","");
    return;     
  }  
  
  sRecipeInstance = ds[1];
  sRecipeDesc = ds[2];
  sInitial = "FALSE";
  sRecipeClass = recipeObject[fwConfigurationDB_RO_META_CLASSNAME];   
  
  _unRecipeFunctions_writeInRecipeLog("Creating the recipe '" + sRecipeInstance + "' of class '" + sRecipeClass + "' ...");
  _unRecipeFunctions_createRecipeInstance(sRecipeClass, sRecipeInstance, sRecipeDesc, 
										  exceptionInfo, sInitial, recipeObject[fwConfigurationDB_RO_VALUE]);
  
  if (dynlen(exceptionInfo)) { 
	dpSet(sRecipeDp+".ProcessInput.State", "");
	DebugTN(exceptionInfo); 
	_unRecipeFunctions_writeInRecipeLog("Error: " + exceptionInfo[2]);
	return; 
  }
  _unRecipeFunctions_writeInRecipeLog("The recipe '" + sRecipeInstance + "' of class '" + sRecipeClass + "' has been created.", TRUE);
  _unRecipeFunctions_loadRecipeInstances(sRecipeClass);
  dpSet(sRecipeDp+".ProcessInput.State", "");
}
//------------------------------------------------------------------------------------------------------------------------
/** 
 * Delete a recipe instance.
 * @param sRecipeDp - [IN] Data point name of the recipe instance to delete.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_deleteRecipeInstance(string sRecipeDp, dyn_string &exceptionInfo)
{
  int rc, iPos;
  dyn_dyn_mixed recipeObject;
  dyn_string recipeInstances, ds;
  dyn_float df;
  string sClassName, sInstanceName, sPvssDpe;
  bool bInitial;
  
  // Set the recipe State to "Delete"
  dpSet($sDpName+".ProcessInput.State", UN_RCP_INSTANCE_STATE_DELETE);
  
  if (!shapeExists("RecipeClasses") || !shapeExists("RecipeInstances") || 
	  !shapeExists("SelectionProgressBar") || !shapeExists("PreviousRecipeDp") ) 
  {
	dpSet($sDpName+".ProcessInput.State", "");
	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	return;
  }
  
  _unRecipeFunctions_getRecipeObjectFromDp(sRecipeDp, recipeObject, exceptionInfo);
  if (dynlen(exceptionInfo)) { 
    dpSet($sDpName+".ProcessInput.State", "");
	DebugTN(exceptionInfo);	
	return; 
  }
  
  // Get the inital recipe value of the recipe instance
  bInitial = recipeObject[fwConfigurationDB_RO_META_PREDEFINED][1];
  if (bInitial) {
	_unRecipeFunctions_writeInRecipeLog("Error: The initial recipes can't be deleted.");
	dpSet(sRecipeDp+".ProcessInput.State", "");
	return;
  }
  
  sClassName = recipeObject[fwConfigurationDB_RO_META_CLASSNAME];
  sInstanceName = recipeObject[fwConfigurationDB_RO_META_ORIGNAME];
  if ((iPos=strpos(sInstanceName, "/")) >=0 ) {
	sInstanceName = substr(sInstanceName, iPos+1);
  }
  
  // Asks for user confirmation
  ChildPanelOnCentralReturn("fwGeneral/fwOkCancel.pnl",
							"Delete Recipe",
							makeDynString("$sTitle:Delete recipe ",
										  "$text: Do you want to delete the recipe " + sClassName + "/" + sInstanceName + "?"  ),
							df,
							ds);
  
  if (df[1]<=0)  {
	dpSet(sRecipeDp+".ProcessInput.State", "");
	return;
  }
  
  setValue("LoadedRecipeDp", "text", "");
  unSelectDeselectHMI_select(sRecipeDp, false, exceptionInfo);
  unProgressBar_setPosition("SelectionProgressBar", 0);    
  
  // Delete the recipe instance from the fwConfigurationDB
  fwConfigurationDB_deleteRecipeOfClass(sInstanceName, sClassName, exceptionInfo);
  if (dynlen(exceptionInfo)) { 
	DebugTN(exceptionInfo);	
	dpSet(sRecipeDp+".ProcessInput.State", "");
	return; 
  }
  
  // Get the list of recipe instances of the recipe class
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(sClassName, sPvssDpe, exceptionInfo);
  rc = dpGet(sPvssDpe+"ProcessInput.Recipes",recipeInstances);
  if (rc) {
  	//DebugTN("unRecipeFunctions_deleteRecipeInstance: Error while getting the data of the recipe class '" +sClassName+"'","");
	dpSet(sRecipeDp+".ProcessInput.State", "");
	return;
  }
  
  // Remove the recipe instance of the list of recipes that belong to the recipe class
  if ((iPos=dynContains(recipeInstances, sInstanceName)) > 0) {
	dynRemove(recipeInstances, iPos);
	rc = dpSet(sPvssDpe+"ProcessInput.Recipes", recipeInstances);
	if (rc) {
		//DebugTN("unRecipeFunctions_deleteRecipeInstance: Error while setting the recipe instances of the recipe class '" +sClassName+"'","");
		dpSet(sRecipeDp+".ProcessInput.State", "");
		return;	
	}
  }
  
  // Delete the recipe instance
  rc = dpDelete(sRecipeDp);
  if (rc < 0) {
	//DebugTN ("Error deleting the recipe instance " + sClassName + "/" + sInstanceName);
	dpSet(sRecipeDp+".ProcessInput.State", "");
  }
   
  unImportDevice_triggerUpdate();
  _unRecipeFunctions_writeInRecipeLog("The recipe " + recipeObject[fwConfigurationDB_RO_META_ORIGNAME] + " has been deleted.", TRUE); 
  _unRecipeFunctions_clearInstanceTables();
  _unRecipeFunctions_loadRecipeInstances(sClassName);
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Delete a recipe class and all its recipe instances.
 * Note: The recipe class and its instances will be deleted ONLY if there are no initial recipes of the class.
 * @param sRcpClassDp - [IN] Data point name of the recipe instance to delete.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_deleteRecipeClass(string sRcpClassDp, dyn_string &exceptionInfo) {
  
  int i, iLen, iRetVal;
  bool bDeleteRecipes = TRUE;
  string sClassName, sInitialRcpDp, sPcoLink;
  dyn_string dsRcpInstances;
  
  unImportDevice_initialize();
  
  // Get the recipe class name
  sClassName = dpGetAlias(sRcpClassDp + ".");
  
  _unRecipeFunctions_writeInRecipeLog("Removing recipe class '" + sClassName + "' and all its instances ...");
  
  // Get the initial recipe of the recipe class
  iRetVal = _unRecipeFunctions_getInitialRecipeOfClass(sClassName, sInitialRcpDp, exceptionInfo);
  if (iRetVal != RCP_INITIAL_NO_DEFINED) {
	fwException_raise(exceptionInfo, "ERROR", "The recipe class has some initial recipes defined and it can not be deleted.","");
	_unRecipeFunctions_writeInRecipeLog("Error: The recipe class '" + sClassName + "' has some initial recipes defined and it can not be deleted.");
	return;
  }
  
  dynClear(exceptionInfo);
  
  // Get the name of all the recipe instances of the recipe class
  dpGet(sRcpClassDp + ".ProcessInput.Recipes", dsRcpInstances);
  
  // Delete all the recipe instances 
  iLen = dynlen(dsRcpInstances);
  for (i=1; i<=iLen; i++) {
    string sRcpInstanceName, sPvssDpe;
	sRcpInstanceName = sClassName + "/" + dsRcpInstances[i];
	unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(sRcpInstanceName, sPvssDpe);  
	dpDelete(sPvssDpe);
  }
  
  // Delete the recipe class
  dpDelete(sRcpClassDp);
  
  // Delete the predefined recipe instance if it exists
  _unRecipeFunctions_deletePredefinedRecipeInstance(sClassName);
  
  // Delete the JCOP recipe class and its instances
  fwConfigurationDB_deleteRecipeClass(sClassName, bDeleteRecipes, exceptionInfo);
  if (dynlen(exceptionInfo)) {
    _unRecipeFunctions_writeInRecipeLog("Error deleting the recipe class and its instances: " + exceptionInfo[2]);
	return;
  }
  
  unImportDevice_triggerUpdate();
  
  getValue("PcoList", "text", sPcoLink);
  unRecipeFunctions_loadRecipeClasses(sPcoLink, exceptionInfo);
  
  _unRecipeFunctions_writeInRecipeLog("The recipe class '" + sClassName + "' and all its instances have been removed.", TRUE);
}

//------------------------------------------------------------------------------------------------------------------------
/**
 * Creates a dp name that is not existing already
 * @param sDeviceType - [IN] Recipe device type name.
 * @param sFrontEndName - [IN] Front end name where the recipe will be linked.
 * @param sAppName - [IN] Application name.
 * @param sDpName - [OUT] Data point name for the recipe instance.
 * @param iDeviceNumber - [OUT] Device number for the recipe.
 */
void _unRecipeFunctions_createNewRecipeDp(string sDeviceType, string sFrontEndName, string sAppName, string &sDpName, int &iDeviceNumber)
{
  string sRandomNumber, sTmp, sPrefix;
  
  dpGet("_unApplication.devicePrefix", sPrefix);
  sDpName = sPrefix + "-" + sFrontEndName + "-" + sAppName + "-" + sDeviceType + "-";
  
  do {
	  sRandomNumber="";
	  for (int i=1; i<=5; i++) {
		sTmp = rand();
		sRandomNumber += sTmp[strlen(sTmp)-1];    
	  }
  } while (dpExists(sDpName+sRandomNumber));
    
  sDpName += sRandomNumber;
  iDeviceNumber = (int) sRandomNumber;
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Get the data point name of the recipe instance selected in the panel.
 * @param sSelected - [OUT] DP name of the recipe instance selected in the panel.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_getSelectedRecipeDp(string &sSelected, dyn_string &exceptionInfo)
{
  sSelected = "";
  string sRecipeClass;
  int iLineCount;
  dyn_int diLines;
  
  if (!shapeExists("RecipeClasses") || !shapeExists("RecipeInstances")) {
  	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	return;
  }
  
  getValue("RecipeClasses", "text", sRecipeClass);
  if (sRecipeClass=="")	return; 
  
  getMultiValue("RecipeInstances", "lineCount", iLineCount,
				"RecipeInstances", "getSelectedLines", diLines);
				
  if (!dynlen(diLines) || (diLines[1] >= iLineCount))  return;

  getValue("RecipeInstances", "cellValueRC", diLines[1], "RecipeInstancesDP", sSelected);
}

//--------------------------------------------------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//		Methods to edit the color on a table 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/** 
 * Set the background color of a column in a table.
 * @param tableName  - [IN] The name of the table.
 * @param columnName - [IN] The name of the column.
 * @param color		 - [IN] The new color for the column.
 */
void _unRecipeFunctions_setTableColumnBGColor(string tableName, string columnName, string color)
{
  int lines;
  int rc=getValue(tableName, "lineCount", lines);
  if(rc)  return;

  for (int i=0; i<lines; i++) {
    setValue(tableName, "cellBackColRC", i, columnName, color);
  }  
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Set the background color of a row in a table.
 * @param tableName - [IN] The name of the table.
 * @param rowIndex  - [IN] The row index.
 * @param color		- [IN] The new color for the row.
 */
void _unRecipeFunctions_setTableRowBGColor(string tableName, int rowIndex, string color)
{
  int cols; 
  int rc=getValue(tableName, "columnCount", cols);
  if(rc)  return;
  
  for (int i=0; i<cols; i++) {
	string colName;
    getValue(tableName, "columnName", i, colName);
    setValue(tableName, "cellBackColRC", rowIndex, colName, color);
  }  
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Set the background color of a cell in a table.
 * @param tableName  - [IN] The name of the table.
 * @param rowIndex   - [IN] The row index.
 * @param columnName - [IN] The name of the column.
 * @param color		 - [IN] The new color for the row.
 */
void _unRecipeFunctions_setTableCellBGColor(string tableName, int rowIndex, string columnName, string color)
{
  int lines;
  int rc=getValue(tableName, "lineCount", lines);
  if(rc)  return;

  setValue(tableName, "cellBackColRC", rowIndex, columnName, color);
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Restore the original colors in the recipe elements table (recipe instance panel).
 */
void _unRecipeFunctions_resetRecipeElementsBgColor() {
  int lines;

  getValue("RecipeElements", "lineCount", lines);
  for (int i=1; i<=lines; i++){
    _unRecipeFunctions_setTableRowBGColor("RecipeElements", (i-1), "{255,255,255}");
	// Resets the font style
	RecipeElements.rowFontType (i-1, makeDynInt(0));
  }
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Set the background color of the rows in the RecipeElements table that contains invalid values.
 * @param rows - [IN] Indices of the invalid rows.
 */
void _unRecipeFunctions_setInvalidRows(dyn_string rows)
{
  int len=dynlen(rows);
  for (int i=1; i<=len; i++) {
    int row = rows[i];
    _unRecipeFunctions_setTableRowBGColor("RecipeElements", (i-1), "{255,0,0}"); 
  }
}

//------------------------------------------------------------------------------------------------------------------------
/** 
 * Method used to trim a label to the appropriate size.
 * @param hiddenShape - [IN] Hidden shape (label) to calculate the width of the string.
 * @param aString 	  - [IN] String to be trimmed if its width is bigger than the size specified.
 * @param width		  - [IN] Width of the shape where the label will be placed.
 * @return A new string that fits in the specified width.
 */
string _unRecipeFunctions_adjustStringToSize(shape hiddenShape, string aString, int width) {
	int x, y;
	string sTmpStr = aString;
	bool bCalculateNewSize, bResized = false;

	do {
		setValue(hiddenShape, "text", sTmpStr);
		getValue(hiddenShape, "size", x, y);
		bCalculateNewSize = (x>width-8);
		if (bCalculateNewSize) {
			sTmpStr = substr(sTmpStr, 0, strlen(sTmpStr)-1);
			bResized = true;
		}
	} while (bCalculateNewSize);
	
	if (bResized) {
	  sTmpStr = sTmpStr + "..";
	}
	
	return sTmpStr;
}
//------------------------------------------------------------------------------------------------------------------------
